@@ -34,15 +34,16 @@
 #include <unistd.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
+#include <gio/gio.h>
 #include <gdk/gdkkeysyms.h>
+#include <libebackend/e-extensible.h>
 #include <e-util/e-util.h>
 #include <e-util/e-alert-sink.h>
 #include <e-util/e-dialog-utils.h>
-#include <e-util/e-extensible.h>
 #include <e-util/e-util-private.h>
-#include <e-util/gconf-bridge.h>
 #include <shell/e-shell.h>
 
+#include <libedataserver/e-data-server-util.h>
 #include <libecal/e-cal-client.h>
 #include <libecal/e-cal-client-view.h>
 
@@ -70,6 +71,8 @@ struct _CompEditorPrivate {
 
 	gpointer shell;  /* weak pointer */
 
+	GSettings *calendar_settings;
+
 	/* EFocusTracker keeps selection actions up-to-date. */
 	EFocusTracker *focus_tracker;
 
@@ -300,7 +303,7 @@ get_attachment_list (CompEditor *editor)
 	GSList *list = NULL;
 	const gchar *comp_uid = NULL;
 	const gchar *local_store;
-	gchar *path;
+	gchar *filename_prefix, *tmp;
 	gint ii;
 
 	struct {
@@ -309,6 +312,9 @@ get_attachment_list (CompEditor *editor)
 		GtkWindow *parent;
 	} status;
 
+	e_cal_component_get_uid (editor->priv->comp, &comp_uid);
+	g_return_val_if_fail (comp_uid != NULL, NULL);
+
 	status.uris = NULL;
 	status.done = FALSE;
 	status.parent = g_object_ref (editor);
@@ -316,17 +322,20 @@ get_attachment_list (CompEditor *editor)
 	view = E_ATTACHMENT_VIEW (editor->priv->attachment_view);
 	store = e_attachment_view_get_store (view);
 
+	tmp = g_strdup (comp_uid);
+	e_filename_make_safe (tmp);
+	filename_prefix = g_strconcat (tmp, "-", NULL);
+	g_free (tmp);
+
 	local_store = e_cal_client_get_local_attachment_store (editor->priv->cal_client);
-	e_cal_component_get_uid (editor->priv->comp, &comp_uid);
-	path = g_build_path ("/", local_store, comp_uid, NULL);
-	destination = g_file_new_for_path (path);
-	g_free (path);
+	destination = g_file_new_for_path (local_store);
 
 	e_attachment_store_save_async (
-		store, destination, (GAsyncReadyCallback)
-		attachment_save_finished, &status);
+		store, destination, filename_prefix,
+		(GAsyncReadyCallback) attachment_save_finished, &status);
 
 	g_object_unref (destination);
+	g_free (filename_prefix);
 
 	/* We can't return until we have results, so crank
 	 * the main loop until the callback gets triggered. */
@@ -951,6 +960,18 @@ action_save_cb (GtkAction *action,
 		return;
 	}
 
+	if ((comp_editor_get_flags (editor) & COMP_EDITOR_IS_ASSIGNED) != 0
+	    && e_cal_component_get_vtype (priv->comp) == E_CAL_COMPONENT_TODO
+	    && e_client_check_capability (E_CLIENT (priv->cal_client), CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT)) {
+		e_alert_submit (
+			E_ALERT_SINK (editor),
+			"calendar:prompt-no-task-assignment-editor",
+			e_source_peek_name (
+				e_client_get_source (E_CLIENT (priv->cal_client))),
+			NULL);
+		return;
+	}
+
 	commit_all_fields (editor);
 	if (e_cal_component_has_recurrences (priv->comp)) {
 		if (!recur_component_dialog (
@@ -1586,6 +1607,8 @@ comp_editor_dispose (GObject *object)
 		g_signal_handlers_disconnect_matched (
 			G_OBJECT (store), G_SIGNAL_MATCH_DATA,
 			0, 0, NULL, NULL, object);
+		g_object_unref (priv->attachment_view);
+		priv->attachment_view = NULL;
 	}
 
 	/* We want to destroy the pages after the widgets get destroyed,
@@ -1616,6 +1639,7 @@ comp_editor_finalize (GObject *object)
 
 	priv = COMP_EDITOR (object)->priv;
 
+	g_object_unref (priv->calendar_settings);
 	g_free (priv->summary);
 
 	/* Chain up to parent's finalize() method. */
@@ -1632,39 +1656,35 @@ comp_editor_constructed (GObject *object)
 }
 
 static void
-comp_editor_bind_gconf (CompEditor *editor)
+comp_editor_bind_settings (CompEditor *editor)
 {
-	GConfBridge *bridge;
 	GtkAction *action;
-	const gchar *key;
 
 	g_return_if_fail (editor != NULL);
 
-	bridge = gconf_bridge_get ();
-
-	key = "/apps/evolution/calendar/display/show_categories";
 	action = comp_editor_get_action (editor, "view-categories");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-categories",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 
-	key = "/apps/evolution/calendar/display/show_role";
 	action = comp_editor_get_action (editor, "view-role");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-role",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 
-	key = "/apps/evolution/calendar/display/show_rsvp";
 	action = comp_editor_get_action (editor, "view-rsvp");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-rsvp",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 
-	key = "/apps/evolution/calendar/display/show_status";
 	action = comp_editor_get_action (editor, "view-status");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-status",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 
-	key = "/apps/evolution/calendar/display/show_timezone";
 	action = comp_editor_get_action (editor, "view-time-zone");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-timezone",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 
-	key = "/apps/evolution/calendar/display/show_type";
 	action = comp_editor_get_action (editor, "view-type");
-	gconf_bridge_bind_property (bridge, key, G_OBJECT (action), "active");
+	g_settings_bind (editor->priv->calendar_settings, "editor-show-type",
+			 G_OBJECT (action), "active", G_SETTINGS_BIND_DEFAULT);
 }
 
 static gboolean
@@ -1960,6 +1980,8 @@ comp_editor_init (CompEditor *editor)
 
 	active_editors = g_list_prepend (active_editors, editor);
 
+	priv->calendar_settings = g_settings_new ("org.gnome.evolution.calendar");
+
 	/* Each editor window gets its own window group. */
 	window = GTK_WINDOW (editor);
 	priv->window_group = gtk_window_group_new ();
@@ -2091,17 +2113,14 @@ comp_editor_init (CompEditor *editor)
 		GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
 
 	widget = e_attachment_paned_new ();
+	e_attachment_paned_set_resize_toplevel (
+		E_ATTACHMENT_PANED (widget), TRUE);
 	gtk_container_set_border_width (GTK_CONTAINER (widget), 6);
 	gtk_box_pack_start (GTK_BOX (container), widget, TRUE, TRUE, 0);
 	priv->attachment_view = g_object_ref (widget);
 	gtk_widget_show (widget);
 
 	if (express_mode) {
-		e_attachment_paned_set_expanded (
-			E_ATTACHMENT_PANED (widget), TRUE);
-		e_attachment_paned_set_expanded (
-			E_ATTACHMENT_PANED (widget), FALSE);
-
 		widget = e_attachment_paned_get_view_combo (
 			E_ATTACHMENT_PANED (widget));
 		gtk_widget_hide (widget);
@@ -2172,9 +2191,10 @@ comp_editor_init (CompEditor *editor)
 		store, "row-inserted",
 		G_CALLBACK (attachment_store_changed_cb), editor);
 
-	comp_editor_bind_gconf (editor);
+	comp_editor_bind_settings (editor);
 
-	e_shell_watch_window (shell, GTK_WINDOW (editor));
+	gtk_application_add_window (
+		GTK_APPLICATION (shell), GTK_WINDOW (editor));
 	e_shell_adapt_window_size (shell, GTK_WINDOW (editor));
 }
 
@@ -2205,6 +2225,18 @@ prompt_and_save_changes (CompEditor *editor,
 			return FALSE;
 		}
 
+		if ((comp_editor_get_flags (editor) & COMP_EDITOR_IS_ASSIGNED) != 0
+		    && e_cal_component_get_vtype (priv->comp) == E_CAL_COMPONENT_TODO
+		    && e_client_check_capability (E_CLIENT (priv->cal_client), CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT)) {
+			e_alert_submit (
+				E_ALERT_SINK (editor),
+				"calendar:prompt-no-task-assignment-editor",
+				e_source_peek_name (
+					e_client_get_source (E_CLIENT (priv->cal_client))),
+				NULL);
+			return FALSE;
+		}
+
 		comp = comp_editor_get_current_comp (editor, &correct);
 		e_cal_component_get_summary (comp, &text);
 		g_object_unref (comp);
@@ -2978,6 +3010,7 @@ attachment_loaded_cb (EAttachment *attachment,
 	GFileInfo *file_info;
 	const gchar *display_name;
 	const gchar *uid;
+	gchar *new_name;
 
 	/* Prior to 2.27.2, attachment files were named:
 	 *
@@ -3008,9 +3041,10 @@ attachment_loaded_cb (EAttachment *attachment,
 	uid = g_object_get_data (G_OBJECT (attachment), "uid");
 
 	if (g_str_has_prefix (display_name, uid)) {
-		g_file_info_set_display_name (
-			file_info, display_name + strlen (uid) + 1);
+		new_name = g_strdup (display_name + strlen (uid) + 1);
+		g_file_info_set_display_name (file_info, new_name);
 		g_object_notify (G_OBJECT (attachment), "file-info");
+		g_free (new_name);
 	}
 
 	e_attachment_load_handle_error (attachment, result, parent);
@@ -3210,6 +3244,7 @@ real_send_comp (CompEditor *editor,
 	g_return_val_if_fail (IS_COMP_EDITOR (editor), FALSE);
 
 	priv = editor->priv;
+
 	flags = comp_editor_get_flags (editor);
 
 	if (priv->mod == CALOBJ_MOD_ALL && e_cal_component_is_instance (priv->comp)) {
