@@ -46,7 +46,6 @@
 #include "e-util/e-account-utils.h"
 #include "e-util/e-categories-config.h"
 #include "e-util/e-dialog-utils.h"
-#include "e-util/e-dialog-widgets.h"
 #include "e-util/e-util-private.h"
 
 #include "../calendar-config.h"
@@ -167,7 +166,7 @@ clear_widgets (MemoPage *mpage)
 	CompEditor *editor;
 
 	/* Summary */
-	e_dialog_editable_set (mpage->priv->summary_entry, NULL);
+	gtk_entry_set_text (GTK_ENTRY (mpage->priv->summary_entry), "");
 
 	/* Description */
 	view = GTK_TEXT_VIEW (mpage->priv->memo_content);
@@ -180,7 +179,7 @@ clear_widgets (MemoPage *mpage)
 	comp_editor_set_classification (editor, E_CAL_COMPONENT_CLASS_PRIVATE);
 
 	/* Categories */
-	e_dialog_editable_set (mpage->priv->categories, NULL);
+	gtk_entry_set_text (GTK_ENTRY (mpage->priv->categories), "");
 }
 
 static void
@@ -279,7 +278,10 @@ memo_page_fill_widgets (CompEditorPage *page,
 
         /* Summary */
 	e_cal_component_get_summary (comp, &text);
-	e_dialog_editable_set (priv->summary_entry, text.value);
+	if (text.value != NULL)
+		gtk_entry_set_text (GTK_ENTRY (priv->summary_entry), text.value);
+	else
+		gtk_entry_set_text (GTK_ENTRY (priv->summary_entry), "");
 
 	e_cal_component_get_description_list (comp, &l);
 	if (l && l->data) {
@@ -312,7 +314,10 @@ memo_page_fill_widgets (CompEditorPage *page,
 
 	/* Categories */
 	e_cal_component_get_categories (comp, &categories);
-	e_dialog_editable_set (priv->categories, categories);
+	if (categories != NULL)
+		gtk_entry_set_text (GTK_ENTRY (priv->categories), categories);
+	else
+		gtk_entry_set_text (GTK_ENTRY (priv->categories), "");
 
 	e_client_get_backend_property_sync (E_CLIENT (client), CAL_BACKEND_PROPERTY_CAL_EMAIL_ADDRESS, &backend_addr, NULL, NULL);
 	set_subscriber_info_string (mpage, backend_addr);
@@ -333,11 +338,19 @@ memo_page_fill_widgets (CompEditorPage *page,
 			if (itip_organizer_is_user (comp, client) || itip_sentby_is_user (comp, client)) {
 				gtk_entry_set_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (priv->org_combo))), string);
 			} else {
-				GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->org_combo));
+				GtkComboBox *combo_box;
+				GtkListStore *list_store;
+				GtkTreeModel *model;
+				GtkTreeIter iter;
 
-				gtk_list_store_clear (GTK_LIST_STORE (model));
-				e_dialog_append_list_store_text (model, 0, string);
-				gtk_combo_box_set_active (GTK_COMBO_BOX (priv->org_combo), 0);
+				combo_box = GTK_COMBO_BOX (priv->org_combo);
+				model = gtk_combo_box_get_model (combo_box);
+				list_store = GTK_LIST_STORE (model);
+
+				gtk_list_store_clear (list_store);
+				gtk_list_store_append (list_store, &iter);
+				gtk_list_store_set (list_store, &iter, 0, string, -1);
+				gtk_combo_box_set_active (combo_box, 0);
 				gtk_editable_set_editable (GTK_EDITABLE (gtk_bin_get_child (GTK_BIN (priv->org_combo))), FALSE);
 			}
 			g_free (string);
@@ -646,8 +659,8 @@ memo_page_fill_component (CompEditorPage *page,
 	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->memo_content));
 
 	/* Summary */
-	str = e_dialog_editable_get (priv->summary_entry);
-	if (!str || strlen (str) == 0)
+	str = gtk_editable_get_chars (GTK_EDITABLE (priv->summary_entry), 0, -1);
+	if (str == NULL || *str == '\0')
 		e_cal_component_set_summary (comp, NULL);
 	else {
 		ECalComponentText text;
@@ -658,10 +671,8 @@ memo_page_fill_component (CompEditorPage *page,
 		e_cal_component_set_summary (comp, &text);
 	}
 
-	if (str) {
-		g_free (str);
-		str = NULL;
-	}
+	g_free (str);
+	str = NULL;
 
 	/* Memo Content */
 
@@ -695,8 +706,7 @@ memo_page_fill_component (CompEditorPage *page,
 		e_cal_component_set_description_list (comp, &l);
 	}
 
-	if (str)
-		g_free (str);
+	g_free (str);
 
 	/* Dates */
 	start_tt = icaltime_null_time ();
@@ -721,15 +731,13 @@ memo_page_fill_component (CompEditorPage *page,
 	e_cal_component_set_classification (comp, classification);
 
 	/* Categories */
-	cat = e_dialog_editable_get (priv->categories);
+	cat = gtk_editable_get_chars (GTK_EDITABLE (priv->categories), 0, -1);
 	str = comp_editor_strip_categories (cat);
-	if (cat)
-		g_free (cat);
+	g_free (cat);
 
 	e_cal_component_set_categories (comp, str);
 
-	if (str)
-		g_free (str);
+	g_free (str);
 
 	/* change recipients only when creating new item, after that no such action is available */
 	if ((flags & COMP_EDITOR_IS_SHARED) && (flags & COMP_EDITOR_NEW_ITEM) && fill_comp_with_recipients (priv->name_selector, comp)) {
@@ -1020,19 +1028,18 @@ set_subscriber_info_string (MemoPage *mpage,
 }
 
 static void
-summary_changed_cb (GtkEditable *editable,
+summary_changed_cb (GtkEntry *entry,
                     CompEditorPage *page)
 {
 	CompEditor *editor;
-	gchar *summary;
+	const gchar *text;
 
 	if (comp_editor_page_get_updating (page))
 		return;
 
 	editor = comp_editor_page_get_editor (page);
-	summary = e_dialog_editable_get (GTK_WIDGET (editable));
-	comp_editor_set_summary (editor, summary);
-	g_free (summary);
+	text = gtk_entry_get_text (entry);
+	comp_editor_set_summary (editor, text);
 }
 
 static void
@@ -1227,18 +1234,28 @@ memo_page_construct (MemoPage *mpage)
 	}
 
 	if (flags & COMP_EDITOR_IS_SHARED) {
+		GtkComboBox *combo_box;
+		GtkListStore *list_store;
 		GtkTreeModel *model;
+		GtkTreeIter iter;
 		gint ii;
 
-		model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->org_combo));
+		combo_box = GTK_COMBO_BOX (priv->org_combo);
+		model = gtk_combo_box_get_model (combo_box);
+		list_store = GTK_LIST_STORE (model);
+
 		priv->address_strings = itip_get_user_identities ();
 		priv->fallback_address = itip_get_fallback_identity ();
 
-		for (ii = 0; priv->address_strings[ii] != NULL; ii++)
-			e_dialog_append_list_store_text (
-				model, 0, priv->address_strings[ii]);
+		/* FIXME Could we just use a GtkComboBoxText? */
+		for (ii = 0; priv->address_strings[ii] != NULL; ii++) {
+			gtk_list_store_append (list_store, &iter);
+			gtk_list_store_set (
+				list_store, &iter,
+				0, priv->address_strings[ii], -1);
+		}
 
-		gtk_combo_box_set_active (GTK_COMBO_BOX (priv->org_combo), 0);
+		gtk_combo_box_set_active (combo_box, 0);
 
 		gtk_widget_show (priv->org_label);
 		gtk_widget_show (priv->org_combo);
