@@ -47,7 +47,6 @@
 
 #include "e-util/e-util.h"
 #include "e-util/e-account-utils.h"
-#include "e-util/e-dialog-widgets.h"
 #include "e-util/e-categories-config.h"
 #include "e-util/e-util-private.h"
 
@@ -201,7 +200,7 @@ clear_widgets (TaskPage *tpage)
 	editor = comp_editor_page_get_editor (COMP_EDITOR_PAGE (tpage));
 
 	/* Summary, description */
-	e_dialog_editable_set (priv->summary, NULL);
+	gtk_entry_set_text (GTK_ENTRY (priv->summary), "");
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->description)), "", 0);
 	e_buffer_tagger_update_tags (GTK_TEXT_VIEW (priv->description));
 
@@ -213,7 +212,7 @@ clear_widgets (TaskPage *tpage)
 	comp_editor_set_classification (editor, E_CAL_COMPONENT_CLASS_PUBLIC);
 
 	/* Categories */
-	e_dialog_editable_set (priv->categories, NULL);
+	gtk_entry_set_text (GTK_ENTRY (priv->categories), "");
 }
 
 static gboolean
@@ -521,7 +520,10 @@ task_page_fill_widgets (CompEditorPage *page,
 
         /* Summary, description(s) */
 	e_cal_component_get_summary (comp, &text);
-	e_dialog_editable_set (priv->summary, text.value);
+	if (text.value != NULL)
+		gtk_entry_set_text (GTK_ENTRY (priv->summary), text.value);
+	else
+		gtk_entry_set_text (GTK_ENTRY (priv->summary), "");
 
 	e_cal_component_get_description_list (comp, &l);
 	if (l && l->data) {
@@ -614,7 +616,10 @@ task_page_fill_widgets (CompEditorPage *page,
 
 	/* Categories */
 	e_cal_component_get_categories (comp, &categories);
-	e_dialog_editable_set (priv->categories, categories);
+	if (categories != NULL)
+		gtk_entry_set_text (GTK_ENTRY (priv->categories), categories);
+	else
+		gtk_entry_set_text (GTK_ENTRY (priv->categories), "");
 
 	/* Source */
 	e_source_combo_box_set_active (
@@ -663,11 +668,19 @@ task_page_fill_widgets (CompEditorPage *page,
 				g_signal_handlers_block_by_func (gtk_bin_get_child (GTK_BIN (priv->organizer)), organizer_changed_cb, tpage);
 
 				if (!priv->user_org) {
-					GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->organizer));
+					GtkComboBox *combo_box;
+					GtkListStore *list_store;
+					GtkTreeModel *model;
+					GtkTreeIter iter;
 
-					gtk_list_store_clear (GTK_LIST_STORE (model));
-					e_dialog_append_list_store_text (model, 0, string);
-					gtk_combo_box_set_active (GTK_COMBO_BOX (priv->organizer), 0);
+					combo_box = GTK_COMBO_BOX (priv->organizer);
+					model = gtk_combo_box_get_model (combo_box);
+					list_store = GTK_LIST_STORE (model);
+
+					gtk_list_store_clear (list_store);
+					gtk_list_store_append (list_store, &iter);
+					gtk_list_store_set (list_store, &iter, 0, string, -1);
+					gtk_combo_box_set_active (combo_box, 0);
 					gtk_editable_set_editable (GTK_EDITABLE (gtk_bin_get_child (GTK_BIN (priv->organizer))), FALSE);
 				} else {
 					gtk_entry_set_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (priv->organizer))), string);
@@ -757,8 +770,8 @@ task_page_fill_component (CompEditorPage *page,
 
 	/* Summary. */
 
-	str = e_dialog_editable_get (priv->summary);
-	if (!str || strlen (str) == 0)
+	str = gtk_editable_get_chars (GTK_EDITABLE (priv->summary), 0, -1);
+	if (str == NULL || *str == '\0')
 		e_cal_component_set_summary (comp, NULL);
 	else {
 		ECalComponentText text;
@@ -769,8 +782,7 @@ task_page_fill_component (CompEditorPage *page,
 		e_cal_component_set_summary (comp, &text);
 	}
 
-	if (str)
-		g_free (str);
+	g_free (str);
 
 	/* Description */
 
@@ -792,8 +804,7 @@ task_page_fill_component (CompEditorPage *page,
 		e_cal_component_set_description_list (comp, &l);
 	}
 
-	if (str)
-		g_free (str);
+	g_free (str);
 
 	/* Dates */
 
@@ -851,10 +862,9 @@ task_page_fill_component (CompEditorPage *page,
 	}
 
 	/* Categories */
-	cat = e_dialog_editable_get (priv->categories);
+	cat = gtk_editable_get_chars (GTK_EDITABLE (priv->categories), 0, -1);
 	str = comp_editor_strip_categories (cat);
-	if (cat)
-		g_free (cat);
+	g_free (cat);
 
 	e_cal_component_set_categories (comp, str);
 
@@ -1515,19 +1525,18 @@ get_widgets (TaskPage *tpage)
 }
 
 static void
-summary_changed_cb (GtkEditable *editable,
+summary_changed_cb (GtkEntry *entry,
                     CompEditorPage *page)
 {
 	CompEditor *editor;
-	gchar *summary;
+	const gchar *text;
 
 	if (comp_editor_page_get_updating (page))
 		return;
 
 	editor = comp_editor_page_get_editor (page);
-	summary = e_dialog_editable_get (GTK_WIDGET (editable));
-	comp_editor_set_summary (editor, summary);
-	g_free (summary);
+	text = gtk_entry_get_text (entry);
+	comp_editor_set_summary (editor, text);
 }
 
 /* Callback used when the start or due date widgets change.  We notify the
@@ -2092,7 +2101,10 @@ task_page_construct (TaskPage *tpage,
                      ECalClient *client)
 {
 	TaskPagePrivate *priv;
+	GtkComboBox *combo_box;
+	GtkListStore *list_store;
 	GtkTreeModel *model;
+	GtkTreeIter iter;
 	gint ii;
 
 	priv = tpage->priv;
@@ -2114,16 +2126,22 @@ task_page_construct (TaskPage *tpage,
 		return NULL;
 	}
 
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->organizer));
+	combo_box = GTK_COMBO_BOX (priv->organizer);
+	model = gtk_combo_box_get_model (combo_box);
+	list_store = GTK_LIST_STORE (model);
 
 	priv->address_strings = itip_get_user_identities ();
 	priv->fallback_address = itip_get_fallback_identity ();
 
-	for (ii = 0; priv->address_strings[ii] != NULL; ii++)
-		e_dialog_append_list_store_text (
-			model, 0, priv->address_strings[ii]);
+	/* FIXME Could we just use a GtkComboBoxText? */
+	for (ii = 0; priv->address_strings[ii] != NULL; ii++) {
+		gtk_list_store_append (list_store, &iter);
+		gtk_list_store_set (
+			list_store, &iter,
+			0, priv->address_strings[ii], -1);
+	}
 
-	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->organizer), 0);
+	gtk_combo_box_set_active (combo_box, 0);
 
 	g_signal_connect (
 		gtk_bin_get_child (GTK_BIN (priv->organizer)), "changed",
