@@ -335,7 +335,7 @@ ivcal_getwidget (EImport *ei,
 		if (!group)
 			group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb));
 		if (first == NULL) {
-			g_datalist_set_data_full(&target->data, "primary-source", g_object_ref(primary), g_object_unref);
+			g_datalist_set_data_full(&target->data, "primary-source", g_object_ref (primary), g_object_unref);
 			g_datalist_set_data(&target->data, "primary-type", GINT_TO_POINTER(import_type_map[i]));
 			first = rb;
 		}
@@ -867,7 +867,8 @@ open_default_source (ICalIntelligentImporter *ici,
 
 	e_import_status (ici->ei, ici->target, _("Opening calendar"), 0);
 
-	e_client_utils_open_new (source, source_type == E_CAL_CLIENT_SOURCE_TYPE_EVENTS ? E_CLIENT_SOURCE_TYPE_EVENTS : E_CLIENT_SOURCE_TYPE_TASKS, FALSE, ici->cancellable,
+	e_client_utils_open_new (
+		source, source_type == E_CAL_CLIENT_SOURCE_TYPE_EVENTS ? E_CLIENT_SOURCE_TYPE_EVENTS : E_CLIENT_SOURCE_TYPE_TASKS, FALSE, ici->cancellable,
 		e_client_utils_authenticate_handler, NULL,
 		default_source_opened_cb, odsd);
 
@@ -1022,13 +1023,13 @@ gnome_calendar_getwidget (EImport *ei,
                           EImportImporter *im)
 {
 	GtkWidget *hbox, *w;
-	GConfClient *gconf;
+	GSettings *settings;
 	gboolean done_cal, done_tasks;
 
-	gconf = gconf_client_get_default ();
-	done_cal = gconf_client_get_bool (gconf, "/apps/evolution/importer/gnome-calendar/calendar", NULL);
-	done_tasks = gconf_client_get_bool (gconf, "/apps/evolution/importer/gnome-calendar/tasks", NULL);
-	g_object_unref (gconf);
+	settings = g_settings_new ("org.gnome.evolution.importer");
+	done_cal = g_settings_get_boolean (settings, "gnome-calendar-done-calendar");
+	done_tasks = g_settings_get_boolean (settings, "gnome-calendar-done-tasks");
+	g_object_unref (settings);
 
 	g_datalist_set_data(&target->data, "gnomecal-do-cal", GINT_TO_POINTER(!done_cal));
 	g_datalist_set_data(&target->data, "gnomecal-do-tasks", GINT_TO_POINTER(!done_tasks));
@@ -1379,11 +1380,11 @@ get_users_timezone (void)
 	if (e_shell_settings_get_boolean (shell_settings, "cal-use-system-timezone")) {
 		location = e_cal_util_get_system_timezone_location ();
 	} else {
-		GConfClient *client = gconf_client_get_default ();
+		GSettings *settings = g_settings_new ("org.gnome.evolution.calendar");
 
-		location = gconf_client_get_string (client, CALENDAR_CONFIG_TIMEZONE, NULL);
+		location = g_settings_get_string (settings, "timezone");
 
-		g_object_unref (client);
+		g_object_unref (settings);
 	}
 
 	if (location) {
