@@ -36,7 +36,15 @@
 
 #define d(x)
 
-typedef GtkWidget * (*EConfigItemSectionFactoryFunc)(EConfig *ec, EConfigItem *, GtkWidget *parent, GtkWidget *old, gpointer data, GtkWidget **real_frame);
+typedef GtkWidget *
+		(*EConfigItemSectionFactoryFunc)
+						(EConfig *ec,
+						 EConfigItem *item,
+						 GtkWidget *parent,
+						 GtkWidget *old,
+						 gint position,
+						 gpointer data,
+						 GtkWidget **real_frame);
 
 struct _EConfigFactory {
 	gchar *id;
@@ -47,8 +55,6 @@ struct _EConfigFactory {
 struct _menu_node {
 	GSList *menu;
 	EConfigItemsFunc free;
-	EConfigItemsFunc abort;
-	EConfigItemsFunc commit;
 	gpointer data;
 };
 
@@ -83,7 +89,22 @@ struct _EConfigPrivate {
 	GList *finish_pages;
 };
 
-static GtkWidget *ech_config_section_factory (EConfig *config, EConfigItem *item, GtkWidget *parent, GtkWidget *old, gpointer data, GtkWidget **real_frame);
+static GtkWidget *
+		ech_config_section_factory	(EConfig *config,
+						 EConfigItem *item,
+						 GtkWidget *parent,
+						 GtkWidget *old,
+						 gint position,
+						 gpointer data,
+						 GtkWidget **real_frame);
+
+enum {
+	ABORT,
+	COMMIT,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
 
 G_DEFINE_TYPE (
 	EConfig,
@@ -117,11 +138,11 @@ config_finalize (GObject *object)
 	while (link != NULL) {
 		struct _widget_node *node = link->data;
 
-		/* disconnect the gtk_widget_destroyed function from the widget */
+		/* disconnect the ec_widget_destroyed function from the widget */
 		if (node->widget)
 			g_signal_handlers_disconnect_matched (
 				node->widget, G_SIGNAL_MATCH_DATA,
-				0, 0, NULL, NULL, &node->widget);
+				0, 0, NULL, NULL, node);
 
 		g_free (node);
 
@@ -182,6 +203,24 @@ e_config_class_init (EConfigClass *class)
 
 	class->set_target = config_set_target;
 	class->target_free = config_target_free;
+
+	signals[ABORT] = g_signal_new (
+		"abort",
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (EConfigClass, abort),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
+
+	signals[COMMIT] = g_signal_new (
+		"commit",
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (EConfigClass, commit),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 }
 
 static void
@@ -203,7 +242,9 @@ e_config_init (EConfig *config)
  * Return value: @ep is returned.
  **/
 EConfig *
-e_config_construct (EConfig *ep, gint type, const gchar *id)
+e_config_construct (EConfig *ep,
+                    gint type,
+                    const gchar *id)
 {
 	g_return_val_if_fail (type == E_CONFIG_BOOK || type == E_CONFIG_ASSISTANT, NULL);
 
@@ -218,10 +259,6 @@ e_config_construct (EConfig *ep, gint type, const gchar *id)
  * @ec: An initialised implementing instance of EConfig.
  * @items: A list of EConfigItem's to add to the configuration manager
  * @ec.
- * @commitfunc: If supplied, called to commit the configuration items
- * to persistent storage.
- * @abortfunc: If supplied, called to abort/undo the storage of these
- * items permanently.
  * @freefunc: If supplied, called to free the item list (and/or items)
  * once they are no longer needed.
  * @data: Data for the callback methods.
@@ -232,14 +269,15 @@ e_config_construct (EConfig *ep, gint type, const gchar *id)
  * TODO: perhaps commit and abort should just be signals.
  **/
 void
-e_config_add_items (EConfig *ec, GSList *items, EConfigItemsFunc commitfunc, EConfigItemsFunc abortfunc, EConfigItemsFunc freefunc, gpointer data)
+e_config_add_items (EConfig *ec,
+                    GSList *items,
+                    EConfigItemsFunc freefunc,
+                    gpointer data)
 {
 	struct _menu_node *node;
 
 	node = g_malloc (sizeof (*node));
 	node->menu = items;
-	node->commit = commitfunc;
-	node->abort = abortfunc;
 	node->free = freefunc;
 	node->data = data;
 
@@ -265,7 +303,10 @@ e_config_add_items (EConfig *ec, GSList *items, EConfigItemsFunc commitfunc, ECo
  * the assistant to continue or the notebook to close.
  **/
 void
-e_config_add_page_check (EConfig *ec, const gchar *pageid, EConfigCheckFunc check, gpointer data)
+e_config_add_page_check (EConfig *ec,
+                         const gchar *pageid,
+                         EConfigCheckFunc check,
+                         gpointer data)
 {
 	struct _check_node *cn;
 
@@ -278,7 +319,8 @@ e_config_add_page_check (EConfig *ec, const gchar *pageid, EConfigCheckFunc chec
 }
 
 static struct _finish_page_node *
-find_page_finish (EConfig *config, const gchar *pageid)
+find_page_finish (EConfig *config,
+                  const gchar *pageid)
 {
 	GList *link;
 
@@ -305,7 +347,9 @@ find_page_finish (EConfig *config, const gchar *pageid)
  * With is_finish set on the pageid the page is treated as the last page in an assistant.
  **/
 void
-e_config_set_page_is_finish (EConfig *ec, const gchar *pageid, gboolean is_finish)
+e_config_set_page_is_finish (EConfig *ec,
+                             const gchar *pageid,
+                             gboolean is_finish)
 {
 	struct _finish_page_node *fp;
 
@@ -342,7 +386,8 @@ ec_add_static_items (EConfig *config)
 }
 
 static gint
-ep_cmp (gconstpointer ap, gconstpointer bp)
+ep_cmp (gconstpointer ap,
+        gconstpointer bp)
 {
 	struct _widget_node *a = *((gpointer *) ap);
 	struct _widget_node *b = *((gpointer *) bp);
@@ -351,7 +396,9 @@ ep_cmp (gconstpointer ap, gconstpointer bp)
 }
 
 static GList *
-ec_assistant_find_page (EConfig *ec, GtkWidget *page, gint *page_index)
+ec_assistant_find_page (EConfig *ec,
+                        GtkWidget *page,
+                        gint *page_index)
 {
 	struct _widget_node *node = NULL;
 	GList *link;
@@ -457,7 +504,8 @@ ec_assistant_check_current (EConfig *ec)
 }
 
 static gint
-ec_assistant_forward (gint current_page, gpointer user_data)
+ec_assistant_forward (gint current_page,
+                      gpointer user_data)
 {
 	GtkAssistant *assistant;
 	EConfig *ec = user_data;
@@ -511,6 +559,16 @@ ec_assistant_forward (gint current_page, gpointer user_data)
 	return next_page;
 }
 
+static void
+ec_widget_destroyed (GtkWidget *widget,
+                     struct _widget_node *node)
+{
+	/* Use our own function instead of gtk_widget_destroyed()
+	 * so it's easier to trap EConfig widgets in a debugger. */
+
+	node->widget = NULL;
+}
+
 static void
 ec_rebuild (EConfig *emp)
 {
@@ -531,10 +589,10 @@ ec_rebuild (EConfig *emp)
 	 * duplication */
 
 	/* because rebuild destroys pages, and destroying active page causes crashes */
-	is_assistant = emp->widget && GTK_IS_ASSISTANT (emp->widget);
+	is_assistant = GTK_IS_ASSISTANT (emp->widget);
 	if (is_assistant) {
 		GtkAssistant *assistant;
-		gint page_index = gtk_assistant_get_current_page (GTK_ASSISTANT (emp->widget));
+		gint page_index;
 
 		assistant = GTK_ASSISTANT (emp->widget);
 		page_index = gtk_assistant_get_current_page (assistant);
@@ -609,10 +667,10 @@ ec_rebuild (EConfig *emp)
 		case E_CONFIG_BOOK:
 		case E_CONFIG_ASSISTANT:
 			/* Only one of BOOK or ASSISTANT may be define, it
-			   is used by the defining code to mark the
-			   type of the config window.  It is
-			   cross-checked with the code's defined
-			   type. */
+			 * is used by the defining code to mark the
+			 * type of the config window.  It is
+			 * cross-checked with the code's defined
+			 * type. */
 			if (root != NULL) {
 				g_warning("EConfig book/assistant redefined at: %s", item->path);
 				break;
@@ -624,7 +682,9 @@ ec_rebuild (EConfig *emp)
 					break;
 				}
 				if (item->factory) {
-					root = item->factory (emp, item, NULL, wn->widget, wn->context->data);
+					root = item->factory (
+						emp, item, NULL, wn->widget,
+						0, wn->context->data);
 				} else if (item->type == E_CONFIG_BOOK) {
 					root = gtk_notebook_new ();
 					gtk_widget_show (root);
@@ -685,7 +745,9 @@ ec_rebuild (EConfig *emp)
 
 			if (wn->widget == NULL) {
 				if (item->factory) {
-					page = item->factory (emp, item, root, wn->frame, wn->context->data);
+					page = item->factory (
+						emp, item, root, wn->frame,
+						pageno, wn->context->data);
 				} else {
 					page = gtk_vbox_new (FALSE, 0);
 					gtk_container_set_border_width (GTK_CONTAINER (page), 12);
@@ -736,9 +798,9 @@ ec_rebuild (EConfig *emp)
 		case E_CONFIG_PAGE:
 		case E_CONFIG_PAGE_PROGRESS:
 			/* CONFIG_PAGEs depend on the config type.
-			   E_CONFIG_BOOK:
+			 * E_CONFIG_BOOK:
 				The page is a VBox, stored in the notebook.
-			   E_CONFIG_ASSISTANT
+			 * E_CONFIG_ASSISTANT
 				The page is a VBox, stored in the GtkAssistant,
 				any sections automatically added inside it. */
 			sectionno = 0;
@@ -753,7 +815,9 @@ ec_rebuild (EConfig *emp)
 			}
 
 			if (item->factory) {
-				page = item->factory (emp, item, root, wn->frame, wn->context->data);
+				page = item->factory (
+					emp, item, root, wn->frame,
+					pageno, wn->context->data);
 				if (emp->type == E_CONFIG_ASSISTANT) {
 					wn->frame = page;
 				} else {
@@ -775,6 +839,7 @@ ec_rebuild (EConfig *emp)
 					} else {
 						gtk_assistant_prepend_page (GTK_ASSISTANT (assistant), page);
 					}
+
 					gtk_assistant_set_page_type (GTK_ASSISTANT (assistant), page, item->type == E_CONFIG_PAGE ? GTK_ASSISTANT_PAGE_CONTENT : GTK_ASSISTANT_PAGE_PROGRESS);
 					gtk_assistant_set_page_title (GTK_ASSISTANT (assistant), page, translated_label);
 					gtk_widget_show_all (page);
@@ -806,12 +871,14 @@ ec_rebuild (EConfig *emp)
 			sectionnode = NULL;
 			wn->widget = page;
 			if (page)
-				g_signal_connect(page, "destroy", G_CALLBACK(gtk_widget_destroyed), &wn->widget);
+				g_signal_connect (
+					page, "destroy",
+					G_CALLBACK (ec_widget_destroyed), wn);
 			break;
 		case E_CONFIG_SECTION:
 		case E_CONFIG_SECTION_TABLE:
 			/* The section factory is always called with
-			   the parent vbox object.  Even for assistant pages. */
+			 * the parent vbox object.  Even for assistant pages. */
 			if (page == NULL) {
 				/*g_warning("EConfig section '%s' has no parent page", item->path);*/
 				section = NULL;
@@ -832,7 +899,9 @@ ec_rebuild (EConfig *emp)
 				 */
 				EConfigItemSectionFactoryFunc factory = (EConfigItemSectionFactoryFunc) item->factory;
 
-				section = factory (emp, item, page, wn->widget, wn->context->data, &wn->real_frame);
+				section = factory (
+					emp, item, page, wn->widget, 0,
+					wn->context->data, &wn->real_frame);
 				wn->frame = section;
 				if (section)
 					itemno = 1;
@@ -906,7 +975,9 @@ ec_rebuild (EConfig *emp)
 			sectionno++;
 			wn->widget = section;
 			if (section)
-				g_signal_connect(section, "destroy", G_CALLBACK(gtk_widget_destroyed), &wn->widget);
+				g_signal_connect (
+					section, "destroy",
+					G_CALLBACK (ec_widget_destroyed), wn);
 			sectionnode = wn;
 			break;
 		case E_CONFIG_ITEM:
@@ -916,8 +987,8 @@ ec_rebuild (EConfig *emp)
 				wn->widget = NULL;
 
 			/* ITEMs are called with the section parent.
-			   The type depends on the section type,
-			   either a GtkTable, or a GtkVBox */
+			 * The type depends on the section type,
+			 * either a GtkTable, or a GtkVBox */
 			w = NULL;
 			if (section == NULL) {
 				wn->widget = NULL;
@@ -927,7 +998,9 @@ ec_rebuild (EConfig *emp)
 				 || (item->type == E_CONFIG_ITEM_TABLE && !GTK_IS_TABLE (section)))
 				g_warning("EConfig item parent type is incorrect: %s", item->path);
 			else if (item->factory)
-				w = item->factory (emp, item, section, wn->widget, wn->context->data);
+				w = item->factory (
+					emp, item, section, wn->widget,
+					0, wn->context->data);
 
 			d(printf("item %d:%s widget %p\n", itemno, item->path, w));
 
@@ -943,7 +1016,9 @@ ec_rebuild (EConfig *emp)
 
 			wn->widget = w;
 			if (w) {
-				g_signal_connect(w, "destroy", G_CALLBACK(gtk_widget_destroyed), &wn->widget);
+				g_signal_connect (
+					w, "destroy",
+					G_CALLBACK (ec_widget_destroyed), wn);
 				itemno++;
 
 				if (gtk_widget_get_visible (w))
@@ -1016,14 +1091,16 @@ ec_rebuild (EConfig *emp)
  * initiate a e_config_target_changed() call where appropriate.
  **/
 void
-e_config_set_target (EConfig *emp, EConfigTarget *target)
+e_config_set_target (EConfig *emp,
+                     EConfigTarget *target)
 {
 	if (emp->target != target)
 		((EConfigClass *) G_OBJECT_GET_CLASS (emp))->set_target (emp, target);
 }
 
 static void
-ec_widget_destroy (GtkWidget *w, EConfig *ec)
+ec_widget_destroy (GtkWidget *w,
+                   EConfig *ec)
 {
 	if (ec->target) {
 		e_config_target_free (ec, ec->target);
@@ -1057,7 +1134,6 @@ e_config_create_widget (EConfig *emp)
 	GPtrArray *items = g_ptr_array_new ();
 	GList *link;
 	GSList *l;
-	/*char *domain = NULL;*/
 	gint i;
 
 	g_return_val_if_fail (emp->target != NULL, NULL);
@@ -1069,7 +1145,7 @@ e_config_create_widget (EConfig *emp)
 	while (link != NULL) {
 		struct _menu_node *mnode = link->data;
 
-		for (l=mnode->menu; l; l = l->next) {
+		for (l = mnode->menu; l; l = l->next) {
 			struct _EConfigItem *item = l->data;
 			struct _widget_node *wn = g_malloc0 (sizeof (*wn));
 
@@ -1084,28 +1160,29 @@ e_config_create_widget (EConfig *emp)
 
 	qsort (items->pdata, items->len, sizeof (items->pdata[0]), ep_cmp);
 
-	for (i=0;i<items->len;i++)
+	for (i = 0; i < items->len; i++)
 		p->widgets = g_list_append (p->widgets, items->pdata[i]);
 
 	g_ptr_array_free (items, TRUE);
 	ec_rebuild (emp);
 
 	/* auto-unref it */
-	g_signal_connect(emp->widget, "destroy", G_CALLBACK(ec_widget_destroy), emp);
+	g_signal_connect (emp->widget, "destroy", G_CALLBACK(ec_widget_destroy), emp);
 
 	/* FIXME: for some reason ec_rebuild puts the widget on page 1, this is just to override that */
 	if (emp->type == E_CONFIG_BOOK)
 		gtk_notebook_set_current_page ((GtkNotebook *) emp->widget, 0);
 	else {
 		gtk_window_set_position (GTK_WINDOW (emp->widget), GTK_WIN_POS_CENTER);
-		gtk_widget_show (emp->widget);
 	}
 
 	return emp->widget;
 }
 
 static void
-ec_dialog_response (GtkWidget *d, gint id, EConfig *ec)
+ec_dialog_response (GtkWidget *d,
+                    gint id,
+                    EConfig *ec)
 {
 	if (id == GTK_RESPONSE_OK)
 		e_config_commit (ec);
@@ -1133,7 +1210,9 @@ ec_dialog_response (GtkWidget *d, gint id, EConfig *ec)
  * Return value: The window widget.  This is also stored in @emp.window.
  **/
 GtkWidget *
-e_config_create_window (EConfig *emp, GtkWindow *parent, const gchar *title)
+e_config_create_window (EConfig *emp,
+                        GtkWindow *parent,
+                        const gchar *title)
 {
 	GtkWidget *window;
 
@@ -1190,7 +1269,7 @@ ec_call_page_check (EConfig *emp)
 static gboolean
 ec_idle_handler_for_rebuild (gpointer data)
 {
-	EConfig *emp = (EConfig*) data;
+	EConfig *emp = (EConfig *) data;
 
 	ec_rebuild (emp);
 	ec_call_page_check (emp);
@@ -1212,7 +1291,8 @@ ec_idle_handler_for_rebuild (gpointer data)
  * button for the Notebook mode.
  **/
 void
-e_config_target_changed (EConfig *emp, e_config_target_change_t how)
+e_config_target_changed (EConfig *emp,
+                         e_config_target_change_t how)
 {
 	if (how == E_CONFIG_TARGET_CHANGED_REBUILD) {
 		g_idle_add (ec_idle_handler_for_rebuild, emp);
@@ -1234,22 +1314,9 @@ e_config_target_changed (EConfig *emp, e_config_target_change_t how)
 void
 e_config_abort (EConfig *config)
 {
-	GList *link;
-
 	g_return_if_fail (E_IS_CONFIG (config));
 
-	/* TODO: should these just be signals? */
-
-	link = config->priv->menus;
-
-	while (link != NULL) {
-		struct _menu_node *node = link->data;
-
-		if (node->abort != NULL)
-			node->abort (config, node->menu, node->data);
-
-		link = g_list_next (link);
-	}
+	g_signal_emit (config, signals[ABORT], 0);
 }
 
 /**
@@ -1263,22 +1330,9 @@ e_config_abort (EConfig *config)
 void
 e_config_commit (EConfig *config)
 {
-	GList *link;
-
 	g_return_if_fail (E_IS_CONFIG (config));
 
-	/* TODO: should these just be signals? */
-
-	link = config->priv->menus;
-
-	while (link != NULL) {
-		struct _menu_node *node = link->data;
-
-		if (node->commit != NULL)
-			node->commit (config, node->menu, node->data);
-
-		link = g_list_next (link);
-	}
+	g_signal_emit (config, signals[COMMIT], 0);
 }
 
 /**
@@ -1292,7 +1346,8 @@ e_config_commit (EConfig *config)
  * Return value: FALSE if the data is inconsistent/incomplete.
  **/
 gboolean
-e_config_page_check (EConfig *config, const gchar *pageid)
+e_config_page_check (EConfig *config,
+                     const gchar *pageid)
 {
 	GList *link;
 
@@ -1325,7 +1380,8 @@ e_config_page_check (EConfig *config, const gchar *pageid)
  * container or the GtkVBox object inside the assistant.
  **/
 GtkWidget *
-e_config_page_get (EConfig *ec, const gchar *pageid)
+e_config_page_get (EConfig *ec,
+                   const gchar *pageid)
 {
 	GList *link;
 
@@ -1360,13 +1416,14 @@ e_config_page_get (EConfig *ec, const gchar *pageid)
  * last configured and visible page.
  **/
 const gchar *
-e_config_page_next (EConfig *ec, const gchar *pageid)
+e_config_page_next (EConfig *ec,
+                    const gchar *pageid)
 {
 	GList *link;
 	gint found;
 
 	link = g_list_first (ec->priv->widgets);
-	found = pageid == NULL ? 1:0;
+	found = pageid == NULL ? 1 : 0;
 
 	while (link != NULL) {
 		struct _widget_node *wn = link->data;
@@ -1389,7 +1446,7 @@ e_config_page_next (EConfig *ec, const gchar *pageid)
 }
 
 /**
- * e_config_page_next:
+ * e_config_page_prev:
  * @ec: an #EConfig
  * @pageid: The path of the page item.
  *
@@ -1400,13 +1457,14 @@ e_config_page_next (EConfig *ec, const gchar *pageid)
  * first configured and visible page.
  **/
 const gchar *
-e_config_page_prev (EConfig *ec, const gchar *pageid)
+e_config_page_prev (EConfig *ec,
+                    const gchar *pageid)
 {
 	GList *link;
 	gint found;
 
 	link = g_list_last (ec->priv->widgets);
-	found = pageid == NULL ? 1:0;
+	found = pageid == NULL ? 1 : 0;
 
 	while (link != NULL) {
 		struct _widget_node *wn = link->data;
@@ -1522,7 +1580,8 @@ gpointer e_config_target_new (EConfig *ep, gint type, gsize size)
  * free custom targets.
  **/
 void
-e_config_target_free (EConfig *ep, gpointer o)
+e_config_target_free (EConfig *ep,
+                      gpointer o)
 {
 	EConfigTarget *t = o;
 
@@ -1534,27 +1593,26 @@ e_config_target_free (EConfig *ep, gpointer o)
 /* Config menu plugin handler */
 
 /*
-<e-plugin
-  class="org.gnome.mail.plugin.config:1.0"
-  id="org.gnome.mail.plugin.config.item:1.0"
-  type="shlib"
-  location="/opt/gnome2/lib/camel/1.0/libcamelimap.so"
-  name="imap"
-  description="IMAP4 and IMAP4v1 mail store">
-  <hook class="org.gnome.mail.configMenu:1.0"
-        handler="HandleConfig">
-  <menu id="any" target="select">
-   <item
-    type="item|toggle|radio|image|submenu|bar"
-    active
-    path="foo/bar"
-    label="label"
-    icon="foo"
-    activate="ep_view_emacs"/>
-  </menu>
-  </extension>
-
-*/
+ * <e-plugin
+ *   class="org.gnome.mail.plugin.config:1.0"
+ *   id="org.gnome.mail.plugin.config.item:1.0"
+ *   type="shlib"
+ *   location="/opt/gnome2/lib/camel/1.0/libcamelimap.so"
+ *   name="imap"
+ *   description="IMAP4 and IMAP4v1 mail store">
+ *   <hook class="org.gnome.mail.configMenu:1.0"
+ *         handler="HandleConfig">
+ *   <menu id="any" target="select">
+ *    <item
+ *     type="item|toggle|radio|image|submenu|bar"
+ *     active
+ *     path="foo/bar"
+ *     label="label"
+ *     icon="foo"
+ *     activate="ep_view_emacs"/>
+ *   </menu>
+ * </e-plugin>
+ */
 
 #define emph ((EConfigHook *)eph)
 
@@ -1578,27 +1636,27 @@ G_DEFINE_TYPE (
 	E_TYPE_PLUGIN_HOOK)
 
 static void
-ech_commit (EConfig *ec, GSList *items, gpointer data)
+ech_commit (EConfig *ec,
+            EConfigHookGroup *group)
 {
-	struct _EConfigHookGroup *group = data;
-
 	if (group->commit && group->hook->hook.plugin->enabled)
 		e_plugin_invoke (group->hook->hook.plugin, group->commit, ec->target);
 }
 
 static void
-ech_abort (EConfig *ec, GSList *items, gpointer data)
+ech_abort (EConfig *ec,
+           EConfigHookGroup *group)
 {
-	struct _EConfigHookGroup *group = data;
-
 	if (group->abort && group->hook->hook.plugin->enabled)
 		e_plugin_invoke (group->hook->hook.plugin, group->abort, ec->target);
 }
 
 static gboolean
-ech_check (EConfig *ec, const gchar *pageid, gpointer data)
+ech_check (EConfig *ec,
+           const gchar *pageid,
+           gpointer data)
 {
-	struct _EConfigHookGroup *group = data;
+	EConfigHookGroup *group = data;
 	EConfigHookPageCheckData hdata;
 
 	if (!group->hook->hook.plugin->enabled)
@@ -1612,9 +1670,10 @@ ech_check (EConfig *ec, const gchar *pageid, gpointer data)
 }
 
 static void
-ech_config_factory (EConfig *emp, gpointer data)
+ech_config_factory (EConfig *emp,
+                    gpointer data)
 {
-	struct _EConfigHookGroup *group = data;
+	EConfigHookGroup *group = data;
 
 	d(printf("config factory called %s\n", group->id?group->id:"all menus"));
 
@@ -1622,8 +1681,15 @@ ech_config_factory (EConfig *emp, gpointer data)
 	    || !group->hook->hook.plugin->enabled)
 		return;
 
-	if (group->items)
-		e_config_add_items (emp, group->items, ech_commit, ech_abort, NULL, group);
+	if (group->items) {
+		e_config_add_items (emp, group->items, NULL, group);
+		g_signal_connect (
+			emp, "abort",
+			G_CALLBACK (ech_abort), group);
+		g_signal_connect (
+			emp, "commit",
+			G_CALLBACK (ech_commit), group);
+	}
 
 	if (group->check)
 		e_config_add_page_check (emp, NULL, ech_check, group);
@@ -1639,7 +1705,7 @@ emph_free_item (struct _EConfigItem *item)
 }
 
 static void
-emph_free_group (struct _EConfigHookGroup *group)
+emph_free_group (EConfigHookGroup *group)
 {
 	g_slist_foreach (group->items, (GFunc) emph_free_item, NULL);
 	g_slist_free (group->items);
@@ -1653,9 +1719,10 @@ ech_config_widget_factory (EConfig *config,
                            EConfigItem *item,
                            GtkWidget *parent,
                            GtkWidget *old,
+                           gint position,
                            gpointer data)
 {
-	struct _EConfigHookGroup *group = data;
+	EConfigHookGroup *group = data;
 	EConfigHookItemFactoryData factory_data;
 	EPlugin *plugin;
 
@@ -1664,6 +1731,7 @@ ech_config_widget_factory (EConfig *config,
 	factory_data.target = config->target;
 	factory_data.parent = parent;
 	factory_data.old = old;
+	factory_data.position = position;
 
 	plugin = group->hook->hook.plugin;
 	return e_plugin_invoke (plugin, item->user_data, &factory_data);
@@ -1674,10 +1742,11 @@ ech_config_section_factory (EConfig *config,
                             EConfigItem *item,
                             GtkWidget *parent,
                             GtkWidget *old,
+                            gint position,
                             gpointer data,
-			    GtkWidget **real_frame)
+                            GtkWidget **real_frame)
 {
-	struct _EConfigHookGroup *group = data;
+	EConfigHookGroup *group = data;
 	GtkWidget *label = NULL;
 	GtkWidget *widget;
 	EPlugin *plugin;
@@ -1743,7 +1812,10 @@ ech_config_section_factory (EConfig *config,
 }
 
 static struct _EConfigItem *
-emph_construct_item (EPluginHook *eph, EConfigHookGroup *menu, xmlNodePtr root, EConfigHookTargetMap *map)
+emph_construct_item (EPluginHook *eph,
+                     EConfigHookGroup *menu,
+                     xmlNodePtr root,
+                     EConfigHookTargetMap *map)
 {
 	struct _EConfigItem *item;
 
@@ -1775,10 +1847,11 @@ error:
 	return NULL;
 }
 
-static struct _EConfigHookGroup *
-emph_construct_menu (EPluginHook *eph, xmlNodePtr root)
+static EConfigHookGroup *
+emph_construct_menu (EPluginHook *eph,
+                     xmlNodePtr root)
 {
-	struct _EConfigHookGroup *menu;
+	EConfigHookGroup *menu;
 	xmlNodePtr node;
 	EConfigHookTargetMap *map;
 	EConfigHookClass *class = (EConfigHookClass *) G_OBJECT_GET_CLASS (eph);
@@ -1825,7 +1898,9 @@ error:
 }
 
 static gint
-emph_construct (EPluginHook *eph, EPlugin *ep, xmlNodePtr root)
+emph_construct (EPluginHook *eph,
+                EPlugin *ep,
+                xmlNodePtr root)
 {
 	xmlNodePtr node;
 	EConfigClass *class;
@@ -1840,7 +1915,7 @@ emph_construct (EPluginHook *eph, EPlugin *ep, xmlNodePtr root)
 	node = root->children;
 	while (node) {
 		if (strcmp((gchar *)node->name, "group") == 0) {
-			struct _EConfigHookGroup *group;
+			EConfigHookGroup *group;
 
 			group = emph_construct_menu (eph, node);
 			if (group) {
