@@ -105,7 +105,7 @@ part_combobox_changed (GtkComboBox *combobox,
 		return;
 
 	/* here we do a widget shuffle, throw away the old widget/rulepart,
-	   and create another */
+	 * and create another */
 	if (data->partwidget)
 		gtk_container_remove (GTK_CONTAINER (data->container), data->partwidget);
 
@@ -203,7 +203,8 @@ less_parts (GtkWidget *button,
 static void
 attach_rule (GtkWidget *rule,
              FilterRuleData *data,
-             EFilterPart *part, gint row)
+             EFilterPart *part,
+             gint row)
 {
 	GtkWidget *remove;
 
@@ -611,7 +612,8 @@ filter_rule_build_code (EFilterRule *rule,
 }
 
 static void
-filter_rule_copy (EFilterRule *dest, EFilterRule *src)
+filter_rule_copy (EFilterRule *dest,
+                  EFilterRule *src)
 {
 	GList *node;
 
@@ -642,6 +644,15 @@ filter_rule_copy (EFilterRule *dest, EFilterRule *src)
 	}
 }
 
+static void
+ensure_scrolled_width_cb (GtkAdjustment *adj,
+			  GParamSpec *param_spec,
+			  GtkScrolledWindow *scrolled_window)
+{
+	gtk_scrolled_window_set_min_content_width (scrolled_window,
+		gtk_adjustment_get_upper (adj));
+}
+
 static GtkWidget *
 filter_rule_get_widget (EFilterRule *rule,
                         ERuleContext *context)
@@ -658,7 +669,7 @@ filter_rule_get_widget (EFilterRule *rule,
 	gint rows, i;
 
 	/* this stuff should probably be a table, but the
-	   rule parts need to be a vbox */
+	 * rule parts need to be a vbox */
 	vbox = gtk_vbox_new (FALSE, 6);
 
 	label = gtk_label_new_with_mnemonic (_("R_ule name:"));
@@ -723,7 +734,7 @@ filter_rule_get_widget (EFilterRule *rule,
 		label = gtk_label_new_with_mnemonic (_("_Find items:"));
 		combobox = gtk_combo_box_text_new ();
 
-		for (i=0;i<2;i++) {
+		for (i = 0; i < 2; i++) {
 			gtk_combo_box_text_append_text (
 				GTK_COMBO_BOX_TEXT (combobox),
 				_(thread_types[i]));
@@ -759,7 +770,7 @@ filter_rule_get_widget (EFilterRule *rule,
 		label = gtk_label_new_with_mnemonic (_("I_nclude threads:"));
 		combobox = gtk_combo_box_text_new ();
 
-		for (i=0;i<5;i++) {
+		for (i = 0; i < 5; i++) {
 			gtk_combo_box_text_append_text (
 				GTK_COMBO_BOX_TEXT (combobox),
 				_(thread_types[i]));
@@ -814,9 +825,11 @@ filter_rule_get_widget (EFilterRule *rule,
 	vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 1.0, 1.0, 1.0));
 	scrolledwindow = gtk_scrolled_window_new (hadj, vadj);
 
+	g_signal_connect (hadj, "notify::upper", G_CALLBACK (ensure_scrolled_width_cb), scrolledwindow);
+
 	gtk_scrolled_window_set_policy (
 		GTK_SCROLLED_WINDOW (scrolledwindow),
-		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
 	gtk_scrolled_window_add_with_viewport (
 		GTK_SCROLLED_WINDOW (scrolledwindow), parts);
@@ -1107,7 +1120,7 @@ e_filter_rule_next_list (GList *list,
 }
 
 EFilterRule *
-e_filter_rule_find_list (GList * list,
+e_filter_rule_find_list (GList *list,
                          const gchar *name,
                          const gchar *source)
 {
