@@ -67,8 +67,6 @@ static void gnome_canvas_pixbuf_bounds (GnomeCanvasItem *item,
 
 G_DEFINE_TYPE (GnomeCanvasPixbuf, gnome_canvas_pixbuf, GNOME_TYPE_CANVAS_ITEM)
 
-
-
 /* Class initialization function for the pixbuf canvas item */
 static void
 gnome_canvas_pixbuf_class_init (GnomeCanvasPixbufClass *class)
@@ -128,14 +126,12 @@ gnome_canvas_pixbuf_dispose (GnomeCanvasItem *object)
 		GNOME_CANVAS_ITEM_CLASS (gnome_canvas_pixbuf_parent_class)->dispose (object);
 }
 
-
-
 /* Set_property handler for the pixbuf canvas item */
 static void
-gnome_canvas_pixbuf_set_property (GObject            *object,
-				  guint               param_id,
-				  const GValue       *value,
-				  GParamSpec         *pspec)
+gnome_canvas_pixbuf_set_property (GObject *object,
+                                  guint param_id,
+                                  const GValue *value,
+                                  GParamSpec *pspec)
 {
 	GnomeCanvasItem *item;
 	GnomeCanvasPixbuf *gcp;
@@ -170,10 +166,10 @@ gnome_canvas_pixbuf_set_property (GObject            *object,
 
 /* Get_property handler for the pixbuf canvasi item */
 static void
-gnome_canvas_pixbuf_get_property (GObject            *object,
-				  guint               param_id,
-				  GValue             *value,
-				  GParamSpec         *pspec)
+gnome_canvas_pixbuf_get_property (GObject *object,
+                                  guint param_id,
+                                  GValue *value,
+                                  GParamSpec *pspec)
 {
 	GnomeCanvasPixbuf *gcp;
 	GnomeCanvasPixbufPrivate *priv;
@@ -195,8 +191,6 @@ gnome_canvas_pixbuf_get_property (GObject            *object,
 	}
 }
 
-
-
 /* Bounds and utilities */
 
 /* Recomputes the bounding box of a pixbuf canvas item.  The horizontal and
@@ -233,8 +227,6 @@ recompute_bounding_box (GnomeCanvasPixbuf *gcp)
 	item->y2 = ceil (y2);
 }
 
-
-
 /* Update sequence */
 
 /* Update handler for the pixbuf canvas item */
@@ -259,12 +251,14 @@ gnome_canvas_pixbuf_update (GnomeCanvasItem *item,
 		item->canvas, item->x1, item->y1, item->x2, item->y2);
 }
 
-
-
 /* Draw handler for the pixbuf canvas item */
 static void
-gnome_canvas_pixbuf_draw (GnomeCanvasItem *item, cairo_t *cr,
-			  gint x, gint y, gint width, gint height)
+gnome_canvas_pixbuf_draw (GnomeCanvasItem *item,
+                          cairo_t *cr,
+                          gint x,
+                          gint y,
+                          gint width,
+                          gint height)
 {
 	GnomeCanvasPixbuf *gcp;
 	GnomeCanvasPixbufPrivate *priv;
@@ -286,8 +280,6 @@ gnome_canvas_pixbuf_draw (GnomeCanvasItem *item, cairo_t *cr,
 	cairo_restore (cr);
 }
 
-
-
 /* Point handler for the pixbuf canvas item */
 static GnomeCanvasItem *
 gnome_canvas_pixbuf_point (GnomeCanvasItem *item,
@@ -329,8 +321,6 @@ gnome_canvas_pixbuf_point (GnomeCanvasItem *item,
 		return item;
 }
 
-
-
 /* Bounds handler for the pixbuf canvas item */
 static void
 gnome_canvas_pixbuf_bounds (GnomeCanvasItem *item,
