@@ -462,6 +462,7 @@ mail_account_store_service_removed (EMailAccountStore *store,
 
 	EAccountList *account_list;
 	EAccount *account;
+	CamelProvider *provider;
 	const gchar *uid;
 
 	account_list = e_get_account_list ();
@@ -469,15 +470,15 @@ mail_account_store_service_removed (EMailAccountStore *store,
 	account = e_get_account_by_uid (uid);
 	g_return_if_fail (account != NULL);
 
-	if (account->enabled) {
-		CamelProvider *provider;
+	/* no change */
+	if (!account->enabled)
+		return;
 
-		provider = camel_service_get_provider (service);
-		g_return_if_fail (provider != NULL);
+	provider = camel_service_get_provider (service);
+	g_return_if_fail (provider != NULL);
 
-		if (provider->flags & CAMEL_PROVIDER_IS_STORAGE)
-			mail_disconnect_store (CAMEL_STORE (service));
-	}
+	if (provider->flags & CAMEL_PROVIDER_IS_STORAGE)
+		mail_disconnect_store (CAMEL_STORE (service));
 
 	/* Remove all the proxies the account has created.
 	 * FIXME This proxy stuff belongs in evolution-groupwise. */
@@ -521,6 +522,10 @@ mail_account_store_service_enabled (EMailAccountStore *store,
 		account = e_get_account_by_uid (uid);
 		g_return_if_fail (account != NULL);
 
+		/* no change */
+		if (account->enabled)
+			return;
+
 		account->enabled = TRUE;
 
 		e_account_list_change (account_list, account);
@@ -562,6 +567,10 @@ mail_account_store_service_disabled (EMailAccountStore *store,
 		account = e_get_account_by_uid (uid);
 		g_return_if_fail (account != NULL);
 
+		/* no change */
+		if (!account->enabled)
+			return;
+
 		account->enabled = FALSE;
 
 		provider = camel_service_get_provider (service);
@@ -1106,7 +1115,7 @@ e_mail_account_store_remove_service (EMailAccountStore *store,
                                      CamelService *service)
 {
 	GtkTreeIter iter;
-	gboolean proceed;
+	gboolean proceed = TRUE;
 
 	g_return_if_fail (E_IS_MAIL_ACCOUNT_STORE (store));
 	g_return_if_fail (CAMEL_IS_SERVICE (service));
@@ -1114,10 +1123,11 @@ e_mail_account_store_remove_service (EMailAccountStore *store,
 	if (!mail_account_store_get_iter (store, service, &iter))
 		g_return_if_reached ();
 
-	/* Possibly request user confirmation. */
-	g_signal_emit (
-		store, signals[REMOVE_REQUESTED], 0,
-		parent_window, service, &proceed);
+	/* If no parent window was given, skip the request signal. */
+	if (GTK_IS_WINDOW (parent_window))
+		g_signal_emit (
+			store, signals[REMOVE_REQUESTED], 0,
+			parent_window, service, &proceed);
 
 	if (proceed) {
 		g_object_ref (service);
@@ -1138,7 +1148,7 @@ e_mail_account_store_enable_service (EMailAccountStore *store,
                                      CamelService *service)
 {
 	GtkTreeIter iter;
-	gboolean proceed;
+	gboolean proceed = TRUE;
 
 	g_return_if_fail (E_IS_MAIL_ACCOUNT_STORE (store));
 	g_return_if_fail (CAMEL_IS_SERVICE (service));
@@ -1146,10 +1156,11 @@ e_mail_account_store_enable_service (EMailAccountStore *store,
 	if (!mail_account_store_get_iter (store, service, &iter))
 		g_return_if_reached ();
 
-	/* Possibly request user confirmation. */
-	g_signal_emit (
-		store, signals[ENABLE_REQUESTED], 0,
-		parent_window, service, &proceed);
+	/* If no parent window was given, skip the request signal. */
+	if (GTK_IS_WINDOW (parent_window))
+		g_signal_emit (
+			store, signals[ENABLE_REQUESTED], 0,
+			parent_window, service, &proceed);
 
 	if (proceed) {
 		gtk_list_store_set (
@@ -1166,7 +1177,7 @@ e_mail_account_store_disable_service (EMailAccountStore *store,
                                       CamelService *service)
 {
 	GtkTreeIter iter;
-	gboolean proceed;
+	gboolean proceed = TRUE;
 
 	g_return_if_fail (E_IS_MAIL_ACCOUNT_STORE (store));
 	g_return_if_fail (CAMEL_IS_SERVICE (service));
@@ -1174,10 +1185,11 @@ e_mail_account_store_disable_service (EMailAccountStore *store,
 	if (!mail_account_store_get_iter (store, service, &iter))
 		g_return_if_reached ();
 
-	/* Possibly request user confirmation. */
-	g_signal_emit (
-		store, signals[DISABLE_REQUESTED], 0,
-		parent_window, service, &proceed);
+	/* If no parent window was given, skip the request signal. */
+	if (GTK_IS_WINDOW (parent_window))
+		g_signal_emit (
+			store, signals[DISABLE_REQUESTED], 0,
+			parent_window, service, &proceed);
 
 	if (proceed) {
 		gtk_list_store_set (
@@ -1264,7 +1276,6 @@ e_mail_account_store_reorder_services (EMailAccountStore *store,
 	gint *new_order;
 	gint n_children;
 	gint new_pos = 0;
-	guint length;
 
 	g_return_if_fail (E_IS_MAIL_ACCOUNT_STORE (store));
 
@@ -1275,13 +1286,15 @@ e_mail_account_store_reorder_services (EMailAccountStore *store,
 	if (ordered_services != NULL && g_queue_is_empty (ordered_services))
 		ordered_services = NULL;
 
-	use_default_order = (ordered_services == NULL);
-
+	/* If the length of the custom ordering disagrees with the
+	 * number of rows in the store, revert to default ordering. */
 	if (ordered_services != NULL) {
-		length = g_queue_get_length (ordered_services);
-		g_return_if_fail (length == n_children);
+		if (g_queue_get_length (ordered_services) != n_children)
+			ordered_services = NULL;
 	}
 
+	use_default_order = (ordered_services == NULL);
+
 	/* Build a queue of CamelServices in the order they appear in
 	 * the list store.  We'll use this to construct the mapping to
 	 * pass to gtk_list_store_reorder(). */
