@@ -45,6 +45,10 @@
 #include "e-mail-tab-picker.h"
 #endif
 
+#define E_MAIL_NOTEBOOK_VIEW_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_MAIL_NOTEBOOK_VIEW, EMailNotebookViewPrivate))
+
 struct _EMailNotebookViewPrivate {
 	GtkNotebook *book;
 	EMailView *current_view;
@@ -75,7 +79,9 @@ static void e_mail_notebook_view_reader_init (EMailReaderInterface *interface);
 G_DEFINE_TYPE_WITH_CODE (
 	EMailNotebookView, e_mail_notebook_view, E_TYPE_MAIL_VIEW,
 	G_IMPLEMENT_INTERFACE (
-		E_TYPE_MAIL_READER, e_mail_notebook_view_reader_init))
+		E_TYPE_MAIL_READER, e_mail_notebook_view_reader_init)
+	G_IMPLEMENT_INTERFACE (
+		E_TYPE_EXTENSIBLE, NULL))
 
 #if HAVE_CLUTTER
 static void
@@ -100,7 +106,7 @@ emnv_get_page_num (EMailNotebookView *view,
 
 	n = gtk_notebook_get_n_pages (priv->book);
 
-	for (i=0; i<n; i++) {
+	for (i = 0; i < n; i++) {
 		GtkWidget *curr = gtk_notebook_get_nth_page (priv->book, i);
 		if (curr == widget)
 			return i;
@@ -122,7 +128,7 @@ mnv_page_changed (GtkNotebook *book,
 	EMailView *mview;
 	CamelFolder *folder;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	priv = E_MAIL_NOTEBOOK_VIEW_GET_PRIVATE (view);
 
 	shell_view = e_mail_view_get_shell_view (view);
 	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
@@ -164,8 +170,8 @@ fix_tab_picker_width (GtkWidget *widget,
 {
 	ClutterActor *stage = g_object_get_data ((GObject *)actor, "stage");
 
-	clutter_actor_set_size (actor, allocation->width-1, -1);
-	clutter_actor_set_size (stage, allocation->width-1, -1);
+	clutter_actor_set_size (actor, allocation->width - 1, -1);
+	clutter_actor_set_size (stage, allocation->width - 1, -1);
 }
 
 static void
@@ -230,8 +236,8 @@ tab_picker_preview_mode_notify (EMailTabPicker *picker,
 
 static void
 mnv_tab_anim_frame_cb (ClutterTimeline *timeline,
-                               gint             frame_num,
-                               EMailTab          *tab)
+                       gint frame_num,
+                       EMailTab *tab)
 {
 	if (!clutter_actor_get_parent (CLUTTER_ACTOR (tab))) {
 		clutter_timeline_stop (timeline);
@@ -246,7 +252,7 @@ mnv_tab_anim_frame_cb (ClutterTimeline *timeline,
 
 static void
 mnv_tab_anim_complete_cb (ClutterTimeline *timeline,
-                                  EMailTab          *tab)
+                          EMailTab *tab)
 {
 	e_mail_tab_set_width (tab, 200);
 	g_object_unref (tab);
@@ -261,7 +267,7 @@ struct _tab_data {
 
 static void
 mnv_tab_closed_cb (ClutterTimeline *timeline,
-		struct _tab_data *data)
+                   struct _tab_data *data)
 {
 	EMailView *page = g_object_get_data ((GObject *)data->tab, "page");
 	EMailView *prev;
@@ -291,7 +297,8 @@ mnv_tab_closed_cb (ClutterTimeline *timeline,
 }
 
 static void
-mnv_tab_closed (EMailTab *tab, EMailNotebookView *view)
+mnv_tab_closed (EMailTab *tab,
+                EMailNotebookView *view)
 {
 	EMailNotebookViewPrivate *priv = view->priv;
 	gint page, cur;
@@ -327,8 +334,8 @@ mnv_tab_closed (EMailTab *tab, EMailNotebookView *view)
 
 static void
 tab_activated_cb (EMailTabPicker *picker,
-		  EMailTab	 *tab,
-		  EMailNotebookView *view)
+                  EMailTab *tab,
+                  EMailNotebookView *view)
 {
 	EMailView *page = g_object_get_data ((GObject *)tab, "page");
 	gint num = emnv_get_page_num (view, (GtkWidget *) page);
@@ -339,7 +346,7 @@ tab_activated_cb (EMailTabPicker *picker,
 
 static void
 tab_remove_gtk_cb (GtkWidget *button,
-		   EMailNotebookView *view)
+                   EMailNotebookView *view)
 {
 	EMailView *page = g_object_get_data ((GObject *)button, "page");
 	EMailView *prev;
@@ -370,8 +377,8 @@ tab_remove_gtk_cb (GtkWidget *button,
 
 static void
 adjust_label_size_request (GtkWidget *view,
-			   GtkAllocation *allocation,
-			   GtkWidget *label)
+                           GtkAllocation *allocation,
+                           GtkWidget *label)
 {
 	GtkRequisition requisition;
 	gint max_width = allocation->width / 2;
@@ -387,7 +394,7 @@ adjust_label_size_request (GtkWidget *view,
 
 static void
 disconnect_label_adjusting (EMailNotebookView *view,
-			    GtkWidget *label)
+                            GtkWidget *label)
 {
 	g_signal_handlers_disconnect_by_func (
 		view,
@@ -397,8 +404,8 @@ disconnect_label_adjusting (EMailNotebookView *view,
 
 static GtkWidget *
 create_tab_label (EMailNotebookView *view,
-		  EMailView *page,
-		  const gchar *str)
+                  EMailView *page,
+                  const gchar *str)
 {
 	GtkWidget *container, *widget;
 	GtkAllocation allocation;
@@ -468,11 +475,12 @@ fix_clutter_embed_width (GtkWidget *widget,
 	GtkAllocation galoc;
 
 	gtk_widget_get_allocation (embed, &galoc);
-	clutter_actor_set_size (actor, allocation->width-1, galoc.height);
+	clutter_actor_set_size (actor, allocation->width - 1, galoc.height);
 }
 
 static GtkWidget *
-create_under_clutter (GtkWidget *widget, GtkWidget *paned)
+create_under_clutter (GtkWidget *widget,
+                      GtkWidget *paned)
 {
 	GtkWidget *embed;
 	ClutterActor *stage, *actor;
@@ -514,7 +522,7 @@ mail_notebook_view_constructed (GObject *object)
 	ClutterTimeline *timeline;
 #endif
 
-	priv = E_MAIL_NOTEBOOK_VIEW (object)->priv;
+	priv = E_MAIL_NOTEBOOK_VIEW_GET_PRIVATE (object);
 
 	container = GTK_WIDGET (object);
 
@@ -622,6 +630,8 @@ mail_notebook_view_constructed (GObject *object)
 		priv->book, "switch-page",
 		G_CALLBACK (mnv_page_changed), object);
 
+	e_extensible_load_extensions (E_EXTENSIBLE (object));
+
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (e_mail_notebook_view_parent_class)->constructed (object);
 }
@@ -632,26 +642,28 @@ mail_notebook_view_set_property (GObject *object,
                                  const GValue *value,
                                  GParamSpec *pspec)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (object)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (object);
+	current_view = notebook_view->priv->current_view;
 
 	switch (property_id) {
 		case PROP_FORWARD_STYLE:
 			e_mail_reader_set_forward_style (
-				E_MAIL_READER (priv->current_view),
+				E_MAIL_READER (current_view),
 				g_value_get_enum (value));
 			return;
 
 		case PROP_GROUP_BY_THREADS:
 			e_mail_reader_set_group_by_threads (
-				E_MAIL_READER (priv->current_view),
+				E_MAIL_READER (current_view),
 				g_value_get_boolean (value));
 			return;
 
 		case PROP_REPLY_STYLE:
 			e_mail_reader_set_reply_style (
-				E_MAIL_READER (priv->current_view),
+				E_MAIL_READER (current_view),
 				g_value_get_enum (value));
 			return;
 	}
@@ -665,30 +677,32 @@ mail_notebook_view_get_property (GObject *object,
                                  GValue *value,
                                  GParamSpec *pspec)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (object)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (object);
+	current_view = notebook_view->priv->current_view;
 
 	switch (property_id) {
 		case PROP_FORWARD_STYLE:
 			g_value_set_enum (
 				value,
 				e_mail_reader_get_forward_style (
-				E_MAIL_READER (priv->current_view)));
+				E_MAIL_READER (current_view)));
 			return;
 
 		case PROP_GROUP_BY_THREADS:
 			g_value_set_boolean (
 				value,
 				e_mail_reader_get_group_by_threads (
-				E_MAIL_READER (priv->current_view)));
+				E_MAIL_READER (current_view)));
 			return;
 
 		case PROP_REPLY_STYLE:
 			g_value_set_enum (
 				value,
 				e_mail_reader_get_reply_style (
-				E_MAIL_READER (priv->current_view)));
+				E_MAIL_READER (current_view)));
 			return;
 	}
 
@@ -699,118 +713,136 @@ static void
 mail_notebook_view_set_search_strings (EMailView *view,
                                        GSList *search_strings)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	e_mail_view_set_search_strings (priv->current_view, search_strings);
+	e_mail_view_set_search_strings (current_view, search_strings);
 }
 
 static GalViewInstance *
 mail_notebook_view_get_view_instance (EMailView *view)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return NULL;
 
-	return e_mail_view_get_view_instance (priv->current_view);
+	return e_mail_view_get_view_instance (current_view);
 }
 
 static void
 mail_notebook_view_update_view_instance (EMailView *view)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return;
 
-	e_mail_view_update_view_instance (priv->current_view);
+	e_mail_view_update_view_instance (current_view);
 }
 
 static void
 mail_notebook_view_set_orientation (EMailView *view,
                                     GtkOrientation orientation)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return;
 
-	e_mail_view_set_orientation (priv->current_view, orientation);
+	e_mail_view_set_orientation (current_view, orientation);
 }
 
 static GtkOrientation
 mail_notebook_view_get_orientation (EMailView *view)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return GTK_ORIENTATION_VERTICAL;
 
-	return e_mail_view_get_orientation (priv->current_view);
+	return e_mail_view_get_orientation (current_view);
 }
 
 static gboolean
 mail_notebook_view_get_show_deleted (EMailView *view)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return FALSE;
 
-	return e_mail_view_get_show_deleted (priv->current_view);
+	return e_mail_view_get_show_deleted (current_view);
 }
 
 static void
 mail_notebook_view_set_show_deleted (EMailView *view,
                                      gboolean show_deleted)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return;
 
-	e_mail_view_set_show_deleted (priv->current_view, show_deleted);
+	e_mail_view_set_show_deleted (current_view, show_deleted);
 }
 
 static gboolean
 mail_notebook_view_get_preview_visible (EMailView *view)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return FALSE;
 
-	return e_mail_view_get_preview_visible (priv->current_view);
+	return e_mail_view_get_preview_visible (current_view);
 }
 
 static void
 mail_notebook_view_set_preview_visible (EMailView *view,
                                         gboolean preview_visible)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (view)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (view);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return;
 
-	e_mail_view_set_preview_visible (priv->current_view, preview_visible);
+	e_mail_view_set_preview_visible (current_view, preview_visible);
 }
 
 static GtkActionGroup *
@@ -871,27 +903,33 @@ mail_notebook_view_get_backend (EMailReader *reader)
 static EMFormatHTML *
 mail_notebook_view_get_formatter (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return NULL;
 
-	return e_mail_reader_get_formatter (E_MAIL_READER (priv->current_view));
+	reader = E_MAIL_READER (current_view);
+
+	return e_mail_reader_get_formatter (reader);
 }
 
 static gboolean
 mail_notebook_view_get_hide_deleted (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return FALSE;
 
-	reader = E_MAIL_READER (priv->current_view);
+	reader = E_MAIL_READER (current_view);
 
 	return e_mail_reader_get_hide_deleted (reader);
 }
@@ -899,14 +937,16 @@ mail_notebook_view_get_hide_deleted (EMailReader *reader)
 static GtkWidget *
 mail_notebook_view_get_message_list (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return NULL;
 
-	reader = E_MAIL_READER (priv->current_view);
+	reader = E_MAIL_READER (current_view);
 
 	return e_mail_reader_get_message_list (reader);
 }
@@ -914,18 +954,37 @@ mail_notebook_view_get_message_list (EMailReader *reader)
 static GtkMenu *
 mail_notebook_view_get_popup_menu (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return NULL;
 
-	reader = E_MAIL_READER (priv->current_view);
+	reader = E_MAIL_READER (current_view);
 
 	return e_mail_reader_get_popup_menu (reader);
 }
 
+static EPreviewPane *
+mail_notebook_view_get_preview_pane (EMailReader *reader)
+{
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
+
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
+
+	if (current_view == NULL)
+		return NULL;
+
+	reader = E_MAIL_READER (current_view);
+
+	return e_mail_reader_get_preview_pane (reader);
+}
+
 static GtkWindow *
 mail_notebook_view_get_window (EMailReader *reader)
 {
@@ -941,7 +1000,8 @@ mail_notebook_view_get_window (EMailReader *reader)
 }
 
 static void
-reconnect_changed_event (EMailReader *child, EMailReader *parent)
+reconnect_changed_event (EMailReader *child,
+                         EMailReader *parent)
 {
 	g_signal_emit_by_name (parent, "changed");
 }
@@ -1023,7 +1083,7 @@ mail_netbook_view_open_mail (EMailView *view,
 
 	e_mail_tab_set_preview_actor (tab, clone);
 	e_mail_tab_set_can_close (tab, TRUE);
-	e_mail_tab_picker_add_tab (priv->tab_picker, tab, pos+1);
+	e_mail_tab_picker_add_tab (priv->tab_picker, tab, pos + 1);
 	e_mail_tab_enable_drag (tab, TRUE);
 
 	page = e_mail_tab_picker_get_tab_no (priv->tab_picker, tab);
@@ -1062,7 +1122,8 @@ mail_netbook_view_open_mail (EMailView *view,
 
 #if HAVE_CLUTTER
 static ClutterActor *
-build_histogram (GtkWidget *widget, CamelFolder *folder)
+build_histogram (GtkWidget *widget,
+                 CamelFolder *folder)
 {
 	gint week_time = 60 * 60 * 24 * 7;
 	gint weeks[54];
@@ -1075,7 +1136,7 @@ build_histogram (GtkWidget *widget, CamelFolder *folder)
 	gint x = 0;
 	time_t now = time (NULL);
 
-	for (i=0; i<54; i++)
+	for (i = 0; i < 54; i++)
 		weeks[i] = 0;
 
 	uids = camel_folder_get_uids (folder);
@@ -1098,12 +1159,12 @@ build_histogram (GtkWidget *widget, CamelFolder *folder)
 		}
 	}
 
-	for (i=0; i< 53; i++) {
+	for (i = 0; i< 53; i++) {
 		if (weeks[i] > max)
 			max = weeks[i];
 	}
 
-	ratio = 50.0/max;
+	ratio = 50.0 / max;
 
 	camel_folder_free_uids (folder, uids);
 
@@ -1119,7 +1180,7 @@ build_histogram (GtkWidget *widget, CamelFolder *folder)
 
 	cairo_set_source_rgba (cr, 0.3, 0.2, 0.4, 1.0);
 
-	for (i=51; i>=0; i--) {
+	for (i = 51; i >= 0; i--) {
 		x+=3;
 		cairo_line_to (cr, x, 50 - (weeks[i]*ratio));
 
@@ -1131,7 +1192,7 @@ build_histogram (GtkWidget *widget, CamelFolder *folder)
 	cairo_save (cr);
 
 	cairo_set_source_rgba (cr, 0.8, 0.5, 0.3, 1.0);
-	cairo_arc (cr, x,  50 - (weeks[0] * ratio), 3, 0, 2*M_PI);
+	cairo_arc (cr, x,  50 - (weeks[0] * ratio), 3, 0, 2 *M_PI);
 
 	cairo_fill (cr);
 	cairo_restore (cr);
@@ -1158,7 +1219,7 @@ mail_notebook_view_set_folder (EMailReader *reader,
 	if (folder == NULL)
 		return;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	priv = E_MAIL_NOTEBOOK_VIEW_GET_PRIVATE (reader);
 
 	folder_uri = e_mail_folder_uri_from_folder (folder);
 	new_view = g_hash_table_lookup (priv->views, folder_uri);
@@ -1307,43 +1368,36 @@ mail_notebook_view_set_folder (EMailReader *reader,
 	}
 }
 
-static void
-mail_notebook_view_show_search_bar (EMailReader *reader)
-{
-	EMailNotebookViewPrivate *priv;
-
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
-
-	if (priv->current_view == NULL)
-		return;
-
-	reader = E_MAIL_READER (priv->current_view);
-
-	e_mail_reader_show_search_bar (reader);
-}
-
 static gboolean
 mail_notebook_view_enable_show_folder (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	if (!priv->current_view)
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
+
+	if (current_view == NULL)
 		return FALSE;
 
-	return e_mail_reader_get_enable_show_folder (E_MAIL_READER (priv->current_view));
+	reader = E_MAIL_READER (current_view);
+
+	return e_mail_reader_get_enable_show_folder (reader);
 }
 
 static guint
 mail_notebook_view_open_selected_mail (EMailReader *reader)
 {
-	EMailNotebookViewPrivate *priv;
+	EMailNotebookView *notebook_view;
+	EMailView *current_view;
 
-	priv = E_MAIL_NOTEBOOK_VIEW (reader)->priv;
+	notebook_view = E_MAIL_NOTEBOOK_VIEW (reader);
+	current_view = notebook_view->priv->current_view;
 
-	if (priv->current_view == NULL)
+	if (current_view == NULL)
 		return 0;
 
-	reader = E_MAIL_READER (priv->current_view);
+	reader = E_MAIL_READER (current_view);
 
 	return e_mail_reader_open_selected_mail (reader);
 }
@@ -1401,25 +1455,24 @@ e_mail_notebook_view_reader_init (EMailReaderInterface *interface)
 	interface->get_hide_deleted = mail_notebook_view_get_hide_deleted;
 	interface->get_message_list = mail_notebook_view_get_message_list;
 	interface->get_popup_menu = mail_notebook_view_get_popup_menu;
+	interface->get_preview_pane = mail_notebook_view_get_preview_pane;
 	interface->get_window = mail_notebook_view_get_window;
 	interface->set_folder = mail_notebook_view_set_folder;
-	interface->show_search_bar = mail_notebook_view_show_search_bar;
 	interface->open_selected_mail = mail_notebook_view_open_selected_mail;
 	interface->enable_show_folder = mail_notebook_view_enable_show_folder;
 }
 
 static void
-e_mail_notebook_view_init (EMailNotebookView  *view)
+e_mail_notebook_view_init (EMailNotebookView *view)
 {
-	view->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		view, E_TYPE_MAIL_NOTEBOOK_VIEW, EMailNotebookViewPrivate);
-
+	view->priv = E_MAIL_NOTEBOOK_VIEW_GET_PRIVATE (view);
 	view->priv->inited = FALSE;
 	view->priv->views = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
-emnv_show_folder (EMailNotebookView *view, gpointer not_used)
+emnv_show_folder (EMailNotebookView *view,
+                  gpointer not_used)
 {
 	gint pos;
 	EMailNotebookViewPrivate *priv = view->priv;
@@ -1438,7 +1491,8 @@ emnv_show_folder (EMailNotebookView *view, gpointer not_used)
 }
 
 static void
-emnv_show_prevtab (EMailNotebookView *view, gpointer not_used)
+emnv_show_prevtab (EMailNotebookView *view,
+                   gpointer not_used)
 {
 	gint pos;
 	EMailNotebookViewPrivate *priv = view->priv;
@@ -1450,19 +1504,20 @@ emnv_show_prevtab (EMailNotebookView *view, gpointer not_used)
 #if HAVE_CLUTTER
 	pos = e_mail_tab_picker_get_current_tab (priv->tab_picker);
 	if (pos > 0) {
-		e_mail_tab_picker_set_current_tab (priv->tab_picker, pos-1);
-		mnv_set_current_tab (E_MAIL_NOTEBOOK_VIEW (view), pos-1);
+		e_mail_tab_picker_set_current_tab (priv->tab_picker, pos - 1);
+		mnv_set_current_tab (E_MAIL_NOTEBOOK_VIEW (view), pos - 1);
 	}
 #else
 	pos = gtk_notebook_get_current_page (priv->book);
 	if (pos > 0 )
-		gtk_notebook_set_current_page (priv->book, pos-1);
+		gtk_notebook_set_current_page (priv->book, pos - 1);
 #endif
 
 }
 
 static void
-emnv_show_nexttab (EMailNotebookView *view, gpointer not_used)
+emnv_show_nexttab (EMailNotebookView *view,
+                   gpointer not_used)
 {
 	gint pos;
 	EMailNotebookViewPrivate *priv = view->priv;
@@ -1470,20 +1525,21 @@ emnv_show_nexttab (EMailNotebookView *view, gpointer not_used)
 #if HAVE_CLUTTER
 	pos = e_mail_tab_picker_get_current_tab (priv->tab_picker);
 
-	if (pos < (gtk_notebook_get_n_pages (priv->book)-1)) {
-		e_mail_tab_picker_set_current_tab (priv->tab_picker, pos+1);
-		mnv_set_current_tab (E_MAIL_NOTEBOOK_VIEW (view), pos+1);
+	if (pos < (gtk_notebook_get_n_pages (priv->book) - 1)) {
+		e_mail_tab_picker_set_current_tab (priv->tab_picker, pos + 1);
+		mnv_set_current_tab (E_MAIL_NOTEBOOK_VIEW (view), pos + 1);
 	}
 #else
 	pos = gtk_notebook_get_current_page (priv->book);
-	if (pos < (gtk_notebook_get_n_pages (priv->book)-1))
-		gtk_notebook_set_current_page (priv->book, pos+1);
+	if (pos < (gtk_notebook_get_n_pages (priv->book) - 1))
+		gtk_notebook_set_current_page (priv->book, pos + 1);
 #endif
 
 }
 
 static void
-emnv_close_tab (EMailNotebookView *view, gpointer not_used)
+emnv_close_tab (EMailNotebookView *view,
+                gpointer not_used)
 {
 	EMailNotebookViewPrivate *priv = view->priv;
 
