@@ -21,17 +21,15 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
-#include <glib.h>
 #include <glib/gi18n.h>
 #include "e-mail-paned-view.h"
 
 #include <libedataserver/e-data-server-util.h>
 
 #include "e-util/e-util-private.h"
-#include "e-util/gconf-bridge.h"
 #include "widgets/menus/gal-view-etable.h"
 #include "widgets/menus/gal-view-instance.h"
 #include "widgets/misc/e-paned.h"
@@ -44,8 +42,13 @@
 #include "mail-config.h"
 #include "mail-ops.h"
 #include "message-list.h"
+#include "e-mail-folder-utils.h"
 #include "e-mail-reader-utils.h"
 
+#define E_MAIL_PANED_VIEW_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_MAIL_PANED_VIEW, EMailPanedViewPrivate))
+
 #define E_SHELL_WINDOW_ACTION_GROUP_MAIL(window) \
 	E_SHELL_WINDOW_ACTION_GROUP ((window), "mail")
 
@@ -53,8 +56,7 @@ struct _EMailPanedViewPrivate {
 	GtkWidget *paned;
 	GtkWidget *scrolled_window;
 	GtkWidget *message_list;
-	GtkWidget *search_bar;
-	GtkWidget *preview;
+	GtkWidget *preview_pane;
 
 	EMFormatHTMLDisplay *formatter;
 	GalViewInstance *view_instance;
@@ -86,7 +88,9 @@ static void e_mail_paned_view_reader_init (EMailReaderInterface *interface);
 G_DEFINE_TYPE_WITH_CODE (
 	EMailPanedView, e_mail_paned_view, E_TYPE_MAIL_VIEW,
 	G_IMPLEMENT_INTERFACE (
-		E_TYPE_MAIL_READER, e_mail_paned_view_reader_init))
+		E_TYPE_MAIL_READER, e_mail_paned_view_reader_init)
+	G_IMPLEMENT_INTERFACE (
+		E_TYPE_EXTENSIBLE, NULL))
 
 static void
 mail_paned_view_save_boolean (EMailView *view,
@@ -95,22 +99,25 @@ mail_paned_view_save_boolean (EMailView *view,
 {
 	EShellView *shell_view;
 	EMailReader *reader;
+	CamelFolder *folder;
 	GKeyFile *key_file;
-	const gchar *folder_uri;
+	gchar *folder_uri;
 	gchar *group_name;
 
 	shell_view = e_mail_view_get_shell_view (view);
 	key_file = e_shell_view_get_state_key_file (shell_view);
 
 	reader = E_MAIL_READER (view);
-	folder_uri = e_mail_reader_get_folder_uri (reader);
+	folder = e_mail_reader_get_folder (reader);
 
-	if (folder_uri == NULL)
+	if (folder == NULL)
 		return;
 
+	folder_uri = e_mail_folder_uri_from_folder (folder);
 	group_name = g_strdup_printf ("Folder %s", folder_uri);
 	g_key_file_set_boolean (key_file, group_name, key, value);
 	g_free (group_name);
+	g_free (folder_uri);
 
 	e_shell_view_set_state_dirty (shell_view);
 }
@@ -124,7 +131,7 @@ mail_paned_view_message_list_built_cb (EMailView *view,
 	EShellWindow *shell_window;
 	GKeyFile *key_file;
 
-	priv = E_MAIL_PANED_VIEW (view)->priv;
+	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (view);
 
 	g_signal_handler_disconnect (
 		message_list, priv->message_list_built_id);
@@ -138,24 +145,29 @@ mail_paned_view_message_list_built_cb (EMailView *view,
 	if (message_list->cursor_uid != NULL)
 		;  /* do nothing */
 
-	else if (message_list->folder_uri == NULL)
+	else if (message_list->folder == NULL)
 		;  /* do nothing */
 
 	else if (e_shell_window_get_safe_mode (shell_window))
 		e_shell_window_set_safe_mode (shell_window, FALSE);
 
 	else {
-		const gchar *folder_uri;
+		CamelFolder *folder;
 		const gchar *key;
+		gchar *folder_uri;
 		gchar *group_name;
 		gchar *uid;
 
+		folder = message_list->folder;
+		folder_uri = e_mail_folder_uri_from_folder (folder);
+
 		key = STATE_KEY_SELECTED_MESSAGE;
-		folder_uri = message_list->folder_uri;
 		group_name = g_strdup_printf ("Folder %s", folder_uri);
 		uid = g_key_file_get_string (key_file, group_name, key, NULL);
 		g_free (group_name);
 
+		g_free (folder_uri);
+
 		/* Use selection fallbacks if UID is not found. */
 		message_list_select_uid (message_list, uid, TRUE);
 
@@ -169,22 +181,25 @@ mail_paned_view_message_selected_cb (EMailView *view,
                                      MessageList *message_list)
 {
 	EShellView *shell_view;
+	CamelFolder *folder;
 	GKeyFile *key_file;
-	const gchar *folder_uri;
 	const gchar *key;
+	gchar *folder_uri;
 	gchar *group_name;
 
-	folder_uri = message_list->folder_uri;
+	folder = message_list->folder;
 
 	/* This also gets triggered when selecting a store name on
 	 * the sidebar such as "On This Computer", in which case
-	 * 'folder_uri' will be NULL. */
-	if (folder_uri == NULL)
+	 * 'folder' will be NULL. */
+	if (folder == NULL)
 		return;
 
 	shell_view = e_mail_view_get_shell_view (view);
 	key_file = e_shell_view_get_state_key_file (shell_view);
 
+	folder_uri = e_mail_folder_uri_from_folder (folder);
+
 	key = STATE_KEY_SELECTED_MESSAGE;
 	group_name = g_strdup_printf ("Folder %s", folder_uri);
 
@@ -195,7 +210,7 @@ mail_paned_view_message_selected_cb (EMailView *view,
 	e_shell_view_set_state_dirty (shell_view);
 
 	g_free (group_name);
-
+	g_free (folder_uri);
 }
 
 static void
@@ -204,23 +219,22 @@ mail_paned_view_restore_state_cb (EShellWindow *shell_window,
                                   EMailPanedView *view)
 {
 	EMailPanedViewPrivate *priv;
-	GConfBridge *bridge;
+	GSettings *settings;
 	GObject *object;
-	const gchar *key;
 
 	priv = E_MAIL_PANED_VIEW (view)->priv;
 
-	/* Bind GObject properties to GConf keys. */
+	/* Bind GObject properties to GSettings keys. */
 
-	bridge = gconf_bridge_get ();
+	settings = g_settings_new ("org.gnome.evolution.mail");
 
 	object = G_OBJECT (priv->paned);
-	key = "/apps/evolution/mail/display/hpaned_size";
-	gconf_bridge_bind_property (bridge, key, object, "hposition");
+	g_settings_bind (settings, "hpaned-size", object, "hposition", G_SETTINGS_BIND_DEFAULT);
 
 	object = G_OBJECT (priv->paned);
-	key = "/apps/evolution/mail/display/paned_size";
-	gconf_bridge_bind_property (bridge, key, object, "vposition");
+	g_settings_bind (settings, "paned-size", object, "vposition", G_SETTINGS_BIND_DEFAULT);
+
+	g_object_unref (settings);
 }
 
 static void
@@ -317,7 +331,7 @@ mail_paned_view_dispose (GObject *object)
 {
 	EMailPanedViewPrivate *priv;
 
-	priv = E_MAIL_PANED_VIEW (object)->priv;
+	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (object);
 
 	if (priv->paned != NULL) {
 		g_object_unref (priv->paned);
@@ -334,9 +348,9 @@ mail_paned_view_dispose (GObject *object)
 		priv->message_list = NULL;
 	}
 
-	if (priv->search_bar != NULL) {
-		g_object_unref (priv->search_bar);
-		priv->search_bar = NULL;
+	if (priv->preview_pane != NULL) {
+		g_object_unref (priv->preview_pane);
+		priv->preview_pane = NULL;
 	}
 
 	if (priv->formatter != NULL) {
@@ -354,17 +368,30 @@ mail_paned_view_dispose (GObject *object)
 }
 
 static GtkActionGroup *
-mail_paned_view_get_action_group (EMailReader *reader)
+mail_paned_view_get_action_group (EMailReader *reader,
+                                  EMailReaderActionGroup group)
 {
 	EMailView *view;
 	EShellView *shell_view;
 	EShellWindow *shell_window;
+	const gchar *group_name;
 
 	view = E_MAIL_VIEW (reader);
 	shell_view = e_mail_view_get_shell_view (view);
 	shell_window = e_shell_view_get_shell_window (shell_view);
 
-	return E_SHELL_WINDOW_ACTION_GROUP_MAIL (shell_window);
+	switch (group) {
+		case E_MAIL_READER_ACTION_GROUP_STANDARD:
+			group_name = "mail";
+			break;
+		case E_MAIL_READER_ACTION_GROUP_SEARCH_FOLDERS:
+			group_name = "search-folders";
+			break;
+		default:
+			g_return_val_if_reached (NULL);
+	}
+
+	return e_shell_window_get_action_group (shell_window, group_name);
 }
 
 static EAlertSink *
@@ -398,11 +425,11 @@ mail_paned_view_get_backend (EMailReader *reader)
 static EMFormatHTML *
 mail_paned_view_get_formatter (EMailReader *reader)
 {
-	EMailPanedViewPrivate *priv;
+	EMailPanedView *paned_view;
 
-	priv = E_MAIL_PANED_VIEW (reader)->priv;
+	paned_view = E_MAIL_PANED_VIEW (reader);
 
-	return EM_FORMAT_HTML (priv->formatter);
+	return EM_FORMAT_HTML (paned_view->priv->formatter);
 }
 
 static gboolean
@@ -414,11 +441,11 @@ mail_paned_view_get_hide_deleted (EMailReader *reader)
 static GtkWidget *
 mail_paned_view_get_message_list (EMailReader *reader)
 {
-	EMailPanedViewPrivate *priv;
+	EMailPanedView *paned_view;
 
-	priv = E_MAIL_PANED_VIEW (reader)->priv;
+	paned_view = E_MAIL_PANED_VIEW (reader);
 
-	return priv->message_list;
+	return paned_view->priv->message_list;
 }
 
 static GtkMenu *
@@ -440,6 +467,16 @@ mail_paned_view_get_popup_menu (EMailReader *reader)
 	return GTK_MENU (widget);
 }
 
+static EPreviewPane *
+mail_paned_view_get_preview_pane (EMailReader *reader)
+{
+	EMailPanedView *paned_view;
+
+	paned_view = E_MAIL_PANED_VIEW (reader);
+
+	return E_PREVIEW_PANE (paned_view->priv->preview_pane);
+}
+
 static GtkWindow *
 mail_paned_view_get_window (EMailReader *reader)
 {
@@ -456,8 +493,7 @@ mail_paned_view_get_window (EMailReader *reader)
 
 static void
 mail_paned_view_set_folder (EMailReader *reader,
-                            CamelFolder *folder,
-                            const gchar *folder_uri)
+                            CamelFolder *folder)
 {
 	EMailPanedViewPrivate *priv;
 	EMailView *view;
@@ -468,12 +504,13 @@ mail_paned_view_set_folder (EMailReader *reader,
 	EMailReaderInterface *default_interface;
 	GtkWidget *message_list;
 	GKeyFile *key_file;
+	gchar *folder_uri;
 	gchar *group_name;
 	const gchar *key;
 	gboolean value;
 	GError *error = NULL;
 
-	priv = E_MAIL_PANED_VIEW (reader)->priv;
+	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (reader);
 
 	view = E_MAIL_VIEW (reader);
 	shell_view = e_mail_view_get_shell_view (view);
@@ -488,7 +525,7 @@ mail_paned_view_set_folder (EMailReader *reader,
 
 	/* Chain up to interface's default set_folder() method. */
 	default_interface = g_type_default_interface_peek (E_TYPE_MAIL_READER);
-	default_interface->set_folder (reader, folder, folder_uri);
+	default_interface->set_folder (reader, folder);
 
 	if (folder == NULL)
 		goto exit;
@@ -507,8 +544,10 @@ mail_paned_view_set_folder (EMailReader *reader,
 
 	/* Restore the folder's preview and threaded state. */
 
+	folder_uri = e_mail_folder_uri_from_folder (folder);
 	key_file = e_shell_view_get_state_key_file (shell_view);
 	group_name = g_strdup_printf ("Folder %s", folder_uri);
+	g_free (folder_uri);
 
 	key = STATE_KEY_GROUP_BY_THREADS;
 	value = g_key_file_get_boolean (key_file, group_name, key, &error);
@@ -545,31 +584,28 @@ exit:
 	message_list_thaw (MESSAGE_LIST (message_list));
 }
 
-static void
-mail_paned_view_show_search_bar (EMailReader *reader)
-{
-	EMailPanedViewPrivate *priv;
-
-	priv = E_MAIL_PANED_VIEW (reader)->priv;
-
-	gtk_widget_show (priv->search_bar);
-}
-
 static guint
 mail_paned_view_reader_open_selected_mail (EMailReader *reader)
 {
-	return E_MAIL_PANED_VIEW_GET_CLASS (reader)->
-		open_selected_mail (E_MAIL_PANED_VIEW (reader));
+	EMailPanedView *paned_view;
+	EMailPanedViewClass *class;
+
+	paned_view = E_MAIL_PANED_VIEW (reader);
+
+	class = E_MAIL_PANED_VIEW_GET_CLASS (paned_view);
+	g_return_val_if_fail (class->open_selected_mail != NULL, 0);
+
+	return class->open_selected_mail (paned_view);
 }
 
 static gboolean
 mail_paned_view_enable_show_folder (EMailReader *reader)
 {
-	EMailPanedViewPrivate *priv;
+	EMailPanedView *paned_view;
 
-	priv = E_MAIL_PANED_VIEW (reader)->priv;
+	paned_view = E_MAIL_PANED_VIEW (reader);
 
-	return priv->enable_show_folder ? TRUE : FALSE;
+	return paned_view->priv->enable_show_folder;
 }
 
 static void
@@ -579,21 +615,39 @@ mail_paned_view_constructed (GObject *object)
 	EShellBackend *shell_backend;
 	EShellWindow *shell_window;
 	EShellView *shell_view;
+	EShell *shell;
+	EShellSettings *shell_settings;
 	ESearchBar *search_bar;
 	EMailReader *reader;
+	EMailBackend *backend;
+	EMailSession *session;
 	EMailView *view;
 	GtkWidget *message_list;
 	GtkWidget *container;
 	GtkWidget *widget;
 	EWebView *web_view;
 
-	priv = E_MAIL_PANED_VIEW (object)->priv;
+	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (object);
 	priv->formatter = em_format_html_display_new ();
 
 	view = E_MAIL_VIEW (object);
 	shell_view = e_mail_view_get_shell_view (view);
 	shell_window = e_shell_view_get_shell_window (shell_view);
 	shell_backend = e_shell_view_get_shell_backend (shell_view);
+	shell = e_shell_window_get_shell (shell_window);
+	shell_settings = e_shell_get_shell_settings (shell);
+
+	backend = E_MAIL_BACKEND (shell_backend);
+	session = e_mail_backend_get_session (backend);
+
+	/* Make headers collapsable and store state of headers in config file */
+	em_format_html_set_headers_collapsable (
+		EM_FORMAT_HTML (priv->formatter), TRUE);
+	g_object_bind_property (
+		shell_settings, "paned-view-headers-state",
+		priv->formatter, "headers-state",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
 
 	web_view = em_format_html_get_web_view (
 		EM_FORMAT_HTML (priv->formatter));
@@ -626,7 +680,7 @@ mail_paned_view_constructed (GObject *object)
 
 	container = widget;
 
-	widget = message_list_new (E_MAIL_BACKEND (shell_backend));
+	widget = message_list_new (session);
 	gtk_container_add (GTK_CONTAINER (container), widget);
 	priv->message_list = g_object_ref (widget);
 	gtk_widget_show (widget);
@@ -636,8 +690,8 @@ mail_paned_view_constructed (GObject *object)
 	gtk_widget_show (GTK_WIDGET (web_view));
 
 	widget = e_preview_pane_new (web_view);
-	priv->preview = widget;
 	gtk_paned_pack2 (GTK_PANED (container), widget, FALSE, FALSE);
+	priv->preview_pane = g_object_ref (widget);
 	gtk_widget_show (widget);
 
 	g_object_bind_property (
@@ -646,7 +700,6 @@ mail_paned_view_constructed (GObject *object)
 		G_BINDING_SYNC_CREATE);
 
 	search_bar = e_preview_pane_get_search_bar (E_PREVIEW_PANE (widget));
-	priv->search_bar = g_object_ref (search_bar);
 
 	g_signal_connect_swapped (
 		search_bar, "changed",
@@ -681,6 +734,8 @@ mail_paned_view_constructed (GObject *object)
 	 * set_preview_visible() method relies on it. */
 	e_mail_view_set_preview_visible (view, TRUE);
 
+	e_extensible_load_extensions (E_EXTENSIBLE (object));
+
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (e_mail_paned_view_parent_class)->constructed (object);
 }
@@ -689,13 +744,14 @@ static void
 mail_paned_view_set_search_strings (EMailView *view,
                                     GSList *search_strings)
 {
-	EMailPanedViewPrivate *priv;
+	EMailReader *reader;
+	EPreviewPane *preview_pane;
 	ESearchBar *search_bar;
 	ESearchingTokenizer *tokenizer;
 
-	priv = E_MAIL_PANED_VIEW (view)->priv;
-
-	search_bar = E_SEARCH_BAR (priv->search_bar);
+	reader = E_MAIL_READER (view);
+	preview_pane = e_mail_reader_get_preview_pane (reader);
+	search_bar = e_preview_pane_get_search_bar (preview_pane);
 	tokenizer = e_search_bar_get_tokenizer (search_bar);
 
 	e_searching_tokenizer_set_secondary_case_sensitivity (tokenizer, FALSE);
@@ -713,11 +769,11 @@ mail_paned_view_set_search_strings (EMailView *view,
 static GalViewInstance *
 mail_paned_view_get_view_instance (EMailView *view)
 {
-	EMailPanedViewPrivate *priv;
+	EMailPanedView *paned_view;
 
-	priv = E_MAIL_PANED_VIEW (view)->priv;
+	paned_view = E_MAIL_PANED_VIEW (view);
 
-	return priv->view_instance;
+	return paned_view->priv->view_instance;
 }
 
 static void
@@ -737,10 +793,9 @@ mail_paned_view_update_view_instance (EMailView *view)
 	GtkOrientation orientation;
 	gboolean outgoing_folder;
 	gboolean show_vertical_view;
-	const gchar *folder_uri;
 	gchar *view_id;
 
-	priv = E_MAIL_PANED_VIEW (view)->priv;
+	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (view);
 
 	shell_view = e_mail_view_get_shell_view (view);
 	shell_view_class = E_SHELL_VIEW_GET_CLASS (shell_view);
@@ -752,30 +807,28 @@ mail_paned_view_update_view_instance (EMailView *view)
 
 	reader = E_MAIL_READER (view);
 	folder = e_mail_reader_get_folder (reader);
-	folder_uri = e_mail_reader_get_folder_uri (reader);
 
 	/* If no folder is selected, return silently. */
 	if (folder == NULL)
 		return;
 
-	/* If we have a folder, we should also have a URI. */
-	g_return_if_fail (folder_uri != NULL);
-
 	if (priv->view_instance != NULL) {
 		g_object_unref (priv->view_instance);
 		priv->view_instance = NULL;
 	}
 
-	view_id = g_strdup (folder_uri);
+	view_id = e_mail_folder_uri_from_folder (folder);
 	e_filename_make_safe (view_id);
 
 	outgoing_folder =
-		em_utils_folder_is_drafts (folder, folder_uri) ||
-		em_utils_folder_is_outbox (folder, folder_uri) ||
-		em_utils_folder_is_sent (folder, folder_uri);
+		em_utils_folder_is_drafts (folder) ||
+		em_utils_folder_is_outbox (folder) ||
+		em_utils_folder_is_sent (folder);
 
 	if (e_shell_settings_get_boolean (shell_settings, "mail-global-view-setting"))
-		view_instance = e_shell_view_new_view_instance (shell_view, outgoing_folder ? "global_view_sent_setting" : "global_view_setting");
+		view_instance = e_shell_view_new_view_instance (
+			shell_view, outgoing_folder ?
+			"global_view_sent_setting" : "global_view_setting");
 	else
 		view_instance = e_shell_view_new_view_instance (shell_view, view_id);
 
@@ -967,9 +1020,9 @@ e_mail_paned_view_reader_init (EMailReaderInterface *interface)
 	interface->get_hide_deleted = mail_paned_view_get_hide_deleted;
 	interface->get_message_list = mail_paned_view_get_message_list;
 	interface->get_popup_menu = mail_paned_view_get_popup_menu;
+	interface->get_preview_pane = mail_paned_view_get_preview_pane;
 	interface->get_window = mail_paned_view_get_window;
 	interface->set_folder = mail_paned_view_set_folder;
-	interface->show_search_bar = mail_paned_view_show_search_bar;
 	interface->open_selected_mail = mail_paned_view_reader_open_selected_mail;
 	interface->enable_show_folder = mail_paned_view_enable_show_folder;
 }
@@ -977,8 +1030,7 @@ e_mail_paned_view_reader_init (EMailReaderInterface *interface)
 static void
 e_mail_paned_view_init (EMailPanedView *view)
 {
-	view->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		view, E_TYPE_MAIL_PANED_VIEW, EMailPanedViewPrivate);
+	view->priv = E_MAIL_PANED_VIEW_GET_PRIVATE (view);
 	view->priv->enable_show_folder = FALSE;
 
 	g_signal_connect (
@@ -1009,16 +1061,9 @@ e_mail_paned_view_hide_message_list_pane (EMailPanedView *view,
 		gtk_widget_hide (view->priv->scrolled_window);
 }
 
-GtkWidget *
-e_mail_paned_view_get_preview (EMailPanedView *view)
-{
-	g_return_val_if_fail (E_IS_MAIL_PANED_VIEW (view), NULL);
-
-	return view->priv->preview;
-}
-
 void
-e_mail_paned_view_set_enable_show_folder (EMailPanedView *view, gboolean set)
+e_mail_paned_view_set_enable_show_folder (EMailPanedView *view,
+                                          gboolean set)
 {
 	view->priv->enable_show_folder = set ? 1 : 0;
 }
