@@ -27,7 +27,6 @@
 #include <glib/gi18n-lib.h>
 
 #include <mail/mail-tools.h>
-#include <mail/e-mail-local.h>
 #include <mail/e-mail-folder-utils.h>
 #include <e-util/e-account-utils.h>
 #include <filter/e-filter-rule.h>
@@ -131,10 +130,8 @@ mail_session_handle_draft_headers_thread (GSimpleAsyncResult *simple,
 	e_mail_session_handle_draft_headers_sync (
 		session, context->message, cancellable, &error);
 
-	if (error != NULL) {
-		g_simple_async_result_set_from_error (simple, error);
-		g_error_free (error);
-	}
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
 }
 
 gboolean
@@ -249,10 +246,8 @@ mail_session_handle_source_headers_thread (GSimpleAsyncResult *simple,
 	e_mail_session_handle_source_headers_sync (
 		session, context->message, cancellable, &error);
 
-	if (error != NULL) {
-		g_simple_async_result_set_from_error (simple, error);
-		g_error_free (error);
-	}
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
 }
 
 gboolean
@@ -426,8 +421,7 @@ mail_session_send_to_thread (GSimpleAsyncResult *simple,
 
 			/* XXX This API does not allow for cancellation. */
 			if (!em_utils_connect_service_sync (service, cancellable, &error)) {
-				g_simple_async_result_set_from_error (simple, error);
-				g_error_free (error);
+				g_simple_async_result_take_error (simple, error);
 				return;
 			}
 		}
@@ -448,8 +442,7 @@ mail_session_send_to_thread (GSimpleAsyncResult *simple,
 				cancellable, error ? NULL : &error);
 
 		if (error != NULL) {
-			g_simple_async_result_set_from_error (simple, error);
-			g_error_free (error);
+			g_simple_async_result_take_error (simple, error);
 			return;
 		}
 	}
@@ -466,8 +459,7 @@ mail_session_send_to_thread (GSimpleAsyncResult *simple,
 
 		if (error != NULL) {
 			g_warn_if_fail (folder == NULL);
-			g_simple_async_result_set_from_error (simple, error);
-			g_error_free (error);
+			g_simple_async_result_take_error (simple, error);
 			return;
 		}
 
@@ -480,8 +472,7 @@ mail_session_send_to_thread (GSimpleAsyncResult *simple,
 		g_object_unref (folder);
 
 		if (error != NULL) {
-			g_simple_async_result_set_from_error (simple, error);
-			g_error_free (error);
+			g_simple_async_result_take_error (simple, error);
 			return;
 		}
 	}
@@ -516,7 +507,9 @@ mail_session_send_to_thread (GSimpleAsyncResult *simple,
 
 	/* Append the sent message to a Sent folder. */
 
-	local_sent_folder = e_mail_local_get_folder (E_MAIL_LOCAL_FOLDER_SENT);
+	local_sent_folder =
+		e_mail_session_get_local_folder (
+		session, E_MAIL_LOCAL_FOLDER_SENT);
 
 	/* Try to extract a CamelFolder from the Sent folder URI. */
 	if (context->sent_folder_uri != NULL) {
@@ -615,8 +608,7 @@ exit:
 
 	/* If we were cancelled, disregard any other errors. */
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		g_simple_async_result_set_from_error (simple, error);
-		g_error_free (error);
+		g_simple_async_result_take_error (simple, error);
 
 	/* Stuff the accumulated error messages in a GError. */
 	} else if (error_messages->len > 0) {
@@ -695,14 +687,14 @@ e_mail_session_send_to (EMailSession *session,
 		sent_folder_uri = g_strdup (account->sent_folder_uri);
 	}
 
-	string = camel_header_raw_find (&xev, "X-Evolution-Transport", NULL);
-	if (transport_uid == NULL && string != NULL)
-		transport_uid = g_strstrip (g_strdup (string));
-
 	string = camel_header_raw_find (&xev, "X-Evolution-Fcc", NULL);
 	if (sent_folder_uri == NULL && string != NULL)
 		sent_folder_uri = g_strstrip (g_strdup (string));
 
+	string = camel_header_raw_find (&xev, "X-Evolution-Transport", NULL);
+	if (transport_uid == NULL && string != NULL)
+		transport_uid = g_strstrip (g_strdup (string));
+
 	post_to_uris = g_ptr_array_new ();
 	for (header = xev; header != NULL; header = header->next) {
 		gchar *folder_uri;
@@ -843,10 +835,8 @@ mail_session_unsubscribe_folder_thread (GSimpleAsyncResult *simple,
 	e_mail_session_unsubscribe_folder_sync (
 		session, context->folder_uri, cancellable, &error);
 
-	if (error != NULL) {
-		g_simple_async_result_set_from_error (simple, error);
-		g_error_free (error);
-	}
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
 }
 
 gboolean
