@@ -52,6 +52,11 @@
 #include "e-util/e-alert-dialog.h"
 #include "e-util/e-util-private.h"
 
+#include "shell/e-shell.h"
+#include "shell/e-shell-view.h"
+#include "shell/e-shell-content.h"
+#include "shell/e-shell-window.h"
+
 #include "e-mail-account-store.h"
 #include "e-mail-folder-utils.h"
 #include "e-mail-junk-filter.h"
@@ -76,6 +81,7 @@ typedef struct _SourceContext SourceContext;
 struct _EMailSessionPrivate {
 	EMailAccountStore *account_store;
 	MailFolderCache *folder_cache;
+	EMailLabelListStore *label_store;
 
 	EAccountList *account_list;
 	gulong account_added_handler_id;
@@ -113,6 +119,7 @@ enum {
 	PROP_ACCOUNT_STORE,
 	PROP_FOLDER_CACHE,
 	PROP_JUNK_FILTER_NAME,
+	PROP_LABEL_STORE,
 	PROP_LOCAL_STORE,
 	PROP_VFOLDER_STORE
 };
@@ -170,9 +177,10 @@ static void user_message_exec (struct _user_message_msg *m,
 
 static void
 user_message_response_free (GtkDialog *dialog,
-                            gint button,
-                            struct _user_message_msg *m)
+                            gint button)
 {
+	struct _user_message_msg *m = NULL;
+
 	gtk_widget_destroy ((GtkWidget *) dialog);
 
 	user_message_dialog = NULL;
@@ -194,13 +202,13 @@ user_message_response (GtkDialog *dialog,
                        gint button,
                        struct _user_message_msg *m)
 {
-	/* if !allow_cancel, then we've already replied */
-	if (m->button_captions) {
+	/* if !m or !button_captions, then we've already replied */
+	if (m && m->button_captions) {
 		m->result = button;
 		e_flag_set (m->done);
 	}
 
-	user_message_response_free (dialog, button, m);
+	user_message_response_free (dialog, button);
 }
 
 static void
@@ -208,12 +216,16 @@ user_message_exec (struct _user_message_msg *m,
                    GCancellable *cancellable,
                    GError **error)
 {
+	gboolean info_only;
 	GtkWindow *parent;
+	EShell *shell;
 	const gchar *error_type;
 	gint index;
 	GSList *iter;
 
-	if (!m->ismain && user_message_dialog != NULL) {
+	info_only = g_slist_length (m->button_captions) <= 1;
+
+	if (!m->ismain && user_message_dialog != NULL && !info_only) {
 		g_queue_push_tail (&user_message_queue, mail_msg_ref (m));
 		return;
 	}
@@ -233,8 +245,56 @@ user_message_exec (struct _user_message_msg *m,
 			g_return_if_reached ();
 	}
 
+	shell = e_shell_get_default ();
+
+	/* try to find "mail" view to place the informational alert to */
+	if (info_only) {
+		GtkWindow *active_window;
+		EShellWindow *shell_window;
+		EShellView *shell_view;
+		EShellContent *shell_content = NULL;
+
+		/* check currently active window first, ... */
+		active_window = e_shell_get_active_window (shell);
+		if (active_window && E_IS_SHELL_WINDOW (active_window)) {
+			if (E_IS_SHELL_WINDOW (active_window)) {
+				shell_window = E_SHELL_WINDOW (active_window);
+				shell_view = e_shell_window_peek_shell_view (shell_window, "mail");
+				if (shell_view)
+					shell_content = e_shell_view_get_shell_content (shell_view);
+			}
+		}
+
+		if (!shell_content) {
+			GList *list, *iter;
+
+			list = gtk_application_get_windows (GTK_APPLICATION (shell));
+
+			/* ...then iterate through all opened windows and pick one which has it */
+			for (iter = list; iter != NULL && !shell_content; iter = g_list_next (iter)) {
+				if (E_IS_SHELL_WINDOW (iter->data)) {
+					shell_window = iter->data;
+					shell_view = e_shell_window_peek_shell_view (shell_window, "mail");
+					if (shell_view)
+						shell_content = e_shell_view_get_shell_content (shell_view);
+				}
+			}
+		}
+
+		/* when no shell-content found, which might not happen, but just in case,
+		   process the information alert like usual, through an EAlertDialog machinery
+		*/
+		if (shell_content) {
+			e_alert_submit (E_ALERT_SINK (shell_content), error_type, m->prompt, NULL);
+			return;
+		} else if (!m->ismain && user_message_dialog != NULL) {
+			g_queue_push_tail (&user_message_queue, mail_msg_ref (m));
+			return;
+		}
+	}
+
 	/* Pull in the active window from the shell to get a parent window */
-	parent = e_shell_get_active_window (e_shell_get_default ());
+	parent = e_shell_get_active_window (shell);
 	user_message_dialog = e_alert_dialog_new_for_args (
 		parent, error_type, m->prompt, NULL);
 	g_object_set (user_message_dialog, "resizable", TRUE, NULL);
@@ -267,9 +327,14 @@ user_message_exec (struct _user_message_msg *m,
 		user_message_response (
 			user_message_dialog, response, m);
 	} else {
+		gpointer user_data = m;
+
+		if (g_slist_length (m->button_captions) <= 1)
+			user_data = NULL;
+
 		g_signal_connect (
 			user_message_dialog, "response",
-			G_CALLBACK (user_message_response), m);
+			G_CALLBACK (user_message_response), user_data);
 		gtk_widget_show (user_message_dialog);
 	}
 }
@@ -896,6 +961,13 @@ mail_session_get_property (GObject *object,
 				E_MAIL_SESSION (object)));
 			return;
 
+		case PROP_LABEL_STORE:
+			g_value_set_object (
+				value,
+				e_mail_session_get_label_store (
+				E_MAIL_SESSION (object)));
+			return;
+
 		case PROP_LOCAL_STORE:
 			g_value_set_object (
 				value,
@@ -932,6 +1004,11 @@ mail_session_dispose (GObject *object)
 		priv->folder_cache = NULL;
 	}
 
+	if (priv->label_store != NULL) {
+		g_object_unref (priv->label_store);
+		priv->label_store = NULL;
+	}
+
 	if (priv->account_list != NULL) {
 		g_signal_handler_disconnect (
 			priv->account_list,
@@ -992,6 +1069,46 @@ mail_session_notify (GObject *object,
 		g_object_notify (object, "junk-filter-name");
 }
 
+static gboolean
+mail_session_initialize_stores_idle (gpointer user_data)
+{
+	EMailSession *session = user_data;
+	EAccountList *account_list;
+	EAccount *account;
+	EIterator *iter;
+
+	g_return_val_if_fail (session != NULL, FALSE);
+
+	/* Load user-defined mail accounts. */
+	account_list = session->priv->account_list;
+	iter = e_list_get_iterator (E_LIST (account_list));
+
+	while (e_iterator_is_valid (iter)) {
+		/* XXX EIterator misuses const. */
+		account = (EAccount *) e_iterator_get (iter);
+
+		mail_session_add_by_account (session, account);
+
+		e_iterator_next (iter);
+	}
+
+	g_object_unref (iter);
+
+	/* Initialize which account is default. */
+
+	account = e_get_default_account ();
+	if (account != NULL) {
+		CamelService *service;
+
+		service = camel_session_get_service (
+			CAMEL_SESSION (session), account->uid);
+		e_mail_account_store_set_default_service (
+			session->priv->account_store, service);
+	}
+
+	return FALSE;
+}
+
 static void
 mail_session_constructed (GObject *object)
 {
@@ -1002,8 +1119,6 @@ mail_session_constructed (GObject *object)
 	GList *list, *link;
 	GSettings *settings;
 	EAccountList *account_list;
-	EIterator *iter;
-	EAccount *account;
 	gulong handler_id;
 
 	session = E_MAIL_SESSION (object);
@@ -1036,32 +1151,13 @@ mail_session_constructed (GObject *object)
 	mail_session_add_local_store (session);
 	mail_session_add_vfolder_store (session);
 
-	/* Load user-defined mail accounts. */
-
-	iter = e_list_get_iterator (E_LIST (account_list));
-
-	while (e_iterator_is_valid (iter)) {
-		/* XXX EIterator misuses const. */
-		account = (EAccount *) e_iterator_get (iter);
-
-		mail_session_add_by_account (session, account);
-
-		e_iterator_next (iter);
-	}
-
-	g_object_unref (iter);
-
-	/* Initialize which account is default. */
-
-	account = e_get_default_account ();
-	if (account != NULL) {
-		CamelService *service;
-
-		service = camel_session_get_service (
-			CAMEL_SESSION (session), account->uid);
-		e_mail_account_store_set_default_service (
-			session->priv->account_store, service);
-	}
+	/* Give it a chance to load user settings, they are not loaded yet.
+	 *
+	 * XXX Is this the case where hiding such natural things like loading
+	 *     user setting into an EExtension strikes back and proves itself
+	 *     being suboptimal?
+	 */
+	g_idle_add (mail_session_initialize_stores_idle, object);
 
 	/* Listen for account list updates. */
 
@@ -1184,6 +1280,8 @@ mail_session_add_service (CamelSession *session,
 			camel_settings_load_from_url (settings, url);
 			camel_url_free (url);
 
+			g_object_notify (G_OBJECT (service), "settings");
+
 			/* Migrate files for this service from its old
 			 * URL-based directory to a UID-based directory
 			 * if necessary. */
@@ -1429,6 +1527,8 @@ mail_session_forward_to (CamelSession *session,
 	CamelFolder *out_folder;
 	CamelMessageInfo *info;
 	CamelMedium *medium;
+	const gchar *from_address;
+	const gchar *from_name;
 	const gchar *header_name;
 	struct _camel_header_raw *xev;
 	gchar *subject;
@@ -1454,6 +1554,9 @@ mail_session_forward_to (CamelSession *session,
 		return FALSE;
 	}
 
+	from_address = account->id->address;
+	from_name = account->id->name;
+
 	forward = camel_mime_message_new ();
 
 	/* make copy of the message, because we are going to modify it */
@@ -1496,8 +1599,7 @@ mail_session_forward_to (CamelSession *session,
 
 	/* from */
 	addr = camel_internet_address_new ();
-	camel_internet_address_add (
-		addr, account->id->name, account->id->address);
+	camel_internet_address_add (addr, from_name, from_address);
 	camel_mime_message_set_from (forward, addr);
 	g_object_unref (addr);
 
@@ -1744,6 +1846,17 @@ e_mail_session_class_init (EMailSessionClass *class)
 			G_PARAM_READWRITE |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_LABEL_STORE,
+		g_param_spec_object (
+			"label-store",
+			"Label Store",
+			"Mail label store",
+			E_TYPE_MAIL_LABEL_LIST_STORE,
+			G_PARAM_READABLE |
+			G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_LOCAL_STORE,
@@ -1781,10 +1894,15 @@ static void
 e_mail_session_init (EMailSession *session)
 {
 	GSettings *settings;
+	GHashTable *junk_filters;
+
+	junk_filters = g_hash_table_new (
+		(GHashFunc) g_str_hash,
+		(GEqualFunc) g_str_equal);
 
 	session->priv = E_MAIL_SESSION_GET_PRIVATE (session);
-	session->priv->junk_filters = g_hash_table_new (
-		(GHashFunc) g_str_hash, (GEqualFunc) g_str_equal);
+	session->priv->label_store = e_mail_label_list_store_new ();
+	session->priv->junk_filters = junk_filters;
 	session->priv->proxy = e_proxy_new ();
 
 	session->priv->local_folders =
@@ -1855,6 +1973,14 @@ e_mail_session_get_folder_cache (EMailSession *session)
 	return session->priv->folder_cache;
 }
 
+EMailLabelListStore *
+e_mail_session_get_label_store (EMailSession *session)
+{
+	g_return_val_if_fail (E_IS_MAIL_SESSION (session), NULL);
+
+	return session->priv->label_store;
+}
+
 CamelStore *
 e_mail_session_get_local_store (EMailSession *session)
 {
