@@ -64,6 +64,10 @@
 #include "mail-config.h"
 #include "mail-mt.h"
 
+#define EM_FORMAT_HTML_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), EM_TYPE_FORMAT_HTML, EMFormatHTMLPrivate))
+
 #define d(x)
 
 #define EFM_MESSAGE_START_ANAME "evolution#message#start"
@@ -325,7 +329,7 @@ static MailMsgInfo efh_format_info = {
 };
 
 static gboolean
-efh_format_timeout (struct _format_msg *m)
+efh_format_helper (struct _format_msg *m, gboolean async)
 {
 	GtkHTMLStream *hstream;
 	EMFormatHTML *efh = m->format;
@@ -339,10 +343,12 @@ efh_format_timeout (struct _format_msg *m)
 		return FALSE;
 	}
 
-	d(printf("timeout called ...\n"));
-	if (p->format_id != -1) {
-		d(printf(" still waiting for cancellation to take effect, waiting ...\n"));
-		return TRUE;
+	if (async) {
+		d(printf("timeout called ...\n"));
+		if (p->format_id != -1) {
+			d(printf(" still waiting for cancellation to take effect, waiting ...\n"));
+			return TRUE;
+		}
 	}
 
 	g_return_val_if_fail (g_queue_is_empty (&p->pending_jobs), FALSE);
@@ -396,7 +402,12 @@ efh_format_timeout (struct _format_msg *m)
 		}
 
 		efh->priv->format_id = m->base.seq;
-		mail_msg_unordered_push (m);
+
+		if (async) {
+			mail_msg_unordered_push (m);
+		} else {
+			efh_format_exec(m, NULL, NULL);
+		}
 	}
 
 	efh->priv->format_timeout_id = 0;
@@ -662,6 +673,33 @@ efh_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+efh_format_timeout (struct _format_msg *m)
+{
+	return efh_format_helper (m, TRUE);
+}
+
+void
+em_format_html_clone_sync (CamelFolder *folder,
+                           const gchar *uid,
+                           CamelMimeMessage *msg,
+                           EMFormatHTML *efh,
+                           EMFormat *source)
+{
+	struct _format_msg *m;
+
+	m = mail_msg_new (&efh_format_info);
+	m->format = g_object_ref (efh);
+	if (source)
+		m->format_source = g_object_ref (source);
+	m->folder = g_object_ref (folder);
+	m->uid = g_strdup (uid);
+	m->message = g_object_ref (msg);
+
+	efh_format_helper (m, FALSE);
+	efh_format_free (m);
+}
+
 static void
 efh_format_clone (EMFormat *emf,
                   CamelFolder *folder,
@@ -811,7 +849,7 @@ efh_busy (EMFormat *emf)
 {
 	EMFormatHTMLPrivate *priv;
 
-	priv = EM_FORMAT_HTML (emf)->priv;
+	priv = EM_FORMAT_HTML_GET_PRIVATE (emf);
 
 	return (priv->format_id != -1);
 }
@@ -1009,8 +1047,7 @@ efh_init (EMFormatHTML *efh,
 	EWebView *web_view;
 	GdkColor *color;
 
-	efh->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		efh, EM_TYPE_FORMAT_HTML, EMFormatHTMLPrivate);
+	efh->priv = EM_FORMAT_HTML_GET_PRIVATE (efh);
 
 	g_queue_init (&efh->pending_object_list);
 	g_queue_init (&efh->priv->pending_jobs);
@@ -3396,3 +3433,4 @@ em_format_html_get_cached_image (EMFormatHTML *efh,
 	return camel_data_cache_get (
 		emfh_http_cache, EMFH_HTTP_CACHE_PATH, image_uri, NULL);
 }
+
