@@ -32,7 +32,6 @@
 #include <e-util/e-util.h>
 #include <e-util/e-account-utils.h>
 #include <e-util/e-util-private.h>
-#include <e-util/gconf-bridge.h>
 
 #include "em-folder-utils.h"
 
@@ -48,17 +47,20 @@
 	((obj), EM_TYPE_SUBSCRIPTION_EDITOR, EMSubscriptionEditorPrivate))
 
 typedef struct _AsyncContext AsyncContext;
+typedef struct _TreeRowData TreeRowData;
 typedef struct _StoreData StoreData;
 
 struct _EMSubscriptionEditorPrivate {
-	CamelSession *session;
+	EMailSession *session;
 	CamelStore *initial_store;
 
 	GtkWidget *combo_box;		/* not referenced */
 	GtkWidget *entry;		/* not referenced */
 	GtkWidget *notebook;		/* not referenced */
 	GtkWidget *subscribe_button;	/* not referenced */
+	GtkWidget *subscribe_arrow;	/* not referenced */
 	GtkWidget *unsubscribe_button;	/* not referenced */
+	GtkWidget *unsubscribe_arrow;	/* not referenced */
 	GtkWidget *collapse_all_button;	/* not referenced */
 	GtkWidget *expand_all_button;	/* not referenced */
 	GtkWidget *refresh_button;	/* not referenced */
@@ -76,12 +78,16 @@ struct _EMSubscriptionEditorPrivate {
 	guint timeout_id;
 };
 
-struct _AsyncContext {
-	EMSubscriptionEditor *editor;
+struct _TreeRowData {
 	CamelFolderInfo *folder_info;
 	GtkTreeRowReference *reference;
 };
 
+struct _AsyncContext {
+	EMSubscriptionEditor *editor;
+	GQueue *tree_rows;
+};
+
 struct _StoreData {
 	CamelStore *store;
 	GtkTreeView *tree_view;
@@ -109,11 +115,39 @@ enum {
 
 G_DEFINE_TYPE (EMSubscriptionEditor, em_subscription_editor, GTK_TYPE_DIALOG)
 
+static void
+tree_row_data_free (TreeRowData *tree_row_data)
+{
+	g_return_if_fail (tree_row_data != NULL);
+
+	gtk_tree_row_reference_free (tree_row_data->reference);
+	g_slice_free (TreeRowData, tree_row_data);
+}
+
+static AsyncContext *
+async_context_new (EMSubscriptionEditor *editor,
+                   GQueue *tree_rows)
+{
+	AsyncContext *context;
+
+	context = g_slice_new0 (AsyncContext);
+	context->editor = g_object_ref (editor);
+
+	/* Transfer GQueue contents. */
+	context->tree_rows = g_queue_copy (tree_rows);
+	g_queue_clear (tree_rows);
+
+	return context;
+}
+
 static void
 async_context_free (AsyncContext *context)
 {
+	while (!g_queue_is_empty (context->tree_rows))
+		tree_row_data_free (g_queue_pop_head (context->tree_rows));
+
 	g_object_unref (context->editor);
-	gtk_tree_row_reference_free (context->reference);
+	g_queue_free (context->tree_rows);
 
 	g_slice_free (AsyncContext, context);
 }
@@ -146,13 +180,12 @@ store_data_free (StoreData *data)
 static void
 subscription_editor_populate (EMSubscriptionEditor *editor,
                               CamelFolderInfo *folder_info,
-                              GtkTreeIter *parent)
+                              GtkTreeIter *parent,
+                              GList **expand_paths)
 {
-	GtkTreeView *tree_view;
 	GtkListStore *list_store;
 	GtkTreeStore *tree_store;
 
-	tree_view = editor->priv->active->tree_view;
 	list_store = GTK_LIST_STORE (editor->priv->active->list_store);
 	tree_store = GTK_TREE_STORE (editor->priv->active->tree_store);
 
@@ -189,20 +222,26 @@ subscription_editor_populate (EMSubscriptionEditor *editor,
 
 			path = gtk_tree_model_get_path (
 				GTK_TREE_MODEL (tree_store), &iter);
-			gtk_tree_view_expand_to_path (tree_view, path);
-			gtk_tree_path_free (path);
+			*expand_paths = g_list_prepend (*expand_paths, path);
 		}
 
 		g_free (casefolded);
 
 		if (folder_info->child != NULL)
 			subscription_editor_populate (
-				editor, folder_info->child, &iter);
+				editor, folder_info->child, &iter, expand_paths);
 
 		folder_info = folder_info->next;
 	}
 }
 
+static void
+expand_paths_cb (gpointer path,
+                 gpointer tree_view)
+{
+	gtk_tree_view_expand_to_path (tree_view, path);
+}
+
 static void
 subscription_editor_get_folder_info_done (CamelStore *store,
                                           GAsyncResult *result,
@@ -212,9 +251,11 @@ subscription_editor_get_folder_info_done (CamelStore *store,
 	GtkTreeView *tree_view;
 	GtkTreeModel *list_store;
 	GtkTreeModel *tree_store;
+	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	CamelFolderInfo *folder_info;
 	GdkWindow *window;
+	GList *expand_paths = NULL;
 	GError *error = NULL;
 
 	folder_info = camel_store_get_folder_info_finish (
@@ -253,7 +294,15 @@ subscription_editor_get_folder_info_done (CamelStore *store,
 
 	gtk_list_store_clear (GTK_LIST_STORE (list_store));
 	gtk_tree_store_clear (GTK_TREE_STORE (tree_store));
-	subscription_editor_populate (editor, folder_info, NULL);
+
+	model = gtk_tree_view_get_model (tree_view);
+	gtk_tree_view_set_model (tree_view, NULL);
+	subscription_editor_populate (editor, folder_info, NULL, &expand_paths);
+	gtk_tree_view_set_model (tree_view, model);
+
+	g_list_foreach (expand_paths, expand_paths_cb, tree_view);
+	g_list_foreach (expand_paths, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (expand_paths);
 
 	path = gtk_tree_path_new_first ();
 	selection = gtk_tree_view_get_selection (tree_view);
@@ -267,7 +316,7 @@ exit:
 }
 
 static void
-subscription_editor_subscribe_folder_done (CamelStore *store,
+subscription_editor_subscribe_folder_done (CamelSubscribable *subscribable,
                                            GAsyncResult *result,
                                            AsyncContext *context)
 {
@@ -278,8 +327,10 @@ subscription_editor_subscribe_folder_done (CamelStore *store,
 	GtkTreeIter iter;
 	GdkWindow *window;
 	GError *error = NULL;
+	TreeRowData *tree_row_data;
 
-	camel_store_subscribe_folder_finish (store, result, &error);
+	camel_subscribable_subscribe_folder_finish (
+		subscribable, result, &error);
 
 	/* Just return quietly if we were cancelled. */
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
@@ -287,14 +338,44 @@ subscription_editor_subscribe_folder_done (CamelStore *store,
 		goto exit;
 	}
 
+	tree_row_data = g_queue_pop_head (context->tree_rows);
+
 	/* XXX Do something smarter with errors. */
 	if (error == NULL)
-		context->folder_info->flags |= CAMEL_FOLDER_SUBSCRIBED;
+		tree_row_data->folder_info->flags |= CAMEL_FOLDER_SUBSCRIBED;
 	else {
 		g_warning ("%s", error->message);
 		g_error_free (error);
+		tree_row_data_free (tree_row_data);
+		goto exit;
 	}
 
+	/* Update the toggle renderer in the selected row. */
+	tree_model = gtk_tree_row_reference_get_model (tree_row_data->reference);
+	path = gtk_tree_row_reference_get_path (tree_row_data->reference);
+	gtk_tree_model_get_iter (tree_model, &iter, path);
+	gtk_tree_model_row_changed (tree_model, path, &iter);
+	gtk_tree_path_free (path);
+
+	tree_row_data_free (tree_row_data);
+
+	if (!g_queue_is_empty (context->tree_rows)) {
+		GCancellable *cancellable;
+
+		/* continue with the next to subscribe */
+		tree_row_data = g_queue_peek_head (context->tree_rows);
+		g_return_if_fail (tree_row_data != NULL);
+
+		cancellable = context->editor->priv->active->cancellable;
+
+		camel_subscribable_subscribe_folder (
+			subscribable, tree_row_data->folder_info->full_name,
+			G_PRIORITY_DEFAULT, cancellable, (GAsyncReadyCallback)
+			subscription_editor_subscribe_folder_done, context);
+		return;
+	}
+
+exit:
 	gtk_widget_set_sensitive (context->editor->priv->notebook, TRUE);
 	gtk_widget_set_sensitive (context->editor->priv->refresh_button, TRUE);
 	gtk_widget_set_sensitive (context->editor->priv->stop_button, FALSE);
@@ -307,19 +388,63 @@ subscription_editor_subscribe_folder_done (CamelStore *store,
 	selection = gtk_tree_view_get_selection (tree_view);
 	g_signal_emit_by_name (selection, "changed");
 
-	/* Update the toggle renderer in the selected row. */
-	tree_model = gtk_tree_row_reference_get_model (context->reference);
-	path = gtk_tree_row_reference_get_path (context->reference);
-	gtk_tree_model_get_iter (tree_model, &iter, path);
-	gtk_tree_model_row_changed (tree_model, path, &iter);
-	gtk_tree_path_free (path);
-
-exit:
 	async_context_free (context);
+
+	gtk_widget_grab_focus (GTK_WIDGET (tree_view));
 }
 
 static void
-subscription_editor_unsubscribe_folder_done (CamelStore *store,
+subscription_editor_subscribe_many (EMSubscriptionEditor *editor,
+                                    GQueue *tree_rows)
+{
+	TreeRowData *tree_row_data;
+	GCancellable *cancellable;
+	CamelStore *active_store;
+	AsyncContext *context;
+	GdkCursor *cursor;
+	GdkWindow *window;
+
+	g_return_if_fail (editor != NULL);
+
+	if (g_queue_is_empty (tree_rows))
+		return;
+
+	tree_row_data = g_queue_peek_head (tree_rows);
+	g_return_if_fail (tree_row_data != NULL);
+
+	/* Cancel any operation on this store still in progress. */
+	gtk_button_clicked (GTK_BUTTON (editor->priv->stop_button));
+
+	/* Start a new 'subscription' operation. */
+	editor->priv->active->cancellable = g_cancellable_new ();
+
+	gtk_widget_set_sensitive (editor->priv->notebook, FALSE);
+	gtk_widget_set_sensitive (editor->priv->subscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->subscribe_arrow, FALSE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_arrow, FALSE);
+	gtk_widget_set_sensitive (editor->priv->refresh_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->stop_button, TRUE);
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	window = gtk_widget_get_window (GTK_WIDGET (editor));
+	gdk_window_set_cursor (window, cursor);
+	g_object_unref (cursor);
+
+	context = async_context_new (editor, tree_rows);
+
+	active_store = editor->priv->active->store;
+	cancellable = editor->priv->active->cancellable;
+
+	camel_subscribable_subscribe_folder (
+		CAMEL_SUBSCRIBABLE (active_store),
+		tree_row_data->folder_info->full_name,
+		G_PRIORITY_DEFAULT, cancellable, (GAsyncReadyCallback)
+		subscription_editor_subscribe_folder_done, context);
+}
+
+static void
+subscription_editor_unsubscribe_folder_done (CamelSubscribable *subscribable,
                                              GAsyncResult *result,
                                              AsyncContext *context)
 {
@@ -330,8 +455,10 @@ subscription_editor_unsubscribe_folder_done (CamelStore *store,
 	GtkTreeIter iter;
 	GdkWindow *window;
 	GError *error = NULL;
+	TreeRowData *tree_row_data;
 
-	camel_store_unsubscribe_folder_finish (store, result, &error);
+	camel_subscribable_unsubscribe_folder_finish (
+		subscribable, result, &error);
 
 	/* Just return quietly if we were cancelled. */
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
@@ -339,14 +466,44 @@ subscription_editor_unsubscribe_folder_done (CamelStore *store,
 		goto exit;
 	}
 
+	tree_row_data = g_queue_pop_head (context->tree_rows);
+
 	/* XXX Do something smarter with errors. */
 	if (error == NULL)
-		context->folder_info->flags &= ~CAMEL_FOLDER_SUBSCRIBED;
+		tree_row_data->folder_info->flags &= ~CAMEL_FOLDER_SUBSCRIBED;
 	else {
 		g_warning ("%s", error->message);
 		g_error_free (error);
+		tree_row_data_free (tree_row_data);
+		goto exit;
 	}
 
+	/* Update the toggle renderer in the selected row. */
+	tree_model = gtk_tree_row_reference_get_model (tree_row_data->reference);
+	path = gtk_tree_row_reference_get_path (tree_row_data->reference);
+	gtk_tree_model_get_iter (tree_model, &iter, path);
+	gtk_tree_model_row_changed (tree_model, path, &iter);
+	gtk_tree_path_free (path);
+
+	tree_row_data_free (tree_row_data);
+
+	if (!g_queue_is_empty (context->tree_rows)) {
+		GCancellable *cancellable;
+
+		/* continue with the next to unsubscribe */
+		tree_row_data = g_queue_peek_head (context->tree_rows);
+		g_return_if_fail (tree_row_data != NULL);
+
+		cancellable = context->editor->priv->active->cancellable;
+
+		camel_subscribable_unsubscribe_folder (
+			subscribable, tree_row_data->folder_info->full_name,
+			G_PRIORITY_DEFAULT, cancellable, (GAsyncReadyCallback)
+			subscription_editor_unsubscribe_folder_done, context);
+		return;
+	}
+
+exit:
 	gtk_widget_set_sensitive (context->editor->priv->notebook, TRUE);
 	gtk_widget_set_sensitive (context->editor->priv->refresh_button, TRUE);
 	gtk_widget_set_sensitive (context->editor->priv->stop_button, FALSE);
@@ -359,38 +516,28 @@ subscription_editor_unsubscribe_folder_done (CamelStore *store,
 	selection = gtk_tree_view_get_selection (tree_view);
 	g_signal_emit_by_name (selection, "changed");
 
-	/* Update the toggle renderer in the selected row. */
-	tree_model = gtk_tree_row_reference_get_model (context->reference);
-	path = gtk_tree_row_reference_get_path (context->reference);
-	gtk_tree_model_get_iter (tree_model, &iter, path);
-	gtk_tree_model_row_changed (tree_model, path, &iter);
-	gtk_tree_path_free (path);
-
-exit:
 	async_context_free (context);
+
+	gtk_widget_grab_focus (GTK_WIDGET (tree_view));
 }
 
 static void
-subscription_editor_subscribe (EMSubscriptionEditor *editor)
+subscription_editor_unsubscribe_many (EMSubscriptionEditor *editor,
+                                      GQueue *tree_rows)
 {
-	CamelFolderInfo *folder_info;
-	GtkTreeRowReference *reference;
-	GtkTreeSelection *selection;
-	GtkTreeModel *tree_model;
-	GtkTreeView *tree_view;
-	GtkTreePath *path;
-	GtkTreeIter iter;
+	TreeRowData *tree_row_data;
+	AsyncContext *context;
+	CamelStore *active_store;
 	GdkCursor *cursor;
 	GdkWindow *window;
-	AsyncContext *context;
-	gboolean have_selection;
 
-	tree_view = editor->priv->active->tree_view;
-	selection = gtk_tree_view_get_selection (tree_view);
+	g_return_if_fail (editor != NULL);
 
-	have_selection = gtk_tree_selection_get_selected (
-		selection, &tree_model, &iter);
-	g_return_if_fail (have_selection);
+	if (g_queue_is_empty (tree_rows))
+		return;
+
+	tree_row_data = g_queue_peek_head (tree_rows);
+	g_return_if_fail (tree_row_data != NULL);
 
 	/* Cancel any operation on this store still in progress. */
 	gtk_button_clicked (GTK_BUTTON (editor->priv->stop_button));
@@ -400,7 +547,9 @@ subscription_editor_subscribe (EMSubscriptionEditor *editor)
 
 	gtk_widget_set_sensitive (editor->priv->notebook, FALSE);
 	gtk_widget_set_sensitive (editor->priv->subscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->subscribe_arrow, FALSE);
 	gtk_widget_set_sensitive (editor->priv->unsubscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_arrow, FALSE);
 	gtk_widget_set_sensitive (editor->priv->refresh_button, FALSE);
 	gtk_widget_set_sensitive (editor->priv->stop_button, TRUE);
 
@@ -409,39 +558,243 @@ subscription_editor_subscribe (EMSubscriptionEditor *editor)
 	gdk_window_set_cursor (window, cursor);
 	g_object_unref (cursor);
 
-	gtk_tree_model_get (
-		tree_model, &iter, COL_FOLDER_INFO, &folder_info, -1);
+	context = async_context_new (editor, tree_rows);
 
-	path = gtk_tree_model_get_path (tree_model, &iter);
-	reference = gtk_tree_row_reference_new (tree_model, path);
-	gtk_tree_path_free (path);
+	active_store = editor->priv->active->store;
 
-	context = g_slice_new0 (AsyncContext);
-	context->editor = g_object_ref (editor);
-	context->folder_info = folder_info;
-	context->reference = reference;
-
-	camel_store_subscribe_folder (
-		editor->priv->active->store,
-		folder_info->full_name, G_PRIORITY_DEFAULT,
+	camel_subscribable_unsubscribe_folder (
+		CAMEL_SUBSCRIBABLE (active_store),
+		tree_row_data->folder_info->full_name, G_PRIORITY_DEFAULT,
 		editor->priv->active->cancellable, (GAsyncReadyCallback)
-		subscription_editor_subscribe_folder_done, context);
+		subscription_editor_unsubscribe_folder_done, context);
+}
+
+static GtkWidget *
+subscription_editor_create_menu_item (const gchar *label,
+                                      gboolean sensitive,
+                                      GCallback activate_cb,
+                                      EMSubscriptionEditor *editor)
+{
+	GtkWidget *item;
+
+	item = gtk_menu_item_new_with_mnemonic (label);
+	gtk_widget_set_sensitive (item, sensitive);
+
+	gtk_widget_show (item);
+
+	g_signal_connect_swapped (item, "activate", activate_cb, editor);
+
+	return item;
 }
 
 static void
-subscription_editor_unsubscribe (EMSubscriptionEditor *editor)
+position_below_widget_cb (GtkMenu *menu,
+                          gint *x,
+                          gint *y,
+                          gboolean *push_in,
+                          gpointer under_widget)
 {
-	CamelFolderInfo *folder_info;
+	GtkRequisition menu_requisition;
+	GtkTextDirection direction;
+	GtkAllocation allocation;
+	GdkRectangle monitor;
+	GdkScreen *screen;
+	GdkWindow *window;
+	GtkWidget *widget;
+	gint monitor_num;
+
+	widget = under_widget;
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &menu_requisition, NULL);
+
+	window = gtk_widget_get_parent_window (widget);
+	screen = gtk_widget_get_screen (GTK_WIDGET (menu));
+	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+	if (monitor_num < 0)
+		monitor_num = 0;
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+	gtk_widget_get_allocation (widget, &allocation);
+
+	gdk_window_get_origin (window, x, y);
+	*x += allocation.x;
+	*y += allocation.y + 2 + gtk_widget_get_allocated_height (under_widget);
+
+	direction = gtk_widget_get_direction (widget);
+	if (direction == GTK_TEXT_DIR_LTR)
+		*x += MAX (allocation.width - menu_requisition.width, 0);
+	else if (menu_requisition.width > allocation.width)
+		*x -= menu_requisition.width - allocation.width;
+
+	*push_in = FALSE;
+}
+
+static TreeRowData *
+subscription_editor_tree_row_data_from_iter (GtkTreeView *tree_view,
+                                             GtkTreeModel *model,
+                                             GtkTreeIter *iter,
+                                             gboolean *is_expanded)
+{
+	TreeRowData *tree_row_data;
+	CamelFolderInfo *folder_info = NULL;
 	GtkTreeRowReference *reference;
+	GtkTreePath *path;
+
+	gtk_tree_model_get (
+		model, iter, COL_FOLDER_INFO, &folder_info, -1);
+
+	if (!FOLDER_CAN_SELECT (folder_info))
+		return NULL;
+
+	path = gtk_tree_model_get_path (model, iter);
+	reference = gtk_tree_row_reference_new (model, path);
+	if (is_expanded)
+		*is_expanded = gtk_tree_view_row_expanded (tree_view, path);
+	gtk_tree_path_free (path);
+
+	tree_row_data = g_slice_new0 (TreeRowData);
+	tree_row_data->folder_info = folder_info;
+	tree_row_data->reference = reference;
+
+	return tree_row_data;
+}
+
+typedef enum {
+	PICK_ALL,
+	PICK_SUBSCRIBED,
+	PICK_UNSUBSCRIBED
+} EPickMode;
+
+static gboolean
+can_pick_folder_info (CamelFolderInfo *fi,
+                      EPickMode mode)
+{
+	if (!FOLDER_CAN_SELECT (fi))
+		return FALSE;
+
+	if (mode == PICK_ALL)
+		return TRUE;
+
+	return (FOLDER_SUBSCRIBED (fi) ? 1 : 0) == (mode == PICK_SUBSCRIBED ? 1 : 0);
+}
+
+struct PickAllData {
+	GtkTreeView *tree_view;
+	EPickMode mode;
+	GHashTable *skip_folder_infos;
+	GQueue *out_tree_rows;
+};
+
+static gboolean
+pick_all_cb (GtkTreeModel *model,
+             GtkTreePath *path,
+             GtkTreeIter *iter,
+             gpointer user_data)
+{
+	struct PickAllData *data = user_data;
+	TreeRowData *tree_row_data;
+
+	tree_row_data = subscription_editor_tree_row_data_from_iter (
+		data->tree_view, model, iter, NULL);
+	if (tree_row_data == NULL)
+		return FALSE;
+
+	if (can_pick_folder_info (tree_row_data->folder_info, data->mode) &&
+	    (data->skip_folder_infos == NULL ||
+	    !g_hash_table_lookup_extended (data->skip_folder_infos, tree_row_data->folder_info, NULL, NULL))) {
+		g_queue_push_tail (data->out_tree_rows, tree_row_data);
+	} else
+		tree_row_data_free (tree_row_data);
+
+	return FALSE;
+}
+
+/* skip_folder_infos contains CamelFolderInfo-s to skip;
+ * these should come from the tree view; can be NULL
+ * to include everything.
+*/
+static void
+subscription_editor_pick_all (EMSubscriptionEditor *editor,
+                              EPickMode mode,
+                              GHashTable *skip_folder_infos,
+                              GQueue *out_tree_rows)
+{
+	GtkTreeView *tree_view;
+	GtkTreeModel *tree_model;
+	struct PickAllData data;
+
+	tree_view = editor->priv->active->tree_view;
+	tree_model = gtk_tree_view_get_model (tree_view);
+
+	data.tree_view = tree_view;
+	data.mode = mode;
+	data.skip_folder_infos = skip_folder_infos;
+	data.out_tree_rows = out_tree_rows;
+
+	gtk_tree_model_foreach (tree_model, pick_all_cb, &data);
+}
+
+static void
+subscription_editor_pick_shown (EMSubscriptionEditor *editor,
+                                EPickMode mode,
+                                GQueue *out_tree_rows)
+{
+	GtkTreeView *tree_view;
+	GtkTreeModel *tree_model;
+	GtkTreeIter iter, iter2;
+	gboolean found = TRUE;
+
+	tree_view = editor->priv->active->tree_view;
+	tree_model = gtk_tree_view_get_model (tree_view);
+
+	if (!gtk_tree_model_get_iter_first (tree_model, &iter))
+		return;
+
+	while (found) {
+		TreeRowData *tree_row_data;
+		gboolean is_expanded = FALSE;
+
+		found = FALSE;
+		tree_row_data = subscription_editor_tree_row_data_from_iter (
+			tree_view, tree_model, &iter, &is_expanded);
+
+		if (tree_row_data != NULL) {
+			if (can_pick_folder_info (tree_row_data->folder_info, mode))
+				g_queue_push_tail (out_tree_rows, tree_row_data);
+			else
+				tree_row_data_free (tree_row_data);
+		}
+
+		if (is_expanded && gtk_tree_model_iter_children (tree_model, &iter2, &iter)) {
+			iter = iter2;
+			found = TRUE;
+		} else {
+			iter2 = iter;
+			if (gtk_tree_model_iter_next (tree_model, &iter2)) {
+				iter = iter2;
+				found = TRUE;
+			} else {
+				while (found = gtk_tree_model_iter_parent (tree_model, &iter2, &iter), found) {
+					iter = iter2;
+					if (gtk_tree_model_iter_next (tree_model, &iter2)) {
+						iter = iter2;
+						break;
+					}
+				}
+			}
+		}
+	}
+}
+
+static void
+subscription_editor_subscribe (EMSubscriptionEditor *editor)
+{
 	GtkTreeSelection *selection;
 	GtkTreeModel *tree_model;
 	GtkTreeView *tree_view;
-	GtkTreePath *path;
 	GtkTreeIter iter;
-	GdkCursor *cursor;
-	GdkWindow *window;
-	AsyncContext *context;
 	gboolean have_selection;
+	GQueue tree_rows = G_QUEUE_INIT;
+	TreeRowData *tree_row_data;
 
 	tree_view = editor->priv->active->tree_view;
 	selection = gtk_tree_view_get_selection (tree_view);
@@ -450,40 +803,197 @@ subscription_editor_unsubscribe (EMSubscriptionEditor *editor)
 		selection, &tree_model, &iter);
 	g_return_if_fail (have_selection);
 
-	/* Cancel any operation on this store still in progress. */
-	gtk_button_clicked (GTK_BUTTON (editor->priv->stop_button));
+	tree_row_data = subscription_editor_tree_row_data_from_iter (
+		tree_view, tree_model, &iter, NULL);
 
-	/* Start a new 'unsubscription' operation. */
-	editor->priv->active->cancellable = g_cancellable_new ();
+	g_queue_push_tail (&tree_rows, tree_row_data);
+	subscription_editor_subscribe_many (editor, &tree_rows);
+	g_warn_if_fail (g_queue_is_empty (&tree_rows));
+}
 
-	gtk_widget_set_sensitive (editor->priv->notebook, FALSE);
-	gtk_widget_set_sensitive (editor->priv->subscribe_button, FALSE);
-	gtk_widget_set_sensitive (editor->priv->unsubscribe_button, FALSE);
-	gtk_widget_set_sensitive (editor->priv->refresh_button, FALSE);
-	gtk_widget_set_sensitive (editor->priv->stop_button, TRUE);
+static void
+subscription_editor_subscribe_shown (EMSubscriptionEditor *editor)
+{
+	GQueue tree_rows = G_QUEUE_INIT;
 
-	cursor = gdk_cursor_new (GDK_WATCH);
-	window = gtk_widget_get_window (GTK_WIDGET (editor));
-	gdk_window_set_cursor (window, cursor);
-	g_object_unref (cursor);
+	subscription_editor_pick_shown (
+		editor, PICK_UNSUBSCRIBED, &tree_rows);
+	subscription_editor_subscribe_many (editor, &tree_rows);
+}
 
-	gtk_tree_model_get (
-		tree_model, &iter, COL_FOLDER_INFO, &folder_info, -1);
+static void
+subscription_editor_subscribe_all (EMSubscriptionEditor *editor)
+{
+	GQueue tree_rows = G_QUEUE_INIT;
 
-	path = gtk_tree_model_get_path (tree_model, &iter);
-	reference = gtk_tree_row_reference_new (tree_model, path);
-	gtk_tree_path_free (path);
+	subscription_editor_pick_all (
+		editor, PICK_UNSUBSCRIBED, NULL, &tree_rows);
+	subscription_editor_subscribe_many (editor, &tree_rows);
+}
 
-	context = g_slice_new0 (AsyncContext);
-	context->editor = g_object_ref (editor);
-	context->folder_info = folder_info;
-	context->reference = reference;
+static void
+subscription_editor_subscribe_popup_cb (EMSubscriptionEditor *editor)
+{
+	GtkWidget *menu;
+	GtkTreeIter iter;
+	gboolean tree_filled;
 
-	camel_store_unsubscribe_folder (
-		editor->priv->active->store,
-		folder_info->full_name, G_PRIORITY_DEFAULT,
-		editor->priv->active->cancellable, (GAsyncReadyCallback)
-		subscription_editor_unsubscribe_folder_done, context);
+	tree_filled = editor->priv->active &&
+		gtk_tree_model_get_iter_first (
+			editor->priv->active->filtered_view
+			? editor->priv->active->list_store
+			: editor->priv->active->tree_store,
+			&iter);
+
+	menu = gtk_menu_new ();
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("_Subscribe"),
+			gtk_widget_get_sensitive (
+				editor->priv->subscribe_button),
+			G_CALLBACK (subscription_editor_subscribe),
+			editor));
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("Su_bscribe To Shown"),
+			tree_filled,
+			G_CALLBACK (subscription_editor_subscribe_shown),
+			editor));
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("Subscribe To _All"),
+			tree_filled,
+			G_CALLBACK (subscription_editor_subscribe_all),
+			editor));
+
+	gtk_menu_popup (
+		GTK_MENU (menu), NULL, NULL,
+		position_below_widget_cb,
+		editor->priv->subscribe_button,
+		0, gtk_get_current_event_time ());
+}
+
+static void
+subscription_editor_unsubscribe_hidden (EMSubscriptionEditor *editor)
+{
+	GQueue tree_rows = G_QUEUE_INIT;
+	GHashTable *skip_shown;
+
+	subscription_editor_pick_shown (editor, PICK_ALL, &tree_rows);
+	g_return_if_fail (!g_queue_is_empty (&tree_rows));
+
+	skip_shown = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+	while (!g_queue_is_empty (&tree_rows)) {
+		TreeRowData *tree_row_data;
+
+		tree_row_data = g_queue_pop_head (&tree_rows);
+
+		if (tree_row_data == NULL)
+			continue;
+
+		g_hash_table_insert (
+			skip_shown,
+			tree_row_data->folder_info,
+			GINT_TO_POINTER (1));
+
+		tree_row_data_free (tree_row_data);
+	}
+
+	subscription_editor_pick_all (
+		editor, PICK_SUBSCRIBED, skip_shown, &tree_rows);
+	subscription_editor_unsubscribe_many (editor, &tree_rows);
+
+	g_hash_table_destroy (skip_shown);
+}
+
+static void
+subscription_editor_unsubscribe_all (EMSubscriptionEditor *editor)
+{
+	GQueue tree_rows = G_QUEUE_INIT;
+
+	subscription_editor_pick_all (
+		editor, PICK_SUBSCRIBED, NULL, &tree_rows);
+	subscription_editor_unsubscribe_many (editor, &tree_rows);
+}
+
+static void
+subscription_editor_unsubscribe (EMSubscriptionEditor *editor)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *tree_model;
+	GtkTreeView *tree_view;
+	GtkTreeIter iter;
+	gboolean have_selection;
+	GQueue tree_rows = G_QUEUE_INIT;
+	TreeRowData *tree_row_data;
+
+	tree_view = editor->priv->active->tree_view;
+	selection = gtk_tree_view_get_selection (tree_view);
+
+	have_selection = gtk_tree_selection_get_selected (
+		selection, &tree_model, &iter);
+	g_return_if_fail (have_selection);
+
+	tree_row_data = subscription_editor_tree_row_data_from_iter (
+		tree_view, tree_model, &iter, NULL);
+
+	g_queue_push_tail (&tree_rows, tree_row_data);
+	subscription_editor_unsubscribe_many (editor, &tree_rows);
+}
+
+static void
+subscription_editor_unsubscribe_popup_cb (EMSubscriptionEditor *editor)
+{
+	GtkWidget *menu;
+	GtkTreeIter iter;
+	gboolean tree_filled;
+
+	tree_filled = editor->priv->active &&
+		gtk_tree_model_get_iter_first (
+			editor->priv->active->filtered_view
+			? editor->priv->active->list_store
+			: editor->priv->active->tree_store,
+			&iter);
+
+	menu = gtk_menu_new ();
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("_Unsubscribe"),
+			gtk_widget_get_sensitive (
+				editor->priv->unsubscribe_button),
+			G_CALLBACK (subscription_editor_unsubscribe),
+			editor));
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("Unsu_bscribe From Hidden"),
+			tree_filled,
+			G_CALLBACK (subscription_editor_unsubscribe_hidden),
+			editor));
+
+	gtk_menu_shell_append (
+		GTK_MENU_SHELL (menu),
+		subscription_editor_create_menu_item (
+			_("Unsubscribe From _All"),
+			tree_filled,
+			G_CALLBACK (subscription_editor_unsubscribe_all),
+			editor));
+
+	gtk_menu_popup (
+		GTK_MENU (menu), NULL, NULL,
+		position_below_widget_cb,
+		editor->priv->unsubscribe_button,
+		0, gtk_get_current_event_time ());
 }
 
 static void
@@ -512,7 +1022,9 @@ subscription_editor_refresh (EMSubscriptionEditor *editor)
 
 	gtk_widget_set_sensitive (editor->priv->notebook, FALSE);
 	gtk_widget_set_sensitive (editor->priv->subscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->subscribe_arrow, FALSE);
 	gtk_widget_set_sensitive (editor->priv->unsubscribe_button, FALSE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_arrow, FALSE);
 	gtk_widget_set_sensitive (editor->priv->refresh_button, FALSE);
 	gtk_widget_set_sensitive (editor->priv->stop_button, TRUE);
 
@@ -544,9 +1056,12 @@ subscription_editor_stop (EMSubscriptionEditor *editor)
 
 	gtk_widget_set_sensitive (editor->priv->notebook, TRUE);
 	gtk_widget_set_sensitive (editor->priv->subscribe_button, TRUE);
+	gtk_widget_set_sensitive (editor->priv->subscribe_arrow, TRUE);
 	gtk_widget_set_sensitive (editor->priv->unsubscribe_button, TRUE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_arrow, TRUE);
 	gtk_widget_set_sensitive (editor->priv->refresh_button, TRUE);
 	gtk_widget_set_sensitive (editor->priv->stop_button, FALSE);
+	gtk_widget_grab_focus (GTK_WIDGET (editor->priv->active->tree_view));
 
 	window = gtk_widget_get_window (GTK_WIDGET (editor));
 	gdk_window_set_cursor (window, NULL);
@@ -788,8 +1303,11 @@ subscription_editor_selection_changed_cb (GtkTreeSelection *selection,
 		gtk_widget_set_sensitive (
 			editor->priv->subscribe_button, FALSE);
 		gtk_widget_set_sensitive (
-			editor->priv->subscribe_button, FALSE);
+			editor->priv->unsubscribe_button, FALSE);
 	}
+
+	gtk_widget_set_sensitive (editor->priv->subscribe_arrow, TRUE);
+	gtk_widget_set_sensitive (editor->priv->unsubscribe_arrow, TRUE);
 }
 
 static void
@@ -797,7 +1315,7 @@ subscription_editor_add_store (EMSubscriptionEditor *editor,
                                CamelStore *store)
 {
 	StoreData *data;
-	EAccount *account;
+	CamelService *service;
 	GtkListStore *list_store;
 	GtkTreeStore *tree_store;
 	GtkTreeViewColumn *column;
@@ -806,17 +1324,13 @@ subscription_editor_add_store (EMSubscriptionEditor *editor,
 	GtkComboBoxText *combo_box;
 	GtkWidget *container;
 	GtkWidget *widget;
-	const gchar *uid;
+	const gchar *display_name;
 
-	/* Neither of the built-in stores ("local" or "vfolder") support
-	 * folder subscriptions.  Therefore there should be a corresponding
-	 * EAccount for the store from which we can grab a display name. */
-	uid = camel_service_get_uid (CAMEL_SERVICE (store));
-	account = e_get_account_by_uid (uid);
-	g_return_if_fail (account != NULL);
+	service = CAMEL_SERVICE (store);
+	display_name = camel_service_get_display_name (service);
 
 	combo_box = GTK_COMBO_BOX_TEXT (editor->priv->combo_box);
-	gtk_combo_box_text_append_text (combo_box, account->name);
+	gtk_combo_box_text_append_text (combo_box, display_name);
 
 	tree_store = gtk_tree_store_new (
 		N_COLUMNS,
@@ -851,6 +1365,7 @@ subscription_editor_add_store (EMSubscriptionEditor *editor,
 	widget = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (widget), TRUE);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (widget), FALSE);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (widget), TRUE);
 	gtk_tree_view_set_search_column (
 		GTK_TREE_VIEW (widget), COL_FOLDER_NAME);
 	gtk_container_add (GTK_CONTAINER (container), widget);
@@ -908,15 +1423,15 @@ subscription_editor_set_store (EMSubscriptionEditor *editor,
 {
 	g_return_if_fail (editor->priv->initial_store == NULL);
 
-	if (CAMEL_IS_STORE (store))
+	if (CAMEL_IS_SUBSCRIBABLE (store))
 		editor->priv->initial_store = g_object_ref (store);
 }
 
 static void
 subscription_editor_set_session (EMSubscriptionEditor *editor,
-                                 CamelSession *session)
+                                 EMailSession *session)
 {
-	g_return_if_fail (CAMEL_IS_SESSION (session));
+	g_return_if_fail (E_IS_MAIL_SESSION (session));
 	g_return_if_fail (editor->priv->session == NULL);
 
 	editor->priv->session = g_object_ref (session);
@@ -1020,16 +1535,21 @@ subscription_editor_constructed (GObject *object)
 
 	editor = EM_SUBSCRIPTION_EDITOR (object);
 
+	/* Pick an initial store based on the default mail account, if
+	 * one wasn't already given in em_subscription_editor_new(). */
 	if (editor->priv->initial_store == NULL) {
 		EAccount *account;
 		CamelService *service;
-		CamelSession *session;
+		EMailSession *session;
 
 		account = e_get_default_account ();
-		session = em_subscription_editor_get_session (editor);
-		service = camel_session_get_service (session, account->uid);
 
-		if (CAMEL_IS_STORE (service))
+		session = em_subscription_editor_get_session (editor);
+
+		service = camel_session_get_service (
+			CAMEL_SESSION (session), account->uid);
+
+		if (CAMEL_IS_SUBSCRIBABLE (service))
 			editor->priv->initial_store = g_object_ref (service);
 	}
 
@@ -1059,7 +1579,7 @@ subscription_editor_realize (GtkWidget *widget)
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		CamelStore *store = CAMEL_STORE (link->data);
 
-		if (!camel_store_supports_subscriptions (store))
+		if (!CAMEL_IS_SUBSCRIBABLE (store))
 			continue;
 
 		if (store == editor->priv->initial_store)
@@ -1103,7 +1623,7 @@ em_subscription_editor_class_init (EMSubscriptionEditorClass *class)
 			"session",
 			NULL,
 			NULL,
-			CAMEL_TYPE_SESSION,
+			E_TYPE_MAIL_SESSION,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT_ONLY |
 			G_PARAM_STATIC_STRINGS));
@@ -1136,11 +1656,12 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 
 	gtk_container_set_border_width (GTK_CONTAINER (editor), 5);
 	gtk_window_set_title (GTK_WINDOW (editor), _("Folder Subscriptions"));
+	gtk_window_set_default_size (GTK_WINDOW (editor), 600, 400);
 
-	gconf_bridge_bind_window_size (
-		gconf_bridge_get (),
-		"/apps/evolution/mail/subscription_editor",
-		GTK_WINDOW (editor));
+	e_restore_window (
+		GTK_WINDOW (editor),
+		"/org/gnome/evolution/mail/subscription-window/",
+		E_RESTORE_WINDOW_SIZE);
 
 	gtk_dialog_add_button (
 		GTK_DIALOG (editor),
@@ -1237,20 +1758,25 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
-	widget = gtk_vbutton_box_new ();
+	widget = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
 	gtk_box_set_spacing (GTK_BOX (widget), 6);
 	gtk_button_box_set_layout (
 		GTK_BUTTON_BOX (widget), GTK_BUTTONBOX_START);
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, TRUE, 0);
 	gtk_widget_show (widget);
 
+	container = box = widget;
+
+	widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, TRUE, 0);
+	gtk_widget_show (widget);
+
 	container = widget;
 
 	tooltip = _("Subscribe to the selected folder");
 	widget = gtk_button_new_with_mnemonic (_("Su_bscribe"));
 	gtk_widget_set_sensitive (widget, FALSE);
 	gtk_widget_set_tooltip_text (widget, tooltip);
-	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	editor->priv->subscribe_button = widget;
 	gtk_widget_show (widget);
 
@@ -1258,11 +1784,45 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 		widget, "clicked",
 		G_CALLBACK (subscription_editor_subscribe), editor);
 
+	widget = gtk_button_new ();
+	gtk_button_set_image (
+		GTK_BUTTON (widget),
+		gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE));
+	editor->priv->subscribe_arrow = widget;
+	gtk_widget_show (widget);
+
+	g_signal_connect_swapped (
+		widget, "clicked",
+		G_CALLBACK (subscription_editor_subscribe_popup_cb), editor);
+
+	if (gtk_widget_get_direction (container) == GTK_TEXT_DIR_LTR) {
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->subscribe_button, TRUE, TRUE, 0);
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->subscribe_arrow, FALSE, FALSE, 0);
+	} else {
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->subscribe_arrow, FALSE, FALSE, 0);
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->subscribe_button, TRUE, TRUE, 0);
+	}
+
+	container = box;
+
+	widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, TRUE, 0);
+	gtk_widget_show (widget);
+
+	container = widget;
+
 	tooltip = _("Unsubscribe from the selected folder");
 	widget = gtk_button_new_with_mnemonic (_("_Unsubscribe"));
 	gtk_widget_set_sensitive (widget, FALSE);
 	gtk_widget_set_tooltip_text (widget, tooltip);
-	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	editor->priv->unsubscribe_button = widget;
 	gtk_widget_show (widget);
 
@@ -1270,6 +1830,35 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 		widget, "clicked",
 		G_CALLBACK (subscription_editor_unsubscribe), editor);
 
+	widget = gtk_button_new ();
+	gtk_button_set_image (
+		GTK_BUTTON (widget),
+		gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE));
+	editor->priv->unsubscribe_arrow = widget;
+	gtk_widget_show (widget);
+
+	g_signal_connect_swapped (
+		widget, "clicked",
+		G_CALLBACK (subscription_editor_unsubscribe_popup_cb), editor);
+
+	if (gtk_widget_get_direction (container) == GTK_TEXT_DIR_LTR) {
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->unsubscribe_button, TRUE, TRUE, 0);
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->unsubscribe_arrow, FALSE, FALSE, 0);
+	} else {
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->unsubscribe_arrow, FALSE, FALSE, 0);
+		gtk_box_pack_start (
+			GTK_BOX (container),
+			editor->priv->unsubscribe_button, TRUE, TRUE, 0);
+	}
+
+	container = box;
+
 	tooltip = _("Collapse all folders");
 	widget = gtk_button_new_with_mnemonic (_("C_ollapse All"));
 	gtk_widget_set_tooltip_text (widget, tooltip);
@@ -1319,11 +1908,11 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 
 GtkWidget *
 em_subscription_editor_new (GtkWindow *parent,
-                            CamelSession *session,
+                            EMailSession *session,
                             CamelStore *initial_store)
 {
 	g_return_val_if_fail (GTK_IS_WINDOW (parent), NULL);
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
+	g_return_val_if_fail (E_IS_MAIL_SESSION (session), NULL);
 
 	return g_object_new (
 		EM_TYPE_SUBSCRIPTION_EDITOR,
@@ -1333,7 +1922,7 @@ em_subscription_editor_new (GtkWindow *parent,
 		NULL);
 }
 
-CamelSession *
+EMailSession *
 em_subscription_editor_get_session (EMSubscriptionEditor *editor)
 {
 	g_return_val_if_fail (EM_IS_SUBSCRIPTION_EDITOR (editor), NULL);
