@@ -1665,13 +1665,17 @@ search_address_in_addressbooks (const gchar *address,
 		EBookClient *book_client = NULL;
 		GHook *hook_stop;
 		gboolean cached_book = FALSE;
+		const gchar *display_name;
+		const gchar *uid;
 		GError *err = NULL;
 
+		uid = e_source_peek_uid (source);
+		display_name = e_source_peek_name (source);
+
 		/* failed to load this book last time, skip it now */
-		if (g_hash_table_lookup (emu_broken_books_hash,
-				e_source_peek_uid (source)) != NULL) {
+		if (g_hash_table_lookup (emu_broken_books_hash, uid) != NULL) {
 			d(printf ("%s: skipping broken book '%s'\n",
-				G_STRFUNC, e_source_peek_name (source)));
+				G_STRFUNC, display_name));
 			continue;
 		}
 
@@ -1679,7 +1683,7 @@ search_address_in_addressbooks (const gchar *address,
 
 		hook_stop = mail_cancel_hook_add (emu_addr_cancel_stop, &stop);
 
-		book_client = g_hash_table_lookup (emu_books_hash, e_source_peek_uid (source));
+		book_client = g_hash_table_lookup (emu_books_hash, uid);
 		if (!book_client) {
 			book_client = e_book_client_new (source, &err);
 
@@ -1690,8 +1694,7 @@ search_address_in_addressbooks (const gchar *address,
 				} else if (err) {
 					gchar *source_uid;
 
-					source_uid = g_strdup (
-						e_source_peek_uid (source));
+					source_uid = g_strdup (uid);
 
 					g_hash_table_insert (
 						emu_broken_books_hash,
@@ -1700,7 +1703,7 @@ search_address_in_addressbooks (const gchar *address,
 					g_warning (
 						"%s: Unable to create addressbook '%s': %s",
 						G_STRFUNC,
-						e_source_peek_name (source),
+						display_name,
 						err->message);
 				}
 				g_clear_error (&err);
@@ -1714,8 +1717,7 @@ search_address_in_addressbooks (const gchar *address,
 				} else if (err) {
 					gchar *source_uid;
 
-					source_uid = g_strdup (
-						e_source_peek_uid (source));
+					source_uid = g_strdup (uid);
 
 					g_hash_table_insert (
 						emu_broken_books_hash,
@@ -1724,7 +1726,7 @@ search_address_in_addressbooks (const gchar *address,
 					g_warning (
 						"%s: Unable to open addressbook '%s': %s",
 						G_STRFUNC,
-						e_source_peek_name (source),
+						display_name,
 						err->message);
 				}
 				g_clear_error (&err);
@@ -1760,13 +1762,14 @@ search_address_in_addressbooks (const gchar *address,
 			    (g_error_matches (err, E_CLIENT_ERROR, E_CLIENT_ERROR_CANCELLED) ||
 			     g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED)));
 			if (err && !stop) {
-				gchar *source_uid = g_strdup (e_source_peek_uid (source));
+				gchar *source_uid = g_strdup (uid);
 
 				g_hash_table_insert (emu_broken_books_hash, source_uid, source_uid);
 
 				g_warning (
 					"%s: Can't get contacts from '%s': %s",
-					G_STRFUNC, e_source_peek_name (source),
+					G_STRFUNC,
+					display_name,
 					err->message);
 			}
 			g_clear_error (&err);
@@ -1778,16 +1781,14 @@ search_address_in_addressbooks (const gchar *address,
 			g_object_unref (book_client);
 		} else if (!stop && book_client && !cached_book) {
 			g_hash_table_insert (
-				emu_books_hash, g_strdup (
-				e_source_peek_uid (source)), book_client);
+				emu_books_hash, g_strdup (uid), book_client);
 		}
 	}
 
 	mail_cancel_hook_remove (hook_cancellable);
 	g_object_unref (cancellable);
 
-	g_slist_foreach (addr_sources, (GFunc) g_object_unref, NULL);
-	g_slist_free (addr_sources);
+	g_slist_free_full (addr_sources, (GDestroyNotify) g_object_unref);
 
 	g_free (query);
 
@@ -2101,64 +2102,6 @@ guess_account_from_message (CamelMimeMessage *message)
 	return (uid != NULL) ? e_get_account_by_uid (uid) : NULL;
 }
 
-GHashTable *
-em_utils_generate_account_hash (void)
-{
-	GHashTable *account_hash;
-	EAccount *account, *def;
-	EAccountList *account_list;
-	EIterator *iterator;
-
-	account_list = e_get_account_list ();
-	account_hash = g_hash_table_new (camel_strcase_hash, camel_strcase_equal);
-
-	def = e_get_default_account ();
-
-	iterator = e_list_get_iterator (E_LIST (account_list));
-
-	while (e_iterator_is_valid (iterator)) {
-		account = (EAccount *) e_iterator_get (iterator);
-
-		if (account->id->address) {
-			EAccount *acnt;
-
-			/* Accounts with identical email addresses that are
-			 * enabled take precedence over the accounts that
-			 * aren't. If all accounts with matching email
-			 * addresses are disabled, then the first one in
-			 * the list takes precedence. The default account
-			 * always takes precedence no matter what. */
-			acnt = g_hash_table_lookup (
-				account_hash, account->id->address);
-			if (acnt && acnt != def && !acnt->enabled && account->enabled) {
-				g_hash_table_remove (
-					account_hash, acnt->id->address);
-				acnt = NULL;
-			}
-
-			if (!acnt)
-				g_hash_table_insert (
-					account_hash, (gchar *)
-					account->id->address,
-					(gpointer) account);
-		}
-
-		e_iterator_next (iterator);
-	}
-
-	g_object_unref (iterator);
-
-	/* The default account has to be there if none
-	 * of the enabled accounts are present. */
-	if (g_hash_table_size (account_hash) == 0 && def && def->id->address)
-		g_hash_table_insert (
-			account_hash, (gchar *)
-			def->id->address,
-			(gpointer) def);
-
-	return account_hash;
-}
-
 EAccount *
 em_utils_guess_account (CamelMimeMessage *message,
                         CamelFolder *folder)
@@ -2379,3 +2322,80 @@ em_utils_disconnect_service_sync (CamelService *service,
 	return res;
 }
 
+static gboolean
+check_prefix (const gchar *subject,
+	      const gchar *prefix,
+	      gint *skip_len)
+{
+	gint plen;
+
+	g_return_val_if_fail (subject != NULL, FALSE);
+	g_return_val_if_fail (prefix != NULL, FALSE);
+	g_return_val_if_fail (*prefix, FALSE);
+	g_return_val_if_fail (skip_len != NULL, FALSE);
+
+	plen = strlen (prefix);
+	if (g_ascii_strncasecmp (subject, prefix, plen) != 0)
+		return FALSE;
+
+	if (g_ascii_strncasecmp (subject + plen, ": ", 2) == 0) {
+		*skip_len = plen + 2;
+		return TRUE;
+	}
+
+	if (g_ascii_strncasecmp (subject + plen, " : ", 3) == 0) {
+		*skip_len = plen + 3;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+em_utils_is_re_in_subject (EShell *shell,
+			   const gchar *subject,
+			   gint *skip_len)
+{
+	EShellSettings *shell_settings;
+	gchar *prefixes, **prefixes_strv;
+	gboolean res;
+	gint ii;
+
+	g_return_val_if_fail (shell != NULL, FALSE);
+	g_return_val_if_fail (subject != NULL, FALSE);
+	g_return_val_if_fail (skip_len != NULL, FALSE);
+
+	*skip_len = -1;
+
+	if (strlen (subject) < 3)
+		return FALSE;
+
+	if (check_prefix (subject, "Re", skip_len))
+		return TRUE;
+
+	shell_settings = e_shell_get_shell_settings (shell);
+	prefixes = e_shell_settings_get_string (shell_settings, "composer-localized-re");
+	if (!prefixes || !*prefixes) {
+		g_free (prefixes);
+		return FALSE;
+	}
+
+	prefixes_strv = g_strsplit (prefixes, ",", -1);
+	g_free (prefixes);
+
+	if (!prefixes_strv)
+		return FALSE;
+
+	res = FALSE;
+
+	for (ii = 0; !res && prefixes_strv[ii]; ii++) {
+		const gchar *prefix = prefixes_strv[ii];
+
+		if (*prefix)
+			res = check_prefix (subject, prefix, skip_len);
+	}
+
+	g_strfreev (prefixes_strv);
+
+	return res;
+}
