@@ -176,8 +176,8 @@ import_contact (EBookClient *book_client,
 static void
 import_contacts (void)
 {
-	ESource *primary;
-	ESourceList *source_list;
+	ESource *source;
+	ESourceList *source_list = NULL;
 	EBookClient *book_client;
 	gchar *name;
 	GString *line;
@@ -200,32 +200,36 @@ import_contacts (void)
 	name = g_build_filename(g_get_home_dir(), ".addressbook", NULL);
 	fp = fopen(name, "r");
 	g_free (name);
-	if (fp == NULL)
-		return;
-
-	primary = e_source_list_peek_source_any (source_list);
-	/* FIXME Better error handling */
-	if ((book_client = e_book_client_new (primary, &error)) == NULL) {
-		fclose (fp);
-		g_warning ("Could not create EBook: %s", error->message);
-		if (error)
-			g_error_free (error);
+	if (fp == NULL) {
+		g_object_unref (source_list);
 		return;
 	}
 
-	if (!e_client_open_sync (E_CLIENT (book_client), TRUE, NULL, &error)) {
+	source = e_source_list_peek_source_any (source_list);
+	if (!source) {
 		g_object_unref (source_list);
 		fclose (fp);
 
-		g_warning (
-			"%s: Failed to open book client: %s",
-			G_STRFUNC, error->message);
-		if (error)
-			g_error_free (error);
+		g_warning ("%s: No book source found, skipping.", G_STRFUNC);
 		return;
 	}
+
+	book_client = e_book_client_new (source, &error);
+
+	if (book_client != NULL)
+		e_client_open_sync (E_CLIENT (book_client), TRUE, NULL, &error);
+
 	g_object_unref (source_list);
 
+	if (error != NULL || !book_client) {
+		g_warning (
+			"%s: Failed to open book client: %s",
+			G_STRFUNC, error ? error->message : "Unknown error");
+		g_clear_error (&error);
+		fclose (fp);
+		return;
+	}
+
 	line = g_string_new("");
 	g_string_set_size (line, 256);
 	offset = 0;
