@@ -225,7 +225,7 @@ fetch_mail_exec (struct _fetch_mail_msg *m,
 	fm->destination = e_mail_session_get_local_folder (
 		E_MAIL_SESSION (session), E_MAIL_LOCAL_FOLDER_LOCAL_INBOX);
 	if (fm->destination == NULL)
-		goto fail;
+		goto exit;
 	g_object_ref (fm->destination);
 
 	url = camel_service_new_camel_url (service);
@@ -335,7 +335,7 @@ fetch_mail_exec (struct _fetch_mail_msg *m,
 		fm->source_folder = NULL;
 	}
 
-fail:
+exit:
 	/* we unref this here as it may have more work to do (syncing
 	 * folders and whatnot) before we are really done */
 	/* should this be cancellable too? (i.e. above unregister above) */
@@ -1688,4 +1688,3 @@ mail_disconnect_store (CamelStore *store)
 
 	return id;
 }
-
