@@ -416,8 +416,9 @@ format_url (CamelService *service)
 static send_info_t
 get_receive_type (CamelService *service)
 {
-	CamelProvider *provider;
 	CamelURL *url;
+	CamelProvider *provider;
+	const gchar *uid;
 	gboolean is_local_delivery;
 
 	url = camel_service_new_camel_url (service);
@@ -434,6 +435,13 @@ get_receive_type (CamelService *service)
 	if (provider == NULL)
 		return SEND_INVALID;
 
+	/* skip some well-known services */
+	uid = camel_service_get_uid (service);
+	if (g_strcmp0 (uid, E_MAIL_SESSION_LOCAL_UID) == 0)
+		return SEND_INVALID;
+	if (g_strcmp0 (uid, E_MAIL_SESSION_VFOLDER_UID) == 0)
+		return SEND_INVALID;
+
 	if (provider->object_types[CAMEL_PROVIDER_STORE]) {
 		if (provider->flags & CAMEL_PROVIDER_IS_STORAGE)
 			return SEND_UPDATE;
@@ -1102,7 +1110,7 @@ receive_update_got_store (CamelStore *store,
 	folder_cache = e_mail_session_get_folder_cache (
 		E_MAIL_SESSION (info->session));
 
-	if (store) {
+	if (store != NULL) {
 		mail_folder_cache_note_store (
 			folder_cache, store, info->cancellable,
 			receive_update_got_folderinfo, info);
