@@ -56,6 +56,10 @@
 #include "smime/gui/certificate-manager.h"
 #endif
 
+#define E_BOOK_SHELL_BACKEND_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_BOOK_SHELL_BACKEND, EBookShellBackendPrivate))
+
 struct _EBookShellBackendPrivate {
 	ESourceList *source_list;
 };
@@ -65,8 +69,10 @@ enum {
 	PROP_SOURCE_LIST
 };
 
-static gpointer parent_class;
-static GType book_shell_backend_type;
+G_DEFINE_DYNAMIC_TYPE (
+	EBookShellBackend,
+	e_book_shell_backend,
+	E_TYPE_SHELL_BACKEND)
 
 static void
 book_shell_backend_ensure_sources (EShellBackend *shell_backend)
@@ -84,11 +90,15 @@ book_shell_backend_ensure_sources (EShellBackend *shell_backend)
 	on_this_computer = NULL;
 	personal = NULL;
 
-	priv = E_BOOK_SHELL_BACKEND (shell_backend)->priv;
+	priv = E_BOOK_SHELL_BACKEND_GET_PRIVATE (shell_backend);
 
-	if (!e_book_client_get_sources (&priv->source_list, &error)) {
-		g_warning ("Could not get addressbook sources: %s", error ? error->message : "Unknown error");
-		g_clear_error (&error);
+	e_book_client_get_sources (&priv->source_list, &error);
+
+	if (error != NULL) {
+		g_warning (
+			"Could not get addressbook sources: %s",
+			error->message);
+		g_error_free (error);
 		return;
 	}
 
@@ -123,9 +133,11 @@ book_shell_backend_ensure_sources (EShellBackend *shell_backend)
 		e_source_group_add_source (on_this_computer, source, -1);
 		e_source_set_property (source, "completion", "true");
 		g_object_unref (source);
-	} else {
+		e_source_list_sync (priv->source_list, NULL);		
+	} else if (!e_source_get_property (personal, "name-changed")) {
 		/* Force the source name to the current locale. */
 		e_source_set_name (personal, name);
+		e_source_list_sync (priv->source_list, NULL);		
 	}
 
 	g_object_unref (on_this_computer);
@@ -156,68 +168,82 @@ book_shell_backend_init_importers (void)
 }
 
 static void
-book_shell_backend_new_contact_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
+book_shell_backend_new_contact_cb (GObject *source_object,
+                                   GAsyncResult *result,
+                                   gpointer user_data)
 {
+	ESource *source = E_SOURCE (source_object);
 	EShell *shell = user_data;
 	EClient *client = NULL;
+	EContact *contact;
+	EABEditor *editor;
 	GError *error = NULL;
 
-	if (!e_client_utils_open_new_finish (E_SOURCE (source_object), result, &client, &error))
-		client = NULL;
+	e_client_utils_open_new_finish (source, result, &client, &error);
 
 	/* XXX Handle errors better. */
-	if (client == NULL) {
-		g_debug ("%s: Failed to open book: %s", G_STRFUNC, error ? error->message : "Unknown error");
-		g_clear_error (&error);
-	} else {
-		EBookClient *book_client = E_BOOK_CLIENT (client);
-		EContact *contact;
-		EABEditor *editor;
-
-		contact = e_contact_new ();
-
-		editor = e_contact_editor_new (
-			shell, book_client, contact, TRUE, TRUE);
-
-		eab_editor_show (editor);
-
-		g_object_unref (contact);
-		g_object_unref (book_client);
+	if (error != NULL) {
+		g_warn_if_fail (client == NULL);
+		g_warning (
+			"%s: Failed to open book: %s",
+			G_STRFUNC, error->message);
+		g_error_free (error);
+		goto exit;
 	}
 
+	g_return_if_fail (E_IS_CLIENT (client));
+
+	contact = e_contact_new ();
+
+	editor = e_contact_editor_new (
+		shell, E_BOOK_CLIENT (client), contact, TRUE, TRUE);
+
+	eab_editor_show (editor);
+
+	g_object_unref (contact);
+	g_object_unref (client);
+
+exit:
 	g_object_unref (shell);
 }
 
 static void
-book_shell_backend_new_contact_list_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
+book_shell_backend_new_contact_list_cb (GObject *source_object,
+                                        GAsyncResult *result,
+                                        gpointer user_data)
 {
+	ESource *source = E_SOURCE (source_object);
 	EShell *shell = user_data;
 	EClient *client = NULL;
+	EContact *contact;
+	EABEditor *editor;
 	GError *error = NULL;
 
-	if (!e_client_utils_open_new_finish (E_SOURCE (source_object), result, &client, &error))
-		client = NULL;
+	e_client_utils_open_new_finish (source, result, &client, &error);
 
 	/* XXX Handle errors better. */
-	if (client == NULL) {
-		g_debug ("%s: Failed to open book: %s", G_STRFUNC, error ? error->message : "Unknown error");
-		g_clear_error (&error);
-	} else {
-		EBookClient *book_client = E_BOOK_CLIENT (client);
-		EContact *contact;
-		EABEditor *editor;
-
-		contact = e_contact_new ();
-
-		editor = e_contact_list_editor_new (
-			shell, book_client, contact, TRUE, TRUE);
-
-		eab_editor_show (editor);
-
-		g_object_unref (contact);
-		g_object_unref (book_client);
+	if (error != NULL) {
+		g_warn_if_fail (client == NULL);
+		g_warning (
+			"%s: Failed to open book: %s",
+			G_STRFUNC, error->message);
+		g_error_free (error);
+		goto exit;
 	}
 
+	g_return_if_fail (E_IS_CLIENT (client));
+
+	contact = e_contact_new ();
+
+	editor = e_contact_list_editor_new (
+		shell, E_BOOK_CLIENT (client), contact, TRUE, TRUE);
+
+	eab_editor_show (editor);
+
+	g_object_unref (contact);
+	g_object_unref (client);
+
+exit:
 	g_object_unref (shell);
 }
 
@@ -227,11 +253,10 @@ action_contact_new_cb (GtkAction *action,
 {
 	EShell *shell;
 	EShellBackend *shell_backend;
-	GConfClient *client;
+	GSettings *settings;
 	ESourceList *source_list;
 	ESource *source = NULL;
 	const gchar *action_name;
-	const gchar *key;
 	gchar *uid;
 
 	/* This callback is used for both contacts and contact lists. */
@@ -242,9 +267,9 @@ action_contact_new_cb (GtkAction *action,
 	g_object_get (shell_backend, "source-list", &source_list, NULL);
 	g_return_if_fail (E_IS_SOURCE_LIST (source_list));
 
-	client = e_shell_get_gconf_client (shell);
-	key = "/apps/evolution/addressbook/display/primary_addressbook";
-	uid = gconf_client_get_string (client, key, NULL);
+	settings = g_settings_new ("org.gnome.evolution.addressbook");
+	uid = g_settings_get_string (settings, "primary-addressbook");
+	g_object_unref (settings);
 
 	if (uid != NULL) {
 		source = e_source_list_peek_source_by_uid (source_list, uid);
@@ -422,8 +447,8 @@ book_shell_backend_handle_uri_cb (EShellBackend *shell_backend,
 }
 
 static void
-book_shell_backend_window_created_cb (EShellBackend *shell_backend,
-                                      GtkWindow *window)
+book_shell_backend_window_added_cb (EShellBackend *shell_backend,
+                                    GtkWindow *window)
 {
 	const gchar *backend_name;
 
@@ -464,7 +489,7 @@ book_shell_backend_dispose (GObject *object)
 {
 	EBookShellBackendPrivate *priv;
 
-	priv = E_BOOK_SHELL_BACKEND (object)->priv;
+	priv = E_BOOK_SHELL_BACKEND_GET_PRIVATE (object);
 
 	if (priv->source_list != NULL) {
 		g_object_unref (priv->source_list);
@@ -472,7 +497,7 @@ book_shell_backend_dispose (GObject *object)
 	}
 
 	/* Chain up to parent's dispose() method. */
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (e_book_shell_backend_parent_class)->dispose (object);
 }
 
 static void
@@ -507,8 +532,8 @@ book_shell_backend_constructed (GObject *object)
 		shell_backend);
 
 	g_signal_connect_swapped (
-		shell, "window-created",
-		G_CALLBACK (book_shell_backend_window_created_cb),
+		shell, "window-added",
+		G_CALLBACK (book_shell_backend_window_added_cb),
 		shell_backend);
 
 	e_book_shell_backend_init_settings (shell);
@@ -518,16 +543,15 @@ book_shell_backend_constructed (GObject *object)
 	g_idle_add ((GSourceFunc) book_shell_backend_init_preferences, shell);
 
 	/* Chain up to parent's constructed() method. */
-	G_OBJECT_CLASS (parent_class)->constructed (object);
+	G_OBJECT_CLASS (e_book_shell_backend_parent_class)->constructed (object);
 }
 
 static void
-book_shell_backend_class_init (EBookShellBackendClass *class)
+e_book_shell_backend_class_init (EBookShellBackendClass *class)
 {
 	GObjectClass *object_class;
 	EShellBackendClass *shell_backend_class;
 
-	parent_class = g_type_class_peek_parent (class);
 	g_type_class_add_private (class, sizeof (EBookShellBackendPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
@@ -557,38 +581,24 @@ book_shell_backend_class_init (EBookShellBackendClass *class)
 }
 
 static void
-book_shell_backend_init (EBookShellBackend *book_shell_backend)
+e_book_shell_backend_class_finalize (EBookShellBackendClass *class)
 {
-	book_shell_backend->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		book_shell_backend, E_TYPE_BOOK_SHELL_BACKEND,
-		EBookShellBackendPrivate);
 }
 
-GType
-e_book_shell_backend_get_type (void)
+static void
+e_book_shell_backend_init (EBookShellBackend *book_shell_backend)
 {
-	return book_shell_backend_type;
+	book_shell_backend->priv =
+		E_BOOK_SHELL_BACKEND_GET_PRIVATE (book_shell_backend);
 }
 
 void
-e_book_shell_backend_register_type (GTypeModule *type_module)
+e_book_shell_backend_type_register (GTypeModule *type_module)
 {
-	const GTypeInfo type_info = {
-		sizeof (EBookShellBackendClass),
-		(GBaseInitFunc) NULL,
-		(GBaseFinalizeFunc) NULL,
-		(GClassInitFunc) book_shell_backend_class_init,
-		(GClassFinalizeFunc) NULL,
-		NULL,  /* class_data */
-		sizeof (EBookShellBackend),
-		0,     /* n_preallocs */
-		(GInstanceInitFunc) book_shell_backend_init,
-		NULL   /* value_table */
-	};
-
-	book_shell_backend_type = g_type_module_register_type (
-		type_module, E_TYPE_SHELL_BACKEND,
-		"EBookShellBackend", &type_info, 0);
+	/* XXX G_DEFINE_DYNAMIC_TYPE declares a static type registration
+	 *     function, so we have to wrap it with a public function in
+	 *     order to register types from a separate compilation unit. */
+	e_book_shell_backend_register_type (type_module);
 }
 
 ESourceList *
