@@ -55,6 +55,10 @@
 
 #include "e-calendar-preferences.h"
 
+#define E_CAL_SHELL_BACKEND_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_CAL_SHELL_BACKEND, ECalShellBackendPrivate))
+
 struct _ECalShellBackendPrivate {
 	ESourceList *source_list;
 };
@@ -64,8 +68,10 @@ enum {
 	PROP_SOURCE_LIST
 };
 
-static gpointer parent_class;
-static GType cal_shell_backend_type;
+G_DEFINE_DYNAMIC_TYPE (
+	ECalShellBackend,
+	e_cal_shell_backend,
+	E_TYPE_SHELL_BACKEND)
 
 static void
 cal_shell_backend_ensure_sources (EShellBackend *shell_backend)
@@ -169,7 +175,7 @@ cal_shell_backend_ensure_sources (EShellBackend *shell_backend)
 		g_slist_foreach (selected, (GFunc) g_free, NULL);
 		g_slist_free (selected);
 		g_free (primary);
-	} else {
+	} else if (!e_source_get_property (personal, "name-changed")) {
 		/* Force the source name to the current locale. */
 		e_source_set_name (personal, name);
 	}
@@ -215,7 +221,7 @@ cal_shell_backend_ensure_sources (EShellBackend *shell_backend)
 
 		/* This is now a borrowed reference. */
 		birthdays = source;
-	} else {
+	} else if (!e_source_get_property (birthdays, "name-changed")) {
 		/* Force the source name to the current locale. */
 		e_source_set_name (birthdays, name);
 	}
@@ -699,8 +705,8 @@ exit:
 }
 
 static void
-cal_shell_backend_window_created_cb (EShellBackend *shell_backend,
-                                     GtkWindow *window)
+cal_shell_backend_window_added_cb (EShellBackend *shell_backend,
+                                   GtkWindow *window)
 {
 	const gchar *backend_name;
 
@@ -741,7 +747,7 @@ cal_shell_backend_dispose (GObject *object)
 {
 	ECalShellBackendPrivate *priv;
 
-	priv = E_CAL_SHELL_BACKEND (object)->priv;
+	priv = E_CAL_SHELL_BACKEND_GET_PRIVATE (object);
 
 	if (priv->source_list != NULL) {
 		g_object_unref (priv->source_list);
@@ -749,7 +755,7 @@ cal_shell_backend_dispose (GObject *object)
 	}
 
 	/* Chain up to parent's dispose() method. */
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (e_cal_shell_backend_parent_class)->dispose (object);
 }
 
 static void
@@ -770,8 +776,8 @@ cal_shell_backend_constructed (GObject *object)
 		shell_backend);
 
 	g_signal_connect_swapped (
-		shell, "window-created",
-		G_CALLBACK (cal_shell_backend_window_created_cb),
+		shell, "window-added",
+		G_CALLBACK (cal_shell_backend_window_added_cb),
 		shell_backend);
 
 	cal_shell_backend_init_importers ();
@@ -790,16 +796,15 @@ cal_shell_backend_constructed (GObject *object)
 		600);
 
 	/* Chain up to parent's constructed() method. */
-	G_OBJECT_CLASS (parent_class)->constructed (object);
+	G_OBJECT_CLASS (e_cal_shell_backend_parent_class)->constructed (object);
 }
 
 static void
-cal_shell_backend_class_init (ECalShellBackendClass *class)
+e_cal_shell_backend_class_init (ECalShellBackendClass *class)
 {
 	GObjectClass *object_class;
 	EShellBackendClass *shell_backend_class;
 
-	parent_class = g_type_class_peek_parent (class);
 	g_type_class_add_private (class, sizeof (ECalShellBackendPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
@@ -829,14 +834,18 @@ cal_shell_backend_class_init (ECalShellBackendClass *class)
 }
 
 static void
-cal_shell_backend_init (ECalShellBackend *cal_shell_backend)
+e_cal_shell_backend_class_finalize (ECalShellBackendClass *class)
+{
+}
+
+static void
+e_cal_shell_backend_init (ECalShellBackend *cal_shell_backend)
 {
 	icalarray *builtin_timezones;
 	gint ii;
 
-	cal_shell_backend->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		cal_shell_backend, E_TYPE_CAL_SHELL_BACKEND,
-		ECalShellBackendPrivate);
+	cal_shell_backend->priv =
+		E_CAL_SHELL_BACKEND_GET_PRIVATE (cal_shell_backend);
 
 	/* XXX Pre-load all built-in timezones in libical.
 	 *
@@ -862,31 +871,13 @@ cal_shell_backend_init (ECalShellBackend *cal_shell_backend)
 	}
 }
 
-GType
-e_cal_shell_backend_get_type (void)
-{
-	return cal_shell_backend_type;
-}
-
 void
-e_cal_shell_backend_register_type (GTypeModule *type_module)
+e_cal_shell_backend_type_register (GTypeModule *type_module)
 {
-	const GTypeInfo type_info = {
-		sizeof (ECalShellBackendClass),
-		(GBaseInitFunc) NULL,
-		(GBaseFinalizeFunc) NULL,
-		(GClassInitFunc) cal_shell_backend_class_init,
-		(GClassFinalizeFunc) NULL,
-		NULL,  /* class_data */
-		sizeof (ECalShellBackend),
-		0,     /* n_preallocs */
-		(GInstanceInitFunc) cal_shell_backend_init,
-		NULL   /* value_table */
-	};
-
-	cal_shell_backend_type = g_type_module_register_type (
-		type_module, E_TYPE_SHELL_BACKEND,
-		"ECalShellBackend", &type_info, 0);
+	/* XXX G_DEFINE_DYNAMIC_TYPE declares a static type registration
+	 *     function, so we have to wrap it with a public function in
+	 *     order to register types from a separate compilation unit. */
+	e_cal_shell_backend_register_type (type_module);
 }
 
 ESourceList *
@@ -901,18 +892,25 @@ e_cal_shell_backend_get_source_list (ECalShellBackend *cal_shell_backend)
 GSList *
 e_cal_shell_backend_get_selected_calendars (ECalShellBackend *cal_shell_backend)
 {
-	GConfClient *client;
-	GSList *selected_calendars;
-	const gchar *key;
+	GSettings *settings;
+	GSList *selected_calendars = NULL;
+	gchar **strv;
+	gint ii;
 
 	g_return_val_if_fail (
 		E_IS_CAL_SHELL_BACKEND (cal_shell_backend), NULL);
 
-	client = gconf_client_get_default ();
-	key = "/apps/evolution/calendar/display/selected_calendars";
-	selected_calendars = gconf_client_get_list (
-		client, key, GCONF_VALUE_STRING, NULL);
-	g_object_unref (client);
+	settings = g_settings_new ("org.gnome.evolution.calendar");
+	strv = g_settings_get_strv (settings, "selected-calendars");
+	g_object_unref (settings);
+
+	if (strv != NULL) {
+		for (ii = 0; strv[ii] != NULL; ii++)
+			selected_calendars = g_slist_append (
+				selected_calendars, g_strdup (strv[ii]));
+
+		g_strfreev (strv);
+	}
 
 	return selected_calendars;
 }
@@ -921,16 +919,23 @@ void
 e_cal_shell_backend_set_selected_calendars (ECalShellBackend *cal_shell_backend,
                                             GSList *selected_calendars)
 {
-	GConfClient *client;
-	const gchar *key;
+	GSettings *settings;
+	GSList *link;
+	GPtrArray *array = g_ptr_array_new ();
 
 	g_return_if_fail (E_IS_CAL_SHELL_BACKEND (cal_shell_backend));
 
-	client = gconf_client_get_default ();
-	key = "/apps/evolution/calendar/display/selected_calendars";
-	gconf_client_set_list (
-		client, key, GCONF_VALUE_STRING, selected_calendars, NULL);
-	g_object_unref (client);
+	for (link = selected_calendars; link != NULL; link = link->next)
+		g_ptr_array_add (array, link->data);
+	g_ptr_array_add (array, NULL);
+
+	settings = g_settings_new ("org.gnome.evolution.calendar");
+	g_settings_set_strv (
+		settings, "selected-calendars",
+		(const gchar *const *) array->pdata);
+	g_object_unref (settings);
+
+	g_ptr_array_free (array, FALSE);
 }
 
 void
@@ -943,18 +948,21 @@ e_cal_shell_backend_open_date_range (ECalShellBackend *cal_shell_backend,
 	EShellBackend *shell_backend;
 	EShellSidebar *shell_sidebar;
 	GtkWidget *shell_window = NULL;
+	GtkApplication *application;
 	ECalendar *navigator;
-	GList *watched_windows;
+	GList *list;
 
 	g_return_if_fail (E_IS_CAL_SHELL_BACKEND (cal_shell_backend));
 
 	shell_backend = E_SHELL_BACKEND (cal_shell_backend);
 	shell = e_shell_backend_get_shell (shell_backend);
-	watched_windows = e_shell_get_watched_windows (shell);
+
+	application = GTK_APPLICATION (shell);
+	list = gtk_application_get_windows (application);
 
 	/* Try to find an EShellWindow already in calendar view. */
-	while (watched_windows != NULL) {
-		GtkWidget *window = GTK_WIDGET (watched_windows->data);
+	while (list != NULL) {
+		GtkWidget *window = GTK_WIDGET (list->data);
 
 		if (E_IS_SHELL_WINDOW (window)) {
 			const gchar *active_view;
@@ -968,7 +976,7 @@ e_cal_shell_backend_open_date_range (ECalShellBackend *cal_shell_backend,
 			}
 		}
 
-		watched_windows = g_list_next (watched_windows);
+		list = g_list_next (list);
 	}
 
 	/* Otherwise create a new EShellWindow in calendar view. */
