@@ -191,7 +191,8 @@ update_day_second_zone_caption (ECalendarPreferences *prefs)
 }
 
 static void
-on_set_day_second_zone (GtkWidget *item, ECalendarPreferences *prefs)
+on_set_day_second_zone (GtkWidget *item,
+                        ECalendarPreferences *prefs)
 {
 	if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item)))
 		return;
@@ -201,7 +202,8 @@ on_set_day_second_zone (GtkWidget *item, ECalendarPreferences *prefs)
 }
 
 static void
-on_select_day_second_zone (GtkWidget *item, ECalendarPreferences *prefs)
+on_select_day_second_zone (GtkWidget *item,
+                           ECalendarPreferences *prefs)
 {
 	g_return_if_fail (prefs != NULL);
 
@@ -210,7 +212,8 @@ on_select_day_second_zone (GtkWidget *item, ECalendarPreferences *prefs)
 }
 
 static void
-day_second_zone_clicked (GtkWidget *widget, ECalendarPreferences *prefs)
+day_second_zone_clicked (GtkWidget *widget,
+                         ECalendarPreferences *prefs)
 {
 	GtkWidget *menu, *item;
 	GSList *group = NULL, *recent_zones, *s;
@@ -263,7 +266,8 @@ day_second_zone_clicked (GtkWidget *widget, ECalendarPreferences *prefs)
 }
 
 static void
-start_of_day_changed (GtkWidget *widget, ECalendarPreferences *prefs)
+start_of_day_changed (GtkWidget *widget,
+                      ECalendarPreferences *prefs)
 {
 	gint start_hour, start_minute, end_hour, end_minute;
 	EDateEdit *start, *end;
@@ -292,7 +296,8 @@ start_of_day_changed (GtkWidget *widget, ECalendarPreferences *prefs)
 }
 
 static void
-end_of_day_changed (GtkWidget *widget, ECalendarPreferences *prefs)
+end_of_day_changed (GtkWidget *widget,
+                    ECalendarPreferences *prefs)
 {
 	gint start_hour, start_minute, end_hour, end_minute;
 	EDateEdit *start, *end;
@@ -321,19 +326,21 @@ end_of_day_changed (GtkWidget *widget, ECalendarPreferences *prefs)
 }
 
 static void
-notify_with_tray_toggled (GtkToggleButton *toggle, ECalendarPreferences *prefs)
+notify_with_tray_toggled (GtkToggleButton *toggle,
+                          ECalendarPreferences *prefs)
 {
-	GConfClient *gconf;
+	GSettings *settings;
 
 	g_return_if_fail (toggle != NULL);
 
-	gconf = gconf_client_get_default ();
-	gconf_client_set_bool (gconf, "/apps/evolution/calendar/notify/notify_with_tray", gtk_toggle_button_get_active (toggle), NULL);
-	g_object_unref (gconf);
+	settings = g_settings_new ("org.gnome.evolution.calendar");
+	g_settings_set_boolean (settings, "notify-with-tray", gtk_toggle_button_get_active (toggle));
+	g_object_unref (settings);
 }
 
 static void
-alarms_selection_changed (ESourceSelector *selector, ECalendarPreferences *prefs)
+alarms_selection_changed (ESourceSelector *selector,
+                          ECalendarPreferences *prefs)
 {
 	ESourceList *source_list = prefs->alarms_list;
 	GSList *selection;
@@ -358,7 +365,7 @@ alarms_selection_changed (ESourceSelector *selector, ECalendarPreferences *prefs
 	}
 
 	/* then we loop over the selector's selection, setting the
-	   property on those sources */
+	 * property on those sources */
 	selection = e_source_selector_get_selection (selector);
 	for (l = selection; l; l = l->next) {
 		source = E_SOURCE (l->data);
@@ -402,17 +409,30 @@ update_system_tz_widgets (EShellSettings *shell_settings,
 static void
 setup_changes (ECalendarPreferences *prefs)
 {
-	g_signal_connect (G_OBJECT (prefs->day_second_zone), "clicked", G_CALLBACK (day_second_zone_clicked), prefs);
+	g_signal_connect (
+		prefs->day_second_zone, "clicked",
+		G_CALLBACK (day_second_zone_clicked), prefs);
 
-	g_signal_connect (G_OBJECT (prefs->start_of_day), "changed", G_CALLBACK (start_of_day_changed), prefs);
-	g_signal_connect (G_OBJECT (prefs->end_of_day), "changed", G_CALLBACK (end_of_day_changed), prefs);
+	g_signal_connect (
+		prefs->start_of_day, "changed",
+		G_CALLBACK (start_of_day_changed), prefs);
 
-	g_signal_connect (G_OBJECT (prefs->notify_with_tray), "toggled", G_CALLBACK (notify_with_tray_toggled), prefs);
-	g_signal_connect (G_OBJECT (prefs->alarm_list_widget), "selection_changed", G_CALLBACK (alarms_selection_changed), prefs);
+	g_signal_connect (
+		prefs->end_of_day, "changed",
+		G_CALLBACK (end_of_day_changed), prefs);
+
+	g_signal_connect (
+		prefs->notify_with_tray, "toggled",
+		G_CALLBACK (notify_with_tray_toggled), prefs);
+
+	g_signal_connect (
+		prefs->alarm_list_widget, "selection_changed",
+		G_CALLBACK (alarms_selection_changed), prefs);
 }
 
 static void
-initialize_selection (ESourceSelector *selector, ESourceList *source_list)
+initialize_selection (ESourceSelector *selector,
+                      ESourceList *source_list)
 {
 	GSList *groups;
 
@@ -434,7 +454,7 @@ initialize_selection (ESourceSelector *selector, ESourceList *source_list)
 static void
 show_alarms_config (ECalendarPreferences *prefs)
 {
-	GConfClient *gconf;
+	GSettings *settings;
 
 	if (e_cal_client_get_sources (&prefs->alarms_list, E_CAL_CLIENT_SOURCE_TYPE_EVENTS, NULL)) {
 		prefs->alarm_list_widget = e_source_selector_new (prefs->alarms_list);
@@ -444,9 +464,9 @@ show_alarms_config (ECalendarPreferences *prefs)
 		initialize_selection (E_SOURCE_SELECTOR (prefs->alarm_list_widget), prefs->alarms_list);
 	}
 
-	gconf = gconf_client_get_default ();
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->notify_with_tray), gconf_client_get_bool (gconf, "/apps/evolution/calendar/notify/notify_with_tray", NULL));
-	g_object_unref (gconf);
+	settings = g_settings_new ("org.gnome.evolution.calendar");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->notify_with_tray), g_settings_get_boolean (settings, "notify-with-tray"));
+	g_object_unref (settings);
 }
 
 /* Shows the current config settings in the dialog. */
@@ -490,13 +510,16 @@ static ECalConfigItem eccp_items[] = {
 	{ E_CONFIG_PAGE,          (gchar *) "10.display",                   (gchar *) "display",           calendar_preferences_get_config_widget },
 	{ E_CONFIG_SECTION,       (gchar *) "10.display/00.general",        (gchar *) "displayGeneral",    calendar_preferences_get_config_widget },
 	{ E_CONFIG_SECTION,       (gchar *) "10.display/10.taskList",       (gchar *) "taskList",          calendar_preferences_get_config_widget },
+	{ E_CONFIG_PAGE,          (gchar *) "12.tasks",                     (gchar *) "tasks-vbox",        calendar_preferences_get_config_widget },
 	{ E_CONFIG_PAGE,          (gchar *) "15.alarms",                    (gchar *) "alarms",            calendar_preferences_get_config_widget },
 	{ E_CONFIG_PAGE,          (gchar *) "20.freeBusy",                  (gchar *) "freebusy",          calendar_preferences_get_config_widget },
 	{ E_CONFIG_SECTION,       (gchar *) "20.freeBusy/00.defaultServer", (gchar *) "default-freebusy-vbox",   calendar_preferences_get_config_widget },
 };
 
 static void
-eccp_free (EConfig *ec, GSList *items, gpointer data)
+eccp_free (EConfig *ec,
+           GSList *items,
+           gpointer data)
 {
 	g_slist_free (items);
 }
@@ -774,6 +797,13 @@ calendar_preferences_construct (ECalendarPreferences *prefs,
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
+	widget = e_builder_get_widget (prefs->builder, "tasks_due_today_highlight");
+	g_object_bind_property (
+		shell_settings, "cal-tasks-highlight-due-today",
+		widget, "active",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
 	widget = e_builder_get_widget (prefs->builder, "tasks_due_today_color");
 	g_object_bind_property_full (
 		shell_settings, "cal-tasks-color-due-today",
@@ -783,6 +813,17 @@ calendar_preferences_construct (ECalendarPreferences *prefs,
 		e_binding_transform_string_to_color,
 		e_binding_transform_color_to_string,
 		NULL, (GDestroyNotify) NULL);
+	g_object_bind_property (
+		shell_settings, "cal-tasks-highlight-due-today",
+		widget, "sensitive",
+		G_BINDING_SYNC_CREATE);
+
+	widget = e_builder_get_widget (prefs->builder, "tasks_overdue_highlight");
+	g_object_bind_property (
+		shell_settings, "cal-tasks-highlight-overdue",
+		widget, "active",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
 
 	widget = e_builder_get_widget (prefs->builder, "tasks_overdue_color");
 	g_object_bind_property_full (
@@ -793,6 +834,10 @@ calendar_preferences_construct (ECalendarPreferences *prefs,
 		e_binding_transform_string_to_color,
 		e_binding_transform_color_to_string,
 		(GDestroyNotify) NULL, NULL);
+	g_object_bind_property (
+		shell_settings, "cal-tasks-highlight-overdue",
+		widget, "sensitive",
+		G_BINDING_SYNC_CREATE);
 
 	widget = e_builder_get_widget (prefs->builder, "tasks_hide_completed");
 	g_object_bind_property (
