@@ -590,6 +590,24 @@ mail_shell_view_notify_view_id_cb (EMailShellView *mail_shell_view)
 	gal_view_instance_set_current_view_id (view_instance, view_id);
 }
 
+static void
+mail_shell_view_search_filter_changed_cb (EMailShellView *mail_shell_view)
+{
+	EMailShellContent *mail_shell_content;
+	EMailView *mail_view;
+
+	g_return_if_fail (mail_shell_view != NULL);
+	g_return_if_fail (mail_shell_view->priv != NULL);
+
+	if (e_shell_view_is_execute_search_blocked (E_SHELL_VIEW (mail_shell_view)))
+		return;
+
+	mail_shell_content = mail_shell_view->priv->mail_shell_content;
+	mail_view = e_mail_shell_content_get_mail_view (mail_shell_content);
+
+	e_mail_reader_avoid_next_mark_as_seen (E_MAIL_READER (mail_view));
+}
+
 void
 e_mail_shell_view_private_init (EMailShellView *mail_shell_view,
                                 EShellViewClass *shell_view_class)
@@ -612,7 +630,6 @@ e_mail_shell_view_private_constructed (EMailShellView *mail_shell_view)
 	EShellView *shell_view;
 	EShellBackend *shell_backend;
 	EShellContent *shell_content;
-	EShellSettings *shell_settings;
 	EShellSidebar *shell_sidebar;
 	EShellTaskbar *shell_taskbar;
 	EShellWindow *shell_window;
@@ -623,9 +640,11 @@ e_mail_shell_view_private_constructed (EMailShellView *mail_shell_view)
 	ERuleContext *context;
 	EFilterRule *rule = NULL;
 	GtkTreeSelection *selection;
-	GtkTreeModel *tree_model;
 	GtkUIManager *ui_manager;
 	GtkWidget *message_list;
+	EMailLabelListStore *label_store;
+	EMailBackend *backend;
+	EMailSession *session;
 	EMailReader *reader;
 	EMailView *mail_view;
 	EWebView *web_view;
@@ -643,10 +662,10 @@ e_mail_shell_view_private_constructed (EMailShellView *mail_shell_view)
 	ui_manager = e_shell_window_get_ui_manager (shell_window);
 
 	shell = e_shell_window_get_shell (shell_window);
-	shell_settings = e_shell_get_shell_settings (shell);
 
-	tree_model = e_shell_settings_get_object (
-		shell_settings, "mail-label-list-store");
+	backend = E_MAIL_BACKEND (shell_backend);
+	session = e_mail_backend_get_session (backend);
+	label_store = e_mail_session_get_label_store (session);
 
 	e_shell_window_add_action_group (shell_window, "mail");
 	e_shell_window_add_action_group (shell_window, "mail-filter");
@@ -684,6 +703,12 @@ e_mail_shell_view_private_constructed (EMailShellView *mail_shell_view)
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
+	combo_box = e_shell_searchbar_get_filter_combo_box (searchbar);
+	g_signal_connect_object (
+		combo_box, "changed",
+		G_CALLBACK (mail_shell_view_search_filter_changed_cb),
+		mail_shell_view, G_CONNECT_SWAPPED);
+
 	web_view = em_format_html_get_web_view (formatter);
 
 	g_signal_connect_object (
@@ -743,17 +768,17 @@ e_mail_shell_view_private_constructed (EMailShellView *mail_shell_view)
 		mail_shell_view, G_CONNECT_SWAPPED);
 
 	g_signal_connect_object (
-		tree_model, "row-changed",
+		label_store, "row-changed",
 		G_CALLBACK (e_mail_shell_view_update_search_filter),
 		mail_shell_view, G_CONNECT_SWAPPED);
 
 	g_signal_connect_object (
-		tree_model, "row-deleted",
+		label_store, "row-deleted",
 		G_CALLBACK (e_mail_shell_view_update_search_filter),
 		mail_shell_view, G_CONNECT_SWAPPED);
 
 	g_signal_connect_object (
-		tree_model, "row-inserted",
+		label_store, "row-inserted",
 		G_CALLBACK (e_mail_shell_view_update_search_filter),
 		mail_shell_view, G_CONNECT_SWAPPED);
 
