@@ -21,15 +21,14 @@
 #endif
 
 #include <glib/gi18n-lib.h>
+#include <libebackend/e-extension.h>
 
 #include <shell/e-shell.h>
 #include <e-util/e-account-utils.h>
 #include <e-util/e-alert-dialog.h>
-#include <e-util/e-extension.h>
 #include <e-util/e-import.h>
 
 #include <mail/e-mail-backend.h>
-#include <mail/e-mail-local.h>
 #include <mail/em-account-editor.h>
 #include <capplet/settings/mail-capplet-shell.h>
 #include <calendar/gui/calendar-config.h>
@@ -82,7 +81,8 @@ GType e_startup_wizard_get_type (void);
 G_DEFINE_DYNAMIC_TYPE (EStartupWizard, e_startup_wizard, E_TYPE_EXTENSION)
 
 G_GNUC_NORETURN static void
-startup_wizard_terminate (void) {
+startup_wizard_terminate (void)
+{
 	gtk_main_quit ();
 	_exit (0);
 }
@@ -167,7 +167,6 @@ completed:
 
 static void
 startup_wizard_config_abort (EConfig *config,
-                             GSList *items,
                              EStartupWizard *extension)
 {
 	GtkAssistant *assistant;
@@ -209,7 +208,6 @@ startup_wizard_config_abort (EConfig *config,
 
 static void
 startup_wizard_config_commit (EConfig *config,
-                              GSList *items,
                               EStartupWizard *extension)
 {
 	EShell *shell;
@@ -397,7 +395,7 @@ startup_wizard_progress_page (EConfig *config,
 	gtk_widget_show (widget);
 
 	widget = gtk_label_new (
-		_("Import cancelled. Click \"Forward\" to continue."));
+		_("Import cancelled."));
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 
@@ -415,7 +413,7 @@ startup_wizard_progress_page (EConfig *config,
 	gtk_widget_show (widget);
 
 	widget = gtk_label_new (
-		_("Import complete. Click \"Forward\" to continue."));
+		_("Import complete."));
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 
@@ -434,22 +432,16 @@ startup_wizard_new_assistant (EStartupWizard *extension)
 	EShell *shell;
 	EShellBackend *shell_backend;
 	EMailBackend *backend;
-	EMailSession *session;
 	EMAccountEditor *emae;
 	EConfig *config;
 	EConfigItem *config_item;
 	GtkWidget *widget;
 	GSList *items = NULL;
-	const gchar *data_dir;
 
 	shell = startup_wizard_get_shell (extension);
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
 
 	backend = E_MAIL_BACKEND (shell_backend);
-	session = e_mail_backend_get_session (backend);
-	data_dir = e_shell_backend_get_data_dir (shell_backend);
-
-	e_mail_local_init (session, data_dir);
 
 	emae = em_account_editor_new (
 		NULL, EMAE_ASSISTANT, backend,
@@ -480,8 +472,6 @@ startup_wizard_new_assistant (EStartupWizard *extension)
 
 	e_config_add_items (
 		config, items,
-		(EConfigItemsFunc) startup_wizard_config_commit,
-		(EConfigItemsFunc) startup_wizard_config_abort,
 		(EConfigItemsFunc) startup_wizard_config_free,
 		g_object_ref (extension));
 
@@ -489,6 +479,14 @@ startup_wizard_new_assistant (EStartupWizard *extension)
 		config, "70.progress", (EConfigCheckFunc)
 		startup_wizard_check_progress, extension);
 
+	g_signal_connect (
+		config, "abort",
+		G_CALLBACK (startup_wizard_config_abort), extension);
+
+	g_signal_connect (
+		config, "commit",
+		G_CALLBACK (startup_wizard_config_commit), extension);
+
 	e_config_create_window (config, NULL, _("Evolution Setup Assistant"));
 
 	/* Additional tweaks.  The window must be created at this point. */
@@ -502,8 +500,7 @@ startup_wizard_new_assistant (EStartupWizard *extension)
 		GTK_LABEL (widget),
 		_("Welcome to Evolution. The next few screens will "
 		  "allow Evolution to connect to your email accounts, "
-		  "and to import files from other applications. \n\n"
-		  "Please click the \"Forward\" button to continue. "));
+		  "and to import files from other applications."));
 
 	/* Finalize the EMAccountEditor along with the GtkAssistant. */
 	g_object_set_data_full (
