@@ -22,8 +22,6 @@
 #ifndef E_SHELL_H
 #define E_SHELL_H
 
-#include <gconf/gconf-client.h>
-
 #include <e-util/e-activity.h>
 #include <e-util/e-alert.h>
 
@@ -106,9 +104,6 @@ struct _EShellClass {
 						 EActivity *activity);
 	void		(*quit_requested)	(EShell *shell,
 						 EShellQuitReason reason);
-	void		(*window_created)	(EShell *shell,
-						 GtkWindow *window);
-	void		(*window_destroyed)	(EShell *shell);
 };
 
 GType		e_shell_get_type		(void);
@@ -122,7 +117,6 @@ EShellBackend *	e_shell_get_backend_by_name	(EShell *shell,
 EShellBackend *	e_shell_get_backend_by_scheme	(EShell *shell,
 						 const gchar *scheme);
 EShellSettings *e_shell_get_shell_settings	(EShell *shell);
-GConfClient *	e_shell_get_gconf_client	(EShell *shell);
 GtkWidget *	e_shell_create_shell_window	(EShell *shell,
 						 const gchar *view_name);
 guint		e_shell_handle_uris		(EShell *shell,
@@ -130,9 +124,6 @@ guint		e_shell_handle_uris		(EShell *shell,
 						 gboolean do_import);
 void		e_shell_submit_alert		(EShell *shell,
 						 EAlert *alert);
-void		e_shell_watch_window		(EShell *shell,
-						 GtkWindow *window);
-GList *		e_shell_get_watched_windows	(EShell *shell);
 GtkWindow *     e_shell_get_active_window	(EShell *shell);
 gboolean	e_shell_get_meego_mode		(EShell *shell);
 gboolean	e_shell_get_express_mode	(EShell *shell);
